@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-	Copyright (C) 2010  Belledonne Communications SARL
-
-	This program is free software: you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation, either version 2 of the License, or
-	(at your option) any later version.
-
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
-
-	You should have received a copy of the GNU General Public License
-	along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef belle_sip_object_h
 #define belle_sip_object_h
@@ -84,19 +85,27 @@ typedef unsigned int belle_sip_type_id_t;
 		(belle_sip_object_get_vptr_t)BELLE_SIP_OBJECT_GET_VPTR_FUNC(parent_type), \
 		(belle_sip_interface_desc_t**)object_type##interfaces_table
 
+#define BELLE_SIP_INSTANCIATE_VPTR3(object_type,parent_type,destroy,clone,marshal,on_first_ref,on_last_ref,unowned,is_cpp) \
+static BELLE_SIP_OBJECT_VPTR_TYPE(object_type) BELLE_SIP_OBJECT_VPTR_NAME(object_type)={ \
+	BELLE_SIP_VPTR_INIT(object_type,parent_type,unowned), \
+	(belle_sip_object_destroy_t)destroy,	\
+	(belle_sip_object_clone_t)clone,	\
+	(belle_sip_object_marshal_t)marshal,\
+	(belle_sip_object_on_first_ref_t)on_first_ref,\
+	(belle_sip_object_on_last_ref_t)on_last_ref,\
+	BELLE_SIP_DEFAULT_BUFSIZE_HINT,\
+	is_cpp, 0\
+	}; \
+	BELLE_SIP_OBJECT_VPTR_TYPE(object_type) * BELLE_SIP_OBJECT_GET_VPTR_FUNC(object_type)(void){\
+		return &BELLE_SIP_OBJECT_VPTR_NAME(object_type); \
+	}
+
+#define BELLE_SIP_INSTANCIATE_VPTR2(object_type,parent_type,destroy,clone,marshal,on_first_ref,on_last_ref,unowned) \
+	BELLE_SIP_INSTANCIATE_VPTR3(object_type,parent_type,destroy,clone,marshal,on_first_ref,on_last_ref,unowned,FALSE)
+
 
 #define BELLE_SIP_INSTANCIATE_VPTR(object_type,parent_type,destroy,clone,marshal,unowned) \
-		static BELLE_SIP_OBJECT_VPTR_TYPE(object_type) BELLE_SIP_OBJECT_VPTR_NAME(object_type)={ \
-		BELLE_SIP_VPTR_INIT(object_type,parent_type,unowned), \
-		(belle_sip_object_destroy_t)destroy,	\
-		(belle_sip_object_clone_t)clone,	\
-		(belle_sip_object_marshal_t)marshal,\
-		BELLE_SIP_DEFAULT_BUFSIZE_HINT\
-		}; \
-		BELLE_SIP_OBJECT_VPTR_TYPE(object_type) * BELLE_SIP_OBJECT_GET_VPTR_FUNC(object_type)(void){\
-			return &BELLE_SIP_OBJECT_VPTR_NAME(object_type); \
-		}
-
+		BELLE_SIP_INSTANCIATE_VPTR2(object_type, parent_type, destroy, clone, marshal, NULL, NULL, unowned)
 /**
  * belle_sip_object_t is the base object.
  * It is the base class for all belle sip non trivial objects.
@@ -106,8 +115,8 @@ typedef unsigned int belle_sip_type_id_t;
  * It contains a generic data store that allows users to store named data in it and retrieve them afterwards.
  *
  * About object lifecycle<br>
- * In belle-sip, objects can be, depending on their types, initially owned, that there are created with a ref count of 1, or
- * initially unowned, that is with reference count of 0. Such objets are also referred as "floating object". They are automatically destroyed
+ * In belle-sip, objects can be, depending on their types, initially owned, meaning that they are created with a ref count of 1, or
+ * initially unowned, meaning that they are created with reference count of 0. Such objets are also referred as "floating object". They are automatically destroyed
  * by the main loop iteration, so a floating object can be seen as a temporary object, until someones calls belle_sip_object_ref() on it.
  *
  * In order to know whether a kind of object is initially owned or initially unowned, you can use the test program tester/belle_sip_object_describe.
@@ -136,13 +145,17 @@ typedef unsigned int belle_sip_type_id_t;
  * Internally, belle-sip objects containing pointers to other objects must take a reference count on the other objects they hold; and leave this reference
  * when they no longer need it. This rule must be strictly followed by developers doing things inside belle-sip.
 **/
-typedef struct _belle_sip_object belle_sip_object_t;
 
+typedef struct _belle_sip_object belle_sip_object_t;
+//cpp object differ only in vptr, just typedef to belle_sip_object_t for simplicity
+typedef struct _belle_sip_object belle_sip_cpp_object_t;
 
 typedef void (*belle_sip_object_destroy_t)(belle_sip_object_t*);
 typedef void (*belle_sip_object_clone_t)(belle_sip_object_t* obj, const belle_sip_object_t *orig);
 typedef belle_sip_error_code (*belle_sip_object_marshal_t)(belle_sip_object_t* obj, char* buff, size_t buff_size, size_t *offset);
 typedef struct _belle_sip_object_vptr *(*belle_sip_object_get_vptr_t)(void);
+typedef void (*belle_sip_object_on_first_ref_t)(belle_sip_object_t*);
+typedef void (*belle_sip_object_on_last_ref_t)(belle_sip_object_t*);
 
 struct _belle_sip_object_vptr{
 	belle_sip_type_id_t id;
@@ -154,7 +167,11 @@ struct _belle_sip_object_vptr{
 	belle_sip_object_destroy_t destroy;
 	belle_sip_object_clone_t clone;
 	belle_sip_object_marshal_t marshal;
+	belle_sip_object_on_first_ref_t on_first_ref; /*called when object is ref'd for the first time*/
+	belle_sip_object_on_last_ref_t on_last_ref; /*called in unref() when the last reference of the object remains*/
 	int tostring_bufsize_hint; /*optimization: you can suggest here the typical size for a to_string() result.*/
+	int is_cpp; /*indicates whether this object kind is defined in c++*/
+	int cpp_offset; /*offset to apply to the belle_sip_object_t to find the pointer to the corresponding bellesip::Object, if any*/
 };
 
 typedef struct _belle_sip_object_vptr belle_sip_object_vptr_t;
@@ -169,14 +186,16 @@ struct _belle_sip_object{
 	belle_sip_list_t *data_store;
 };
 
-
 BELLE_SIP_BEGIN_DECLS
 
 
 BELLESIP_EXPORT belle_sip_object_t * _belle_sip_object_new(size_t objsize, belle_sip_object_vptr_t *vptr);
 
+#ifdef __cplusplus
+#define belle_sip_object_new(_type) reinterpret_cast<_type*>(static_cast<void *>(_belle_sip_object_new(sizeof(_type),(belle_sip_object_vptr_t*)BELLE_SIP_OBJECT_GET_VPTR_FUNC(_type)())))
+#else
 #define belle_sip_object_new(_type) (_type*)_belle_sip_object_new(sizeof(_type),(belle_sip_object_vptr_t*)BELLE_SIP_OBJECT_GET_VPTR_FUNC(_type)())
-
+#endif
 
 /**
  * Activates checks on object marshalling.
@@ -200,6 +219,10 @@ BELLESIP_EXPORT int belle_sip_object_get_object_count(void);
 BELLESIP_EXPORT void belle_sip_object_flush_active_objects(void);
 
 BELLESIP_EXPORT void belle_sip_object_dump_active_objects(void);
+/*
+ * Might be used to explicitly remove an object from leak detector.
+*/
+BELLESIP_EXPORT void belle_sip_object_remove_from_leak_detector(belle_sip_object_t *obj);
 
 /**
  * Suspend leak detector from this point. If the leak detector wasn't activated, this function does nothing.
@@ -221,6 +244,10 @@ BELLESIP_EXPORT belle_sip_object_t * belle_sip_object_ref(void *obj);
 **/
 BELLESIP_EXPORT void belle_sip_object_unref(void *obj);
 
+/**
+ * Decrements the reference counter. When it drops to zero, the object is destroyed and returns TRUE. Otherwise returns FALSE.
+**/
+BELLESIP_EXPORT int belle_sip_object_unref_2(void *obj);
 
 typedef void (*belle_sip_object_destroy_notify_t)(void *userpointer, belle_sip_object_t *obj_being_destroyed);
 /**
@@ -350,7 +377,7 @@ BELLESIP_EXPORT void *belle_sip_object_cast(belle_sip_object_t *obj, belle_sip_t
  * WHen the object is a sip header, uri or message, this is the textual representation of the header, uri or message.
  * This function internally calls belle_sip_object_marshal().
 **/
-BELLESIP_EXPORT char* belle_sip_object_to_string(void* obj);
+BELLESIP_EXPORT char* belle_sip_object_to_string(const void* obj);
 
 /**
  * Writes a string representation of the object into the supplied buffer.
@@ -517,4 +544,3 @@ operator<<( std::ostream& __os, const belle_sip_object_t* object)
 }
 #endif
 #endif
-
