@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef BELLE_SIP_UTILS_H
 #define BELLE_SIP_UTILS_H
@@ -25,19 +26,11 @@
 #include <errno.h>
 #include "belle-sip/defs.h"
 
-#ifdef BCTBX_LOG_DOMAIN
-#undef BCTBX_LOG_DOMAIN
-#endif
-#ifndef BELLE_SIP_LOG_DOMAIN
-#define BELLE_SIP_LOG_DOMAIN "belle-sip"
-#endif
-
-#define BCTBX_LOG_DOMAIN BELLE_SIP_LOG_DOMAIN
-
-
 #include "bctoolbox/logging.h"
 #include "bctoolbox/vconnect.h"
 
+#define BELLE_SIP_LOG_DOMAIN "belle-sip"
+
 BELLE_SIP_BEGIN_DECLS
 
 #define belle_sip_malloc bctbx_malloc
@@ -60,7 +53,7 @@ BELLE_SIP_END_DECLS
 #define	BELLE_SIP_LOG_END BCTBX_LOG_END
 #define  belle_sip_log_level BctbxLogLevel
 
-#define belle_sip_log_function_t BctoolboxLogFunc
+#define belle_sip_log_function_t BctbxLogFunc
 
 
 typedef enum {
@@ -105,9 +98,8 @@ BELLE_SIP_BEGIN_DECLS
 #endif
 
 
-#define belle_sip_set_log_file bctbx_set_log_file
-#define belle_sip_set_log_handler bctbx_set_log_handler
-#define belle_sip_get_log_handler bctbx_get_log_handler
+BELLESIP_EXPORT void  belle_sip_set_log_handler(belle_sip_log_function_t func);
+
 
 #define belle_sip_strdup_printf bctbx_strdup_printf
 #define belle_sip_strcat_vprintf bctbx_strcat_vprintf
@@ -119,9 +111,12 @@ BELLESIP_EXPORT belle_sip_error_code belle_sip_snprintf_valist(char *buff, size_
 #define belle_sip_set_log_level(level) bctbx_set_log_level(BELLE_SIP_LOG_DOMAIN,level);
 
 BELLESIP_EXPORT char * belle_sip_random_token(char *ret, size_t size);
+BELLESIP_EXPORT char * belle_sip_random_token_with_charset(char *ret, size_t size, const char * charset, size_t charset_length);
 
 BELLESIP_EXPORT unsigned char * belle_sip_random_bytes(unsigned char *ret, size_t size);
 
+BELLESIP_EXPORT uint32_t belle_sip_random(void);
+
 BELLESIP_EXPORT char * belle_sip_octets_to_text(const unsigned char *hash, size_t hash_len, char *ret, size_t size);
 
 BELLESIP_EXPORT char * belle_sip_create_tag(char *ret, size_t size);
@@ -135,8 +130,6 @@ BELLESIP_EXPORT char *belle_sip_unquote_strdup(const char *str);
 
 BELLESIP_EXPORT uint64_t belle_sip_time_ms(void);
 
-BELLESIP_EXPORT unsigned int belle_sip_random(void);
-
 /** Connect API */
 BELLESIP_EXPORT void belle_sip_set_socket_api(bctbx_vsocket_api_t* my_api);
 
@@ -171,7 +164,10 @@ BELLESIP_EXPORT void belle_sip_end_background_task(unsigned long id);
  */
 BELLESIP_EXPORT int belle_sip_mkdir(const char *path);
 
+BELLESIP_EXPORT	char* belle_sip_uri_to_escaped_username(const char* buff);
+BELLESIP_EXPORT char* belle_sip_username_unescape_unnecessary_characters(const char* buff);
+BELLESIP_EXPORT char* belle_sip_to_unescaped_string(const char* buff);
+
 BELLE_SIP_END_DECLS
 
 #endif
-
