@@ -1,20 +1,22 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-	Copyright (C) 2010  Belledonne Communications SARL
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
-	This program is free software: you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation, either version 2 of the License, or
-	(at your option) any later version.
-
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
-
-	You should have received a copy of the GNU General Public License
-	along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
 #ifndef belle_utils_h
 #define belle_utils_h
 
@@ -66,7 +68,11 @@
 
 #endif
 
+#ifdef HAVE_DNS_SERVICE // must be tested after config.h is included
+#include "dispatch/dispatch.h"
+#endif /* HAVE_DNS_SERVICE */
 
+#define SOCKET_NOT_SET ((belle_sip_socket_t)-1)
 /*etc*/
 
 #define BELLE_SIP_INTERFACE_GET_METHODS(obj,interface) \
@@ -133,6 +139,9 @@ typedef struct weak_ref{
 	void *userpointer;
 }weak_ref_t;
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 void *belle_sip_object_get_interface_methods(belle_sip_object_t *obj, belle_sip_interface_id_t ifid);
 /*used internally by unref()*/
@@ -140,6 +149,12 @@ void belle_sip_object_delete(void *obj);
 void belle_sip_object_pool_add(belle_sip_object_pool_t *pool, belle_sip_object_t *obj);
 void belle_sip_object_pool_remove(belle_sip_object_pool_t *pool, belle_sip_object_t *obj);
 
+
+
+belle_sip_object_t * _belle_sip_object_init(belle_sip_object_t *obj, belle_sip_object_vptr_t *vptr);
+void belle_sip_cpp_object_delete(belle_sip_object_t *obj);
+const char * belle_sip_cpp_object_get_type_name(const belle_sip_object_t *obj);
+void belle_sip_object_uninit(belle_sip_object_t *obj);
 #define belle_sip_object_init(obj)		/*nothing*/
 
 
@@ -162,6 +177,7 @@ BELLE_SIP_DECLARE_VPTR(belle_sip_request_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_response_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_parameters_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_call_id_t);
+BELLE_SIP_DECLARE_VPTR(belle_sip_header_retry_after_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_cseq_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_content_type_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_route_t);
@@ -191,6 +207,14 @@ BELLE_SIP_DECLARE_VPTR(belle_sdp_raw_attribute_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_repeate_time_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_rtcp_fb_attribute_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_rtcp_xr_attribute_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_content_attribute_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_label_attribute_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_creq_attribute_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_csup_attribute_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_tcap_attribute_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_acap_attribute_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_acfg_attribute_t);
+BELLE_SIP_DECLARE_VPTR(belle_sdp_pcfg_attribute_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_session_description_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_session_name_t);
 BELLE_SIP_DECLARE_VPTR(belle_sdp_time_t);
@@ -205,6 +229,7 @@ BELLE_SIP_DECLARE_VPTR(belle_sip_header_service_route_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_refer_to_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_referred_by_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_replaces_t);
+BELLE_SIP_DECLARE_VPTR(belle_sip_header_session_expires_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_date_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_hop_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_object_pool_t);
@@ -224,11 +249,17 @@ BELLE_SIP_DECLARE_VPTR(belle_tls_crypto_config_t);
 BELLE_SIP_DECLARE_VPTR(belle_http_header_authorization_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_event_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_supported_t);
+BELLE_SIP_DECLARE_VPTR(belle_sip_header_require_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_content_disposition_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_accept_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_reason_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_header_authentication_info_t);
-
+#ifdef HAVE_MDNS
+BELLE_SIP_DECLARE_VPTR(belle_sip_mdns_source_t);
+BELLE_SIP_DECLARE_VPTR(belle_sip_mdns_register_t);
+#endif
+BELLE_SIP_DECLARE_VPTR(belle_sip_resolver_results_t);
+BELLE_SIP_DECLARE_VPTR(belle_sip_digest_authentication_policy_t);
 
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_resolver_context_t,belle_sip_source_t)
 	void (*cancel)(belle_sip_resolver_context_t *);
@@ -256,7 +287,7 @@ struct belle_sip_source{
 	long armed_events;
 	unsigned short pad;
 #endif
-	int timeout;
+	int64_t timeout;
 	void *data;
 	uint64_t expire_ms;
 	int index; /* index in pollfd table */
@@ -268,7 +299,7 @@ struct belle_sip_source{
 	unsigned char oneshot;
 	unsigned char notify_required; /*for testing purpose, use to ask for being scheduled*/
 	bctbx_iterator_t *it; /*for fast removal*/
-	belle_sip_main_loop_t *ml; 
+	belle_sip_main_loop_t *ml;
 };
 
 void belle_sip_socket_source_init(belle_sip_source_t *s, belle_sip_source_func_t func, void *data, belle_sip_socket_t fd, unsigned int events, unsigned int timeout_value_ms);
@@ -408,14 +439,8 @@ struct belle_sip_dict {
 		belle_sip_parameters_set_parameter(BELLE_SIP_PARAMETERS(obj),#attribute,NULL);\
 	}
 
-#ifdef HAVE_ANTLR_STRING_STREAM_NEW
 #define ANTLR_STREAM_NEW(object_type, value,length) \
 antlr3StringStreamNew((pANTLR3_UINT8)value,ANTLR3_ENC_8BIT,(ANTLR3_UINT32)length,(pANTLR3_UINT8)#object_type)
-#else
-#define ANTLR_STREAM_NEW(object_type, value, length) \
-antlr3NewAsciiStringCopyStream((pANTLR3_UINT8)value,(ANTLR3_UINT32)length,NULL)
-#endif /*HAVE_ANTLR_STRING_STREAM_NEW*/
-
 
 #define BELLE_PARSE(parser_name, object_type_prefix, object_type) \
 	object_type_prefix##object_type##_t* object_type_prefix##object_type##_parse (const char* value) { \
@@ -484,10 +509,7 @@ belle_sip_param_pair_t* belle_sip_param_pair_ref(belle_sip_param_pair_t* obj);
 
 void belle_sip_param_pair_unref(belle_sip_param_pair_t* obj);
 
-
-
-
-/*calss header*/
+/*class header*/
 struct _belle_sip_header {
 	belle_sip_object_t base;
 	belle_sip_header_t* next;
@@ -495,11 +517,11 @@ struct _belle_sip_header {
 	char *unparsed_value;
 };
 
-
 void belle_sip_response_fill_for_dialog(belle_sip_response_t *obj, belle_sip_request_t *req);
 void belle_sip_util_copy_headers(belle_sip_message_t *orig, belle_sip_message_t *dest, const char*header, int multiple);
 
 void belle_sip_header_init(belle_sip_header_t* obj);
+
 /*class parameters*/
 struct _belle_sip_parameters {
 	belle_sip_header_t base;
@@ -522,6 +544,7 @@ struct belle_sip_hop{
 	char *host;
 	char *transport;
 	int port;
+	int port_is_explicit;
 };
 
 
@@ -533,12 +556,14 @@ struct belle_sip_stack{
 	belle_sip_main_loop_t *ml;
 	belle_sip_timer_config_t timer_config;
 	int transport_timeout;
+	int unreliable_transport_timeout;
 	int inactive_transport_timeout;
 	int dns_timeout;
 	int tx_delay; /*used to simulate network transmission delay, for tests*/
 	int send_error; /* used to simulate network error. if <0, channel_send will return this value*/
 	int resolver_tx_delay; /*used to simulate network transmission delay, for tests*/
 	int resolver_send_error;	/* used to simulate network error*/
+	int test_bind_port;
 	int dscp;
 	char *dns_user_hosts_file; /* used to load additional hosts file for tests */
 	char *dns_resolv_conf; /*used to load custom resolv.conf, for tests*/
@@ -548,9 +573,16 @@ struct belle_sip_stack{
 	int http_proxy_port;
 	char *http_proxy_username; /*for future use*/
 	char *http_proxy_passwd; /*for future use*/
+	belle_sip_digest_authentication_policy_t *digest_auth_policy;
 
 	unsigned char dns_srv_enabled;
 	unsigned char dns_search_enabled;
+	unsigned char reconnect_to_primary_asap;
+	unsigned char simulate_non_working_srv;
+#ifdef HAVE_DNS_SERVICE
+	dispatch_queue_t dns_service_queue;
+	unsigned char use_dns_service;
+#endif /* HAVE_DNS_SERVICE */
 };
 
 BELLESIP_EXPORT belle_sip_hop_t* belle_sip_hop_new(const char* transport, const char *cname, const char* host,int port);
@@ -558,6 +590,8 @@ BELLESIP_EXPORT belle_sip_hop_t* belle_sip_hop_new_from_uri(const belle_sip_uri_
 BELLESIP_EXPORT belle_sip_hop_t* belle_sip_hop_new_from_generic_uri(const belle_generic_uri_t *uri);
 
 BELLESIP_EXPORT belle_sip_hop_t * belle_sip_stack_get_next_hop(belle_sip_stack_t *stack, belle_sip_request_t *req);
+/* Return -1 if requested authentication is not compatible with local digest authentication security policy, 0 if compatible. */
+BELLESIP_EXPORT int belle_sip_stack_check_digest_compatibility(const belle_sip_stack_t *stack, const belle_sip_header_www_authenticate_t *authenticate);
 
 /*
  belle_sip_provider_t
@@ -573,10 +607,11 @@ struct belle_sip_provider{
 	belle_sip_list_t *server_transactions;
 	belle_sip_list_t *dialogs;
 	belle_sip_list_t *auth_contexts;
+	unsigned short unconditional_answer;
 	unsigned char rport_enabled; /*0 if rport should not be set in via header*/
 	unsigned char nat_helper;
 	unsigned char unconditional_answer_enabled;
-	unsigned short unconditional_answer;
+	unsigned char response_integrity_checking_enabled;
 };
 
 BELLESIP_EXPORT belle_sip_provider_t *belle_sip_provider_new(belle_sip_stack_t *s, belle_sip_listening_point_t *lp);
@@ -588,7 +623,7 @@ BELLESIP_EXPORT belle_sip_server_transaction_t * belle_sip_provider_find_matchin
 void belle_sip_provider_remove_server_transaction(belle_sip_provider_t *prov, belle_sip_server_transaction_t *t);
 void belle_sip_provider_set_transaction_terminated(belle_sip_provider_t *p, belle_sip_transaction_t *t);
 void *belle_sip_transaction_get_application_data_internal(const belle_sip_transaction_t *t);
-belle_sip_channel_t * belle_sip_provider_get_channel(belle_sip_provider_t *p, const belle_sip_hop_t *hop);
+BELLESIP_EXPORT belle_sip_channel_t * belle_sip_provider_get_channel(belle_sip_provider_t *p, const belle_sip_hop_t *hop);
 void belle_sip_provider_add_dialog(belle_sip_provider_t *prov, belle_sip_dialog_t *dialog);
 void belle_sip_provider_remove_dialog(belle_sip_provider_t *prov, belle_sip_dialog_t *dialog);
 void belle_sip_provider_release_channel(belle_sip_provider_t *p, belle_sip_channel_t *chan);
@@ -616,13 +651,12 @@ typedef struct listener_ctx{
 /*
  * http provider
  */
-belle_http_provider_t *belle_http_provider_new(belle_sip_stack_t *s, const char *bind_ip);
+belle_http_provider_t *belle_http_provider_new(belle_sip_stack_t *s, const char *bind_ip, const uint8_t transports);
 
 
 /*
  * SIP and http messages
 **/
-#define BELLESIP_MULTIPART_BOUNDARY "---------------------------14737809831466499882746641449"
 
 void belle_sip_message_init(belle_sip_message_t *message);
 
@@ -630,6 +664,8 @@ struct _belle_sip_message {
 	belle_sip_object_t base;
 	belle_sip_list_t* header_list;
 	belle_sip_body_handler_t *body_handler;
+
+	char *multipart_body_cache;
 };
 
 struct _belle_sip_request {
@@ -656,6 +692,7 @@ struct belle_http_request{
 	int auth_attempt_count;
 	int background_task_id;
 	int cancelled;
+	int resubmitted; /* Indicates whether this request has been resubmitted to a new channel after the first one was closed by the server.*/
 };
 
 void belle_http_request_set_listener(belle_http_request_t *req, belle_http_request_listener_t *l);
@@ -725,6 +762,7 @@ struct belle_sip_client_transaction{
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_client_transaction_t,belle_sip_transaction_t)
 	void (*send_request)(belle_sip_client_transaction_t *);
 	void (*on_response)(belle_sip_client_transaction_t *obj, belle_sip_response_t *resp);
+	void (*stop_retransmissions)(belle_sip_client_transaction_t *obj);
 BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 
 void belle_sip_client_transaction_init(belle_sip_client_transaction_t *obj, belle_sip_provider_t *prov, belle_sip_request_t *req);
@@ -738,6 +776,8 @@ struct belle_sip_ict{
 	belle_sip_source_t *timer_D;
 	belle_sip_source_t *timer_M;
 	belle_sip_request_t *ack;
+	/*last acknoleged provisional response sequence number. Note that "The RSeq numbering space is within a single transaction."*/
+	unsigned int r_cseq;
 };
 
 typedef struct belle_sip_ict belle_sip_ict_t;
@@ -857,7 +897,7 @@ struct belle_sip_dialog{
 
 belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t);
 belle_sip_dialog_t * belle_sip_provider_create_dialog_internal(belle_sip_provider_t *prov, belle_sip_transaction_t *t,unsigned int check_last_resp);
-int belle_sip_dialog_is_authorized_transaction(const belle_sip_dialog_t *dialog,const char* method) ;
+int belle_sip_dialog_can_accept_request(const belle_sip_dialog_t *dialog, belle_sip_request_t *request) ;
 /*returns 1 if message belongs to the dialog, 0 otherwise */
 int belle_sip_dialog_is_null_dialog_with_matching_subscribe(belle_sip_dialog_t *obj, const char *call_id, const char *local_tag, belle_sip_request_t *notify);
 int _belle_sip_dialog_match(belle_sip_dialog_t *obj, const char *call_id, const char *local_tag, const char *remote_tag);
@@ -880,22 +920,36 @@ belle_sip_hop_t* belle_sip_response_get_return_hop(belle_sip_response_t *msg);
  */
 #define BELLE_SDP_PARSE(object_type) \
 belle_sdp_##object_type##_t* belle_sdp_##object_type##_parse (const char* value) { \
-	pANTLR3_INPUT_STREAM           input; \
-	pbelle_sdpLexer               lex; \
-	pANTLR3_COMMON_TOKEN_STREAM    tokens; \
-	pbelle_sdpParser              parser; \
-	belle_sdp_##object_type##_t* l_parsed_object; \
-	input  = ANTLR_STREAM_NEW(object_type, value,strlen(value));\
-	lex    = belle_sdpLexerNew                (input);\
-	tokens = antlr3CommonTokenStreamSourceNew  (ANTLR3_SIZE_HINT, TOKENSOURCE(lex));\
-	parser = belle_sdpParserNew               (tokens);\
-	l_parsed_object = parser->object_type(parser).ret;\
-	parser ->free(parser);\
-	tokens ->free(tokens);\
-	lex    ->free(lex);\
-	input  ->close(input);\
-	if (l_parsed_object == NULL) belle_sip_error(#object_type" parser error for [%s]",value);\
-	return l_parsed_object;\
+	if (!belle_sdp_use_belr) {\
+		pANTLR3_INPUT_STREAM           input; \
+		pbelle_sdpLexer               lex; \
+		pANTLR3_COMMON_TOKEN_STREAM    tokens; \
+		pbelle_sdpParser              parser; \
+		belle_sdp_##object_type##_t* l_parsed_object; \
+		input  = ANTLR_STREAM_NEW(object_type, value,strlen(value));\
+		lex    = belle_sdpLexerNew                (input);\
+		tokens = antlr3CommonTokenStreamSourceNew  (ANTLR3_SIZE_HINT, TOKENSOURCE(lex));\
+		parser = belle_sdpParserNew               (tokens);\
+		l_parsed_object = parser->object_type(parser).ret;\
+		parser ->free(parser);\
+		tokens ->free(tokens);\
+		lex    ->free(lex);\
+		input  ->close(input);\
+		if (l_parsed_object == NULL) belle_sip_error(#object_type" parser error for [%s]",value);\
+		return l_parsed_object;\
+	} else {\
+		auto parser = bellesip::SDP::Parser::getInstance();\
+		auto object = parser->parse(value, #object_type);\
+		if (object == NULL) belle_sip_error(#object_type" parser error for [%s]",value);\
+		return (belle_sdp_##object_type##_t *)object;\
+	}\
+}
+#define BELLE_SDP_BELR_PARSE(object_type) \
+belle_sdp_##object_type##_t* belle_sdp_##object_type##_parse (const char* value) { \
+	auto parser = bellesip::SDP::Parser::getInstance();\
+	auto object = parser->parse(value, #object_type);\
+	if (object == NULL) belle_sip_error(#object_type" parser error for [%s]",value);\
+	return (belle_sdp_##object_type##_t *)object;\
 }
 #define BELLE_SDP_NEW(object_type,super_type) \
 		BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sdp_##object_type##_t); \
@@ -976,6 +1030,7 @@ struct belle_sip_auth_event {
 	char* distinguished_name;
 	belle_sip_certificates_chain_t * cert;
 	belle_sip_signing_key_t* key;
+	char* algorithm; /* either MD5 ot SHA256*/
 };
 
 belle_sip_auth_event_t* belle_sip_auth_event_create(belle_sip_object_t *source, const char* realm,const belle_sip_uri_t * from_uri);
@@ -992,7 +1047,6 @@ belle_sip_refresher_t* belle_sip_refresher_new(belle_sip_client_transaction_t* t
  * returns a char, even if entry is escaped*/
 size_t belle_sip_get_char (const char*a,char*out);
 /*return an escaped string*/
-BELLESIP_EXPORT	char* belle_sip_uri_to_escaped_username(const char* buff) ;
 BELLESIP_EXPORT	char* belle_sip_uri_to_escaped_userpasswd(const char* buff) ;
 BELLESIP_EXPORT	char* belle_sip_uri_to_escaped_parameter(const char* buff) ;
 BELLESIP_EXPORT	char* belle_sip_uri_to_escaped_header(const char* buff) ;
@@ -1036,7 +1090,7 @@ typedef struct _belle_sip_header_extension belle_sip_header_extension_t;
 belle_sip_header_extension_t* belle_sip_header_extension_new(void);
 
 belle_sip_header_extension_t* belle_sip_header_extension_parse (const char* extension) ;
-belle_sip_header_extension_t* belle_sip_header_extension_create (const char* name,const char* value);
+BELLESIP_EXPORT belle_sip_header_extension_t* belle_sip_header_extension_create (const char* name,const char* value);
 BELLESIP_EXPORT const char* belle_sip_header_extension_get_value(const belle_sip_header_extension_t* extension);
 void belle_sip_header_extension_set_value(belle_sip_header_extension_t* extension,const char* value);
 #define BELLE_SIP_HEADER_EXTENSION(t) BELLE_SIP_CAST(t,belle_sip_header_extension_t)
@@ -1089,7 +1143,22 @@ void belle_sip_multipart_body_handler_progress_cb(belle_sip_body_handler_t *obj,
 belle_sip_list_t *belle_sip_parse_directory(const char *path, const char *file_type);
 
 typedef struct authorization_context authorization_context_t;
-BELLESIP_EXPORT void belle_sip_authorization_destroy(authorization_context_t* object);
+typedef authorization_context_t belle_sip_authorization_t;
 
+BELLESIP_EXPORT void belle_sip_authorization_destroy(belle_sip_authorization_t* object);
+BELLESIP_EXPORT const char *belle_sip_authorization_get_algorithm(const belle_sip_authorization_t* object);
+
+/**
+ * Generate a random unsigned int
+ */
+uint32_t belle_sip_random(void);
+#ifdef __cplusplus
+}
+#endif
+
+
+#if TARGET_OS_IPHONE
+int belle_sip_get_ios_device_major_version();
+#endif
 
 #endif
