@@ -1,21 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
-
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #define _CRT_RAND_S
 #include <stddef.h>
@@ -23,6 +23,7 @@
 #include <sys/stat.h>
 #include "belle_sip_internal.h"
 #include "bctoolbox/parser.h"
+#include "bctoolbox/crypto.h"
 
 #include "clock_gettime.h" /*for apple*/
 
@@ -197,68 +198,6 @@ char *belle_sip_unquote_strdup(const char *str){
 	return belle_sip_strdup(str);
 }
 
-#if defined(_WIN32) && !defined(_MSC_VER)
-#include <wincrypt.h>
-static int belle_sip_wincrypto_random(unsigned int *rand_number){
-	static HCRYPTPROV hProv=(HCRYPTPROV)-1;
-	static int initd=0;
-
-	if (!initd){
-		if (!CryptAcquireContext(&hProv,NULL,NULL,PROV_RSA_FULL, CRYPT_VERIFYCONTEXT)){
-			belle_sip_error("Could not acquire a windows crypto context");
-			return -1;
-		}
-		initd=TRUE;
-	}
-	if (hProv==(HCRYPTPROV)-1)
-		return -1;
-
-	if (!CryptGenRandom(hProv,4,(BYTE*)rand_number)){
-		belle_sip_error("CryptGenRandom() failed.");
-		return -1;
-	}
-	return 0;
-}
-#endif
-
-unsigned int belle_sip_random(void){
-#if  defined(__linux) || defined(__APPLE__)
-	static int fd=-1;
-	if (fd==-1) fd=open("/dev/urandom",O_RDONLY);
-	if (fd!=-1){
-		unsigned int tmp;
-		if (read(fd,&tmp,4)!=4){
-			belle_sip_error("Reading /dev/urandom failed.");
-		}else return tmp;
-	}else belle_sip_error("Could not open /dev/urandom");
-#elif defined(_WIN32)
-	static int initd=0;
-	unsigned int ret;
-#ifdef _MSC_VER
-	/*rand_s() is pretty nice and simple function but is not wrapped by mingw.*/
-
-	if (rand_s(&ret)==0){
-		return ret;
-	}
-#else
-	if (belle_sip_wincrypto_random(&ret)==0){
-		return ret;
-	}
-#endif
-	/* Windows's rand() is unsecure but is used as a fallback*/
-	if (!initd) {
-		srand((unsigned int)belle_sip_time_ms());
-		initd=1;
-		belle_sip_warning("Random generator is using rand(), this is unsecure !");
-	}
-	return rand()<<16 | rand();
-#endif
-	/*fallback to UNIX random()*/
-#ifndef _WIN32
-	return (unsigned int) random();
-#endif
-}
-
 
 void belle_sip_set_socket_api(bctbx_vsocket_api_t* my_api){
 
@@ -271,31 +210,19 @@ static const char *symbols="aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ
  * Write a random text token of supplied size.
 **/
 char * belle_sip_random_token(char *ret, size_t size){
-	unsigned int val=0;
-	unsigned int i;
+	return belle_sip_random_token_with_charset(ret, size, symbols, 64);
+}
 
+char * belle_sip_random_token_with_charset(char *ret, size_t size, const char * charset, size_t charset_length){
+	unsigned int i;
+	belle_sip_random_bytes((unsigned char *)ret, size-1);
 	for(i=0;i<size-1;++i){
-		if (i%5==0) val=belle_sip_random();
-		ret[i]=symbols[val & 63];
-		val=val>>6;
+		ret[i]=charset[ret[i] % charset_length];
 	}
 	ret[i]=0;
 	return ret;
 }
 
-/**
- * Write random bytes of supplied size.
-**/
-unsigned char * belle_sip_random_bytes(unsigned char *ret, size_t size){
-	unsigned int val=0;
-	unsigned int i;
-	for(i=0;i<size;++i){
-		if (i%4==0) val=belle_sip_random();
-		ret[i]=val & 0xff;
-		val=val>>8;
-	}
-	return ret;
-}
 
 typedef struct bits_reader{
 	const uint8_t *buffer;
@@ -499,6 +426,9 @@ static const bctbx_noescape_rules_t* get_sip_uri_header_noescapes(void) {
 char* belle_sip_uri_to_escaped_username(const char* buff) {
 	return bctbx_escape(buff, *get_sip_uri_username_noescapes());
 }
+char* belle_sip_username_unescape_unnecessary_characters(const char* buff) {
+	return bctbx_unescaped_string_only_chars_in_rules(buff, *get_sip_uri_username_noescapes());
+}
 char* belle_sip_uri_to_escaped_userpasswd(const char* buff) {
 	return bctbx_escape(buff, *get_sip_uri_userpasswd_noescapes());
 }
@@ -588,12 +518,13 @@ char* belle_generic_uri_to_escaped_path(const char* buff) {
 }
 
 char* belle_sip_string_to_backslash_less_unescaped_string(const char* buff) {
-	char *output_buff=belle_sip_malloc(strlen(buff)+1);
+	size_t buff_len = strlen(buff);
+	char *output_buff=belle_sip_malloc(buff_len+1);
 	unsigned int i;
 	unsigned int out_buff_index=0;
 
 	for(i=0; buff[i] != '\0'; i++) {
-		if (buff[i] == '\\') {
+		if (buff[i] == '\\' && i + 1 < buff_len ) { /*make sure escaped caracter exist*/
 			i++;/*skip \*/
 		}
 		/*make sure to only remove one \ in case of \\*/
@@ -702,4 +633,3 @@ int belle_sip_mkdir(const char *path) {
 	return mkdir(path, 0700);
 #endif
 }
-
