@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2014  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle_sip_internal.h"
 #include "bctoolbox/vfs.h"
@@ -75,6 +76,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_body_handler_t)
 		(belle_sip_object_destroy_t) belle_sip_body_handler_destroy,
 		(belle_sip_object_clone_t) belle_sip_body_handler_clone,
 		(belle_sip_object_marshal_t) belle_sip_body_handler_marshal,
+		(belle_sip_object_on_first_ref_t) NULL,
+		(belle_sip_object_on_last_ref_t) NULL,
 		BELLE_SIP_DEFAULT_BUFSIZE_HINT
 	},
 	NULL, /* begin_recv_transfer */
@@ -122,6 +125,7 @@ size_t belle_sip_body_handler_get_transfered_size(const belle_sip_body_handler_t
 }
 
 static void update_progress(belle_sip_body_handler_t *obj, belle_sip_message_t *msg){
+	belle_sip_debug("body handler [%p] : %llu bytes exchanged of %llu ", obj, (unsigned long long)obj->transfered_size, (unsigned long long)obj->expected_size);
 	if (obj->progress_cb)
 		obj->progress_cb(obj,msg,obj->user_data,obj->transfered_size,obj->expected_size);
 }
@@ -252,10 +256,14 @@ void belle_sip_memory_body_handler_set_buffer(belle_sip_memory_body_handler_t *o
 	obj->buffer = (uint8_t *)buffer;
 }
 
+#define BELLE_SIP_MEMORY_BODY_HANDLER_MINIMUM_DEFLATE_INPUT_SIZE 256
 #define BELLE_SIP_MEMORY_BODY_HANDLER_ZLIB_INITIAL_SIZE 2048
 
-void belle_sip_memory_body_handler_apply_encoding(belle_sip_memory_body_handler_t *obj, const char *encoding) {
-	if ((obj->buffer == NULL) || (obj->encoding_applied == TRUE)) return;
+int belle_sip_memory_body_handler_apply_encoding(belle_sip_memory_body_handler_t *obj, const char *encoding) {
+	if (obj->encoding_applied == TRUE)
+		return 0;
+	if (!obj->buffer || (belle_sip_body_handler_get_size(BELLE_SIP_BODY_HANDLER(obj)) < BELLE_SIP_MEMORY_BODY_HANDLER_MINIMUM_DEFLATE_INPUT_SIZE))
+		return -1;
 
 #ifdef HAVE_ZLIB
 	if (strcmp(encoding, "deflate") == 0) {
@@ -272,7 +280,10 @@ void belle_sip_memory_body_handler_apply_encoding(belle_sip_memory_body_handler_
 		strm.zfree = Z_NULL;
 		strm.opaque = Z_NULL;
 		ret = deflateInit(&strm, Z_DEFAULT_COMPRESSION);
-		if (ret != Z_OK) return;
+		if (ret != Z_OK) {
+			belle_sip_free(outbuf);
+			return -1;
+		}
 		strm.avail_in = (uInt)initial_size;
 		strm.next_in = obj->buffer;
 		do {
@@ -290,15 +301,22 @@ void belle_sip_memory_body_handler_apply_encoding(belle_sip_memory_body_handler_
 		} while (strm.avail_out == 0);
 		deflateEnd(&strm);
 		final_size = outbuf_ptr - outbuf;
+		if ((final_size + 27) >= initial_size) { // 27 is the size of the Content-Encoding header
+			belle_sip_message("Body not compressed because its size would have increased");
+			belle_sip_free(outbuf);
+			return -1;
+		}
 		belle_sip_message("Body has been compressed: %u->%u:\n%s", (unsigned int)initial_size, (unsigned int)final_size, obj->buffer);
 		belle_sip_free(obj->buffer);
 		obj->buffer = outbuf;
 		belle_sip_body_handler_set_size(BELLE_SIP_BODY_HANDLER(obj), final_size);
 		obj->encoding_applied = TRUE;
+		return 0;
 	} else
 #endif
 	{
 		belle_sip_warning("%s: unknown encoding '%s'", __FUNCTION__, encoding);
+		return -1;
 	}
 }
 
@@ -411,6 +429,84 @@ belle_sip_memory_body_handler_t *belle_sip_memory_body_handler_new_copy_from_buf
 	return obj;
 }
 
+/*
+ * Buffer manager, used for buffering user or file body handler
+ */
+typedef struct belle_sip_body_handler_buffer {
+	size_t size; /**< the maximum buffer size + 1, if != 0, all received chunks are modified to have a size which is 0 mod (buffer_size) */
+	size_t index; /**< amount of data currently stored in buffer */
+	uint8_t *data; /**< data stored from previous chunk */
+	off_t next_offset; /**< store the next offset value */
+} belle_sip_body_handler_buffer_t;
+
+/**
+ * Clone a belle_sip_body_handler_buffer_t structure
+ */
+static void belle_sip_body_handler_buffer_clone(belle_sip_body_handler_buffer_t *dst, const belle_sip_body_handler_buffer_t *src) {
+	dst->size=src->size;
+	dst->index=src->index;
+	dst->next_offset=src->next_offset;
+	if (src->data != NULL) {
+		dst->data=(uint8_t *)belle_sip_malloc(src->size);
+		memcpy(dst->data, src->data, src->size);
+	} else {
+		dst->data=NULL;
+	}
+}
+
+/*
+ * Buffer the received data if needed
+ * if buffer.size is not 0, reorganise the incoming data to deliver chunk of which size is 0 mod buffer.size
+ *
+ * @param[in/out] 	buffer	holds buffer and indexes
+ * @param[in/out] 	offset	points at the end of processed data
+ * @param[in]		input	points to the incoming data
+ * @param[out		output	points to the data affected by the buffering
+ * @param[in/out]	size of the incoming buffer and of the outgoing one
+ * @param[out]		free_output_flag	set to TRUE if the caller must free the output buffer after usage
+ */
+static void belle_sip_buffering_recv(belle_sip_body_handler_buffer_t *buffer, off_t *offset, uint8_t *input, uint8_t **output, size_t *size, bool_t *free_output_flag) {
+	*free_output_flag=FALSE; // default is: do not free the output
+
+	/* Do we need to buffer stuff ? */
+	if (buffer->size > 0) {
+		size_t bufferized_size = buffer->index;
+
+		*offset = buffer->next_offset; /* adjust the offset to keep track of past bufferized parts */
+		size_t total_size = buffer->index+*size;
+		if (total_size >= buffer->size) { /* we have enough data to produce a chunk of at least buffer size */
+			/* we must bufferize what is over for next time */
+			buffer->index = total_size%buffer->size; /* save the pointer to the part we shall bufferize as we cannot crash the buffer yet */
+			uint8_t *next_buffer_input = input + *size - buffer->index;
+
+			*size = total_size - buffer->index; /* size must be 0 mod (buffer index) */
+			buffer->next_offset += (off_t)*size;
+			/* create this chunk */
+			if (bufferized_size > 0) { /* if we had some previously buffered data, prepend, otherwise, directly use the input, the modified size will do the trick */
+				*output = (uint8_t *)belle_sip_malloc(*size);
+				memcpy(*output, buffer->data, bufferized_size);
+				memcpy(*output+bufferized_size, input, *size-bufferized_size);
+				*free_output_flag=TRUE;
+			} else {
+				*output = input;
+			}
+			/* crash the buffer with content needed for next round */
+			memcpy(buffer->data, next_buffer_input, buffer->index);
+		} else if (*size == 0 ) { /* this is the end, pass the content of the buffer */
+			*size = buffer->index;
+			buffer->next_offset += (off_t)*size;
+			*output = buffer->data;
+		} else { /* just add the current chunk into the buffer */
+			memcpy(buffer->data+buffer->index, input, *size);
+			buffer->index += *size;
+			*size = 0; /* be sure we won't do anything with current data */
+			*output=NULL;
+		}
+	} else { /* nothing to do, just pass the input */
+		*output = input;
+	}
+}
+
 /*
  * User body handler implementation
  */
@@ -421,6 +517,14 @@ struct belle_sip_user_body_handler{
 	belle_sip_user_body_handler_recv_callback_t recv_cb;
 	belle_sip_user_body_handler_send_callback_t send_cb;
 	belle_sip_user_body_handler_stop_callback_t stop_cb;
+	belle_sip_body_handler_buffer_t buffer;
+};
+
+static void belle_sip_user_body_handler_destroy(belle_sip_user_body_handler_t *obj) {
+	if (obj->buffer.data) {
+		belle_sip_free(obj->buffer.data);
+		obj->buffer.data=NULL;
+	}
 };
 
 static void belle_sip_user_body_handler_begin_transfer(belle_sip_body_handler_t *base) {
@@ -429,17 +533,26 @@ static void belle_sip_user_body_handler_begin_transfer(belle_sip_body_handler_t
 		obj->start_cb((belle_sip_user_body_handler_t*)base, base->user_data);
 }
 
-static void belle_sip_user_body_handler_end_transfer(belle_sip_body_handler_t *base) {
-	belle_sip_user_body_handler_t *obj = (belle_sip_user_body_handler_t *)base;
-	if (obj->stop_cb)
-		obj->stop_cb((belle_sip_user_body_handler_t*)base, base->user_data);
-}
-
 static void belle_sip_user_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t size){
 	belle_sip_user_body_handler_t *obj=(belle_sip_user_body_handler_t*)base;
-	if (obj->recv_cb)
-		obj->recv_cb((belle_sip_user_body_handler_t*)base, msg, base->user_data, offset, buf, size);
-	else belle_sip_warning("belle_sip_user_body_handler_t ignoring received chunk.");
+	if (obj->recv_cb) {
+		/* buffer the incoming data if requested */
+		uint8_t *bufferized_data = NULL;
+		bool_t free_output_flag = FALSE;
+		belle_sip_buffering_recv(&(obj->buffer), &offset, buf, &bufferized_data, &size, &free_output_flag);
+
+		/* send it to the callback if there is data to process */
+		if (size > 0) {
+			obj->recv_cb((belle_sip_user_body_handler_t*)base, msg, base->user_data, offset, bufferized_data, size);
+		}
+
+		/* cleaning */
+		if (free_output_flag) {
+			belle_sip_free(bufferized_data);
+		}
+	} else {
+		belle_sip_warning("belle_sip_user_body_handler_t ignoring received chunk.");
+	}
 }
 
 static int belle_sip_user_body_handler_send_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t *size){
@@ -451,11 +564,25 @@ static int belle_sip_user_body_handler_send_chunk(belle_sip_body_handler_t *base
 	return BELLE_SIP_STOP;
 }
 
+static void belle_sip_user_body_handler_end_transfer(belle_sip_body_handler_t *base) {
+	belle_sip_user_body_handler_t *obj = (belle_sip_user_body_handler_t *)base;
+
+	/* Are we bufferizing things ? */
+	if (obj->buffer.size > 0 && obj->buffer.index > 0) {
+		/* call the recv_chunk function with the buffer content */
+		belle_sip_user_body_handler_recv_chunk(base, NULL, obj->buffer.next_offset, NULL, 0);
+	}
+
+	if (obj->stop_cb)
+		obj->stop_cb((belle_sip_user_body_handler_t*)base, base->user_data);
+}
+
 static void belle_sip_user_body_handler_clone(belle_sip_user_body_handler_t *obj, const belle_sip_user_body_handler_t *orig){
 	obj->start_cb=orig->start_cb;
 	obj->recv_cb=orig->recv_cb;
 	obj->send_cb=orig->send_cb;
 	obj->stop_cb=orig->stop_cb;
+	belle_sip_body_handler_buffer_clone(&(obj->buffer), &(orig->buffer));
 }
 
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_user_body_handler_t);
@@ -463,7 +590,7 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_user_body_handler_t)
 	{
 		{
 			BELLE_SIP_VPTR_INIT(belle_sip_user_body_handler_t,belle_sip_body_handler_t,TRUE),
-			(belle_sip_object_destroy_t) NULL,
+			(belle_sip_object_destroy_t)belle_sip_user_body_handler_destroy,
 			(belle_sip_object_clone_t)belle_sip_user_body_handler_clone,
 			NULL,
 			BELLE_SIP_DEFAULT_BUFSIZE_HINT
@@ -491,10 +618,30 @@ belle_sip_user_body_handler_t *belle_sip_user_body_handler_new(
 	obj->recv_cb=recv_cb;
 	obj->send_cb=send_cb;
 	obj->stop_cb=stop_cb;
+	obj->buffer.size=0;
+	obj->buffer.index=0;
+	obj->buffer.data=NULL;
+	obj->buffer.next_offset=0;
 	return obj;
 }
 
+belle_sip_user_body_handler_t *belle_sip_buffering_user_body_handler_new(
+	size_t total_size,
+	size_t buffer_size,
+	belle_sip_body_handler_progress_callback_t progress_cb,
+	belle_sip_user_body_handler_start_callback_t start_cb,
+	belle_sip_user_body_handler_recv_callback_t recv_cb,
+	belle_sip_user_body_handler_send_callback_t send_cb,
+	belle_sip_user_body_handler_stop_callback_t stop_cb,
+	void *data){
 
+	belle_sip_user_body_handler_t *obj = belle_sip_user_body_handler_new(total_size, progress_cb, start_cb, recv_cb, send_cb, stop_cb, data);
+
+	obj->buffer.size=buffer_size;
+	obj->buffer.data=(uint8_t *)belle_sip_malloc(buffer_size);
+
+	return obj;
+}
 /**
  * File body handler implementation
 **/
@@ -504,6 +651,7 @@ struct belle_sip_file_body_handler{
 	char *filepath;
 	bctbx_vfs_file_t *file;
 	belle_sip_user_body_handler_t *user_bh;
+	belle_sip_body_handler_buffer_t buffer;
 };
 
 static void belle_sip_file_body_handler_destroy(belle_sip_file_body_handler_t *obj) {
@@ -516,6 +664,10 @@ static void belle_sip_file_body_handler_destroy(belle_sip_file_body_handler_t *o
 		}
 		obj->file = NULL;
 	}
+	if (obj->buffer.data) {
+		belle_sip_free(obj->buffer.data);
+		obj->buffer.data=NULL;
+	}
 	if (obj->user_bh) {
 		belle_sip_object_unref(obj->user_bh);
 		obj->user_bh = NULL;
@@ -528,7 +680,10 @@ static void belle_sip_file_body_handler_clone(belle_sip_file_body_handler_t *obj
 	obj->user_bh = orig->user_bh;
 	if (obj->user_bh) {
 		belle_sip_object_ref(obj->user_bh);
+	} else {
+		obj->user_bh = NULL;
 	}
+	belle_sip_body_handler_buffer_clone(&(obj->buffer), &(orig->buffer));
 }
 
 static void belle_sip_file_body_handler_begin_recv_transfer(belle_sip_body_handler_t *base) {
@@ -540,7 +695,7 @@ static void belle_sip_file_body_handler_begin_recv_transfer(belle_sip_body_handl
 	if (!obj->file) {
 		bctbx_error("Can't open file %s", obj->filepath);
 	}
-	
+
 	if (obj->user_bh && obj->user_bh->start_cb) {
 		obj->user_bh->start_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), obj->user_bh->base.user_data);
 	}
@@ -561,14 +716,47 @@ static void belle_sip_file_body_handler_begin_send_transfer(belle_sip_body_handl
 	}
 }
 
+
+static void belle_sip_file_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t size) {
+	belle_sip_file_body_handler_t *obj = (belle_sip_file_body_handler_t *)base;
+	ssize_t ret=0;
+
+	if (obj->file == NULL) return;
+
+	uint8_t *bufferized_data = NULL;
+	bool_t free_output_flag = FALSE;
+
+	if (obj->user_bh && obj->user_bh->recv_cb) {
+		// pass the incoming data through buffering
+		belle_sip_buffering_recv(&(obj->buffer), &offset, buf, &bufferized_data, &size, &free_output_flag);
+
+		if (size>0) {
+			obj->user_bh->recv_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), msg, obj->user_bh->base.user_data, offset, bufferized_data, size);
+		}
+	}
+
+	if (size > 0 ) {
+		ret = bctbx_file_write(obj->file, bufferized_data, size, offset);
+	}
+
+	if (free_output_flag == TRUE) {
+		belle_sip_free(bufferized_data);
+	}
+
+	if (ret == BCTBX_VFS_ERROR) {
+		bctbx_error("File body handler recv write error at offset %lu", (unsigned long)offset);
+	}
+}
+
 static void belle_sip_file_body_handler_end_transfer(belle_sip_body_handler_t *base) {
 	belle_sip_file_body_handler_t *obj = (belle_sip_file_body_handler_t *)base;
-	
-	if (obj->user_bh && obj->user_bh->stop_cb) {
-		obj->user_bh->stop_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), obj->user_bh->base.user_data);
+
+	/* Are we bufferizing things ? */
+	if (obj->buffer.size > 0 && obj->buffer.index > 0) {
+		/* call the recv_chunk function with the buffer content */
+		belle_sip_file_body_handler_recv_chunk(base, NULL, obj->buffer.next_offset, NULL, 0);
 	}
-	
-	if (obj->file) {
+	if (obj->file) {// Close the file before calling stop callback to let cb to do file modifications like renaming.
 		ssize_t ret;
 		ret = bctbx_file_close(obj->file);
 		if (ret == BCTBX_VFS_ERROR) {
@@ -576,22 +764,11 @@ static void belle_sip_file_body_handler_end_transfer(belle_sip_body_handler_t *b
 		}
 		obj->file = NULL;
 	}
-}
-
-static void belle_sip_file_body_handler_recv_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t size) {
-	belle_sip_file_body_handler_t *obj = (belle_sip_file_body_handler_t *)base;
-	ssize_t ret;
-
-	if (obj->file == NULL) return;
-	
-	if (obj->user_bh && obj->user_bh->recv_cb) {
-		obj->user_bh->recv_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), msg, obj->user_bh->base.user_data, offset, buf, size);
+	if (obj->user_bh && obj->user_bh->stop_cb) {
+		obj->user_bh->stop_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), obj->user_bh->base.user_data);
 	}
+
 	
-	ret = bctbx_file_write(obj->file, buf, size, offset);
-	if (ret == BCTBX_VFS_ERROR) {
-		bctbx_error("File body handler recv write error at offset %lu", (unsigned long)offset);
-	}
 }
 
 static int belle_sip_file_body_handler_send_chunk(belle_sip_body_handler_t *base, belle_sip_message_t *msg, off_t offset, uint8_t *buf, size_t *size) {
@@ -606,12 +783,12 @@ static int belle_sip_file_body_handler_send_chunk(belle_sip_body_handler_t *base
 		return BELLE_SIP_STOP;
 	}
 	*size = (size_t)size_t_ret;
-	
+
 	if (obj->user_bh && obj->user_bh->send_cb) {
 		int result = obj->user_bh->send_cb((belle_sip_user_body_handler_t*)&(obj->user_bh->base), msg, obj->user_bh->base.user_data, offset, buf, size);
 		if (result == BELLE_SIP_STOP) return result;
 	}
-	
+
 	return (((obj->base.expected_size - offset) == (size_t)size_t_ret) || (*size == 0)) ? BELLE_SIP_STOP : BELLE_SIP_CONTINUE;
 }
 
@@ -634,14 +811,40 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_file_body_handler_t)
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
 belle_sip_file_body_handler_t *belle_sip_file_body_handler_new(const char *filepath, belle_sip_body_handler_progress_callback_t progress_cb, void *data) {
+	// Get file size if it exists
+	int64_t expected_size = BCTBX_VFS_ERROR;
 	struct stat statbuf;
+	if (stat(filepath, &statbuf) == 0) {
+		bctbx_vfs_t *vfs = bctbx_vfs_get_default();
+		bctbx_vfs_file_t *fp = bctbx_file_open(vfs, filepath, "r");
+		expected_size = bctbx_file_size(fp);
+		bctbx_file_close(fp);
+	}
+
 	belle_sip_file_body_handler_t *obj = belle_sip_object_new(belle_sip_file_body_handler_t);
 	belle_sip_body_handler_init((belle_sip_body_handler_t*)obj, progress_cb, data);
 	obj->filepath = belle_sip_strdup(filepath);
 	obj->user_bh = NULL;
-	if (stat(obj->filepath, &statbuf) == 0) {
-		obj->base.expected_size = statbuf.st_size;
+	if (expected_size != BCTBX_VFS_ERROR) {
+		obj->base.expected_size = (size_t)expected_size;
+		belle_sip_message("file body handler created for file [%s] with size [%zu]", filepath, obj->base.expected_size);
+	} else {
+		belle_sip_message("file body handler created for file [%s] with unknown size", filepath);
 	}
+	obj->buffer.size=0;
+	obj->buffer.index=0;
+	obj->buffer.data=NULL;
+	obj->buffer.next_offset=0;
+	
+	return obj;
+}
+
+belle_sip_file_body_handler_t *belle_sip_buffering_file_body_handler_new(const char *filepath, const size_t buffer_size, belle_sip_body_handler_progress_callback_t progress_cb, void *data) {
+	belle_sip_file_body_handler_t *obj = belle_sip_file_body_handler_new(filepath, progress_cb, data);
+
+	obj->buffer.size=buffer_size;
+	obj->buffer.data=(uint8_t *)belle_sip_malloc(buffer_size);
+
 	return obj;
 }
 
@@ -721,7 +924,7 @@ static void belle_sip_multipart_body_handler_recv_chunk(belle_sip_body_handler_t
 }
 
 static int belle_sip_multipart_body_handler_send_chunk(belle_sip_body_handler_t *obj, belle_sip_message_t *msg, off_t offset,
-							uint8_t *buffer, size_t *size){
+														 uint8_t *buffer, size_t *size){
 	belle_sip_multipart_body_handler_t *obj_multipart=(belle_sip_multipart_body_handler_t*)obj;
 
 	if (obj_multipart->transfer_current_part->data) { /* we have a part, get its content from handler */
@@ -733,7 +936,10 @@ static int belle_sip_multipart_body_handler_send_chunk(belle_sip_body_handler_t
 
 		if (current_part->transfered_size == 0) { /* Nothing transfered yet on this part, include a separator and the header if exists */
 			size_t headersSize = 0;
-			offsetSize = strlen(obj_multipart->boundary) + 4; /* 4 is for "--" and "\r\n" */
+			if (current_part != (belle_sip_body_handler_t *)obj_multipart->parts->data) {
+				offsetSize += 2; /*delimiter := CRLF dash-boundary*/
+			}
+			offsetSize += boundary_len + 4;  /* 4 is for "--" and "\r\n" */
 
 			if (current_part->headerStringBuffer != NULL) {
 				headersSize = strlen(current_part->headerStringBuffer);
@@ -745,10 +951,12 @@ static int belle_sip_multipart_body_handler_send_chunk(belle_sip_body_handler_t
 			}
 
 			/* insert separator */
-			memcpy(buffer, "--", 2);
-			memcpy(buffer + 2, obj_multipart->boundary, boundary_len);
-			memcpy(buffer + 2 + boundary_len, "\r\n", 2);
-			offsetSize = boundary_len + 4;
+			if (current_part != (belle_sip_body_handler_t *)obj_multipart->parts->data) {
+				snprintf((char*)buffer, *size, "\r\n--%s\r\n",obj_multipart->boundary); /*delimiter := CRLF dash-boundary*/
+			} else {
+				snprintf((char*)buffer, *size, "--%s\r\n",obj_multipart->boundary);
+			}
+
 			/* insert part header */
 			if (headersSize!=0) {
 				memcpy(buffer+offsetSize, current_part->headerStringBuffer, headersSize);
@@ -781,6 +989,7 @@ static int belle_sip_multipart_body_handler_send_chunk(belle_sip_body_handler_t
 	return BELLE_SIP_STOP;
 }
 
+
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_multipart_body_handler_t);
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_multipart_body_handler_t)
 	{
@@ -799,6 +1008,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_multipart_body_handler_t)
 	}
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
+static const char *boundary_charset = "aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ0123456789-";
+
 static void belle_sip_multipart_body_handler_set_boundary(belle_sip_multipart_body_handler_t *obj, const char *boundary) {
 	if (obj->boundary != NULL) {
 		belle_sip_free(obj->boundary);
@@ -806,10 +1017,15 @@ static void belle_sip_multipart_body_handler_set_boundary(belle_sip_multipart_bo
 	if (boundary != NULL) {
 		obj->boundary = belle_sip_strdup(boundary);
 	} else {
-		obj->boundary = belle_sip_strdup(BELLESIP_MULTIPART_BOUNDARY);
+		char random_token[60];// Use a random boundary to reduce the probability of having the same boundary in file (like uploading file logs with file transfer).
+		obj->boundary = belle_sip_strdup_printf("----------%s", belle_sip_random_token_with_charset(random_token, sizeof(random_token), boundary_charset, 63));
 	}
 }
 
+const char *belle_sip_multipart_body_handler_get_boundary(const belle_sip_multipart_body_handler_t *obj) {
+	return obj->boundary;
+}
+
 belle_sip_multipart_body_handler_t *belle_sip_multipart_body_handler_new(belle_sip_body_handler_progress_callback_t progress_cb, void *data,
 									 belle_sip_body_handler_t *first_part, const char *boundary){
 	belle_sip_multipart_body_handler_t *obj=belle_sip_object_new(belle_sip_multipart_body_handler_t);
@@ -820,7 +1036,7 @@ belle_sip_multipart_body_handler_t *belle_sip_multipart_body_handler_new(belle_s
 	return obj;
 }
 
-belle_sip_multipart_body_handler_t *belle_sip_multipart_body_handler_new_from_buffer(void *buffer, size_t bufsize, const char *boundary) {
+belle_sip_multipart_body_handler_t *belle_sip_multipart_body_handler_new_from_buffer(const void *buffer, size_t bufsize, const char *boundary) {
 	belle_sip_multipart_body_handler_t *obj_multipart = belle_sip_object_new(belle_sip_multipart_body_handler_t);
 	belle_sip_body_handler_t *obj = (belle_sip_body_handler_t *)obj_multipart;
 	belle_sip_body_handler_init((belle_sip_body_handler_t *)obj, belle_sip_multipart_body_handler_progress_cb, NULL);
@@ -834,6 +1050,10 @@ belle_sip_multipart_body_handler_t *belle_sip_multipart_body_handler_new_from_bu
 
 #define DEFAULT_HEADER_STRING_SIZE 512
 void belle_sip_multipart_body_handler_add_part(belle_sip_multipart_body_handler_t *obj, belle_sip_body_handler_t *part){
+	if (obj->parts != NULL) {
+		obj->base.expected_size += 2; /*delimiter := CRLF dash-boundary*/
+		//add initial CRLF*/
+	}
 	obj->base.expected_size+=part->expected_size+strlen(obj->boundary) + 4; /* add the separator length to the body length as each part start with a separator. 4 is for "--" and "\r\n" */
 	if (part->headers != NULL) { /* there is a declared header for this part, add its length to the expected total length */
 		size_t headerStringBufferSize = DEFAULT_HEADER_STRING_SIZE;
@@ -877,24 +1097,35 @@ void belle_sip_multipart_body_handler_progress_cb(belle_sip_body_handler_t *obj,
 		uint8_t *end_headers_cursor;
 		uint8_t *end_header_cursor;
 		uint8_t *cursor = obj_multipart->buffer;
-		char *boundary = belle_sip_strdup_printf("--%s", obj_multipart->boundary);
-		if (strncmp((char *)cursor, boundary, strlen(boundary))) {
+		char *dash_boundary = belle_sip_strdup_printf("--%s", obj_multipart->boundary);
+		size_t expected_size = obj_multipart->base.expected_size; /* Save expected size. Indeed add_parts() plays with it
+		* for the composing process. For parsing process we actually don't need this, so the correct size will be restored at the end.*/
+		
+		if (strncmp((char *)cursor, dash_boundary, strlen(dash_boundary))) {
 			belle_sip_warning("belle_sip_multipart_body_handler [%p]: body not starting by specified boundary '%s'", obj_multipart, obj_multipart->boundary);
-			belle_sip_free(boundary);
+			belle_sip_free(dash_boundary);
 			return;
 		}
-		cursor += strlen(boundary);
+		cursor += strlen(dash_boundary);
+		
 		do {
+			bool_t delimiter_contains_crlf = FALSE;
 			if (strncmp((char *)cursor, "\r\n", 2)) {
 				belle_sip_warning("belle_sip_multipart_body_handler [%p]: no new-line after boundary", obj_multipart);
+				belle_sip_free(dash_boundary);
 				return;
 			}
 			cursor += 2;
-			end_part_cursor = (uint8_t *)strstr((char *)cursor, boundary);
+			end_part_cursor = (uint8_t *)strstr((char *)cursor, dash_boundary);
 			if (end_part_cursor == NULL) {
 				belle_sip_warning("belle_sip_multipart_body_handler [%p]: cannot find next boundary", obj_multipart);
+				belle_sip_free(dash_boundary);
 				return;
 			} else {
+				if (*(end_part_cursor-1) == '\n' && *(end_part_cursor-2) == '\r') {
+					end_part_cursor-=2; /* delimiter is well formed: delimiter := CRLF dash-boundary */
+					delimiter_contains_crlf = TRUE;
+				}
 				*end_part_cursor = 0;
 				end_headers_cursor = (uint8_t *)strstr((char *)cursor, "\r\n\r\n");
 				if (end_headers_cursor == NULL) {
@@ -913,9 +1144,12 @@ void belle_sip_multipart_body_handler_progress_cb(belle_sip_body_handler_t *obj,
 					} while (end_header_cursor != end_headers_cursor);
 				}
 				belle_sip_multipart_body_handler_add_part(obj_multipart, BELLE_SIP_BODY_HANDLER(memorypart));
-				cursor = end_part_cursor + strlen(boundary);
+				cursor = end_part_cursor + strlen(dash_boundary);
+				if (delimiter_contains_crlf)
+					cursor += 2;
 			}
 		} while (strcmp((char *)cursor, "--\r\n"));
-		belle_sip_free(boundary);
+		belle_sip_free(dash_boundary);
+		obj_multipart->base.expected_size = expected_size;
 	}
 }
