@@ -1,21 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
-
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle_sip_internal.h"
 #include <limits.h>
@@ -34,7 +34,7 @@
 
 #define BELLE_SIP_CHANNEL_INVOKE_STATE_LISTENERS(channel,state) \
 	BELLE_SIP_INVOKE_LISTENERS_REVERSE_ARG1_ARG2(channel->full_listeners, belle_sip_channel_listener_t, on_state_changed, channel, state) \
-	BELLE_SIP_INVOKE_LISTENERS_REVERSE_ARG1_ARG2(channel->state_listeners, belle_sip_channel_listener_t, on_state_changed, channel, state) 
+	BELLE_SIP_INVOKE_LISTENERS_REVERSE_ARG1_ARG2(channel->state_listeners, belle_sip_channel_listener_t, on_state_changed, channel, state)
 
 
 static void channel_prepare_continue(belle_sip_channel_t *obj);
@@ -110,7 +110,6 @@ static size_t belle_sip_channel_input_stream_get_buff_length(belle_sip_channel_i
 
 static void belle_sip_channel_destroy(belle_sip_channel_t *obj){
 	belle_sip_channel_input_stream_reset(&obj->input_stream);
-	if (obj->peer_list) bctbx_freeaddrinfo(obj->peer_list);
 	if (obj->peer_cname) belle_sip_free(obj->peer_cname);
 	belle_sip_free(obj->peer_name);
 	if (obj->local_ip) belle_sip_free(obj->local_ip);
@@ -121,6 +120,8 @@ static void belle_sip_channel_destroy(belle_sip_channel_t *obj){
 		belle_sip_resolver_context_cancel(obj->resolver_ctx);
 		belle_sip_object_unref(obj->resolver_ctx);
 	}
+	SET_OBJECT_PROPERTY(obj, resolver_results, NULL);
+	if (obj->static_peer_list) bctbx_freeaddrinfo(obj->static_peer_list);
 	if (obj->inactivity_timer){
 		belle_sip_main_loop_remove_source(obj->stack->ml,obj->inactivity_timer);
 		belle_sip_object_unref(obj->inactivity_timer);
@@ -152,6 +153,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_channel_t)
 		(belle_sip_object_destroy_t)belle_sip_channel_destroy,
 		NULL, /*clone*/
 		NULL, /*marshal*/
+		NULL, /*on_first_ref*/
+		NULL, /*on_last_ref*/
 		BELLE_SIP_DEFAULT_BUFSIZE_HINT
 	},
 	NULL, /* transport */
@@ -175,8 +178,10 @@ static void fix_incoming_via(belle_sip_request_t *msg, const struct addrinfo* or
 		return;
 	}
 	bctbx_sockaddr_remove_v4_mapping(origin->ai_addr, (struct sockaddr*)&saddr, &slen);
-	err=bctbx_getnameinfo((struct sockaddr*)&saddr,slen,received,sizeof(received),
-	                rport,sizeof(rport),NI_NUMERICHOST|NI_NUMERICSERV);
+	err=bctbx_getnameinfo(
+		(struct sockaddr*)&saddr,slen,received,sizeof(received),
+		rport,sizeof(rport),NI_NUMERICHOST|NI_NUMERICSERV
+	);
 	if (err!=0){
 		belle_sip_error("fix_via: getnameinfo() failed: %s",gai_strerror(errno));
 		return;
@@ -196,10 +201,8 @@ static void fix_incoming_via(belle_sip_request_t *msg, const struct addrinfo* or
 	}
 }
 
-/*token       =  1*(alphanum / "-" / "." / "!" / "%" / "*"
-                     / "_" / "+" / "`" / "'" / "~" )
- *
- * */
+// token =  1*(alphanum / "-" / "." / "!" / "%" / "*" / "_" / "+" / "`" / "'" / "~" )
+
 static int is_token(const char* buff,size_t bufflen ) {
 	size_t i;
 	for (i=0; i<bufflen && buff[i]!='\0';i++) {
@@ -329,9 +332,10 @@ static void uncompress_body_if_required(belle_sip_message_t *msg) {
 			mbh = BELLE_SIP_MEMORY_BODY_HANDLER(bh);
 			if (belle_sip_memory_body_handler_unapply_encoding(mbh, content_encoding) == 0) {
 				belle_sip_header_content_type_t *content_type = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t);
+				belle_sip_header_content_length_t *content_length = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_length_t);
+				belle_sip_header_content_length_set_content_length(content_length, belle_sip_body_handler_get_size(BELLE_SIP_BODY_HANDLER(mbh)));
 				belle_sip_message_remove_header_from_ptr(msg, ceh);
-				if (content_type
-					&& (strcmp(belle_sip_header_content_type_get_type(content_type), "multipart") == 0)) {
+				if (content_type && (strcmp(belle_sip_header_content_type_get_type(content_type), "multipart") == 0)) {
 					const char *unparsed_value = belle_sip_header_get_unparsed_value(BELLE_SIP_HEADER(content_type));
 					const char *boundary = strstr(unparsed_value, ";boundary=");
 					if (boundary != NULL) boundary += 10;
@@ -375,7 +379,7 @@ static int check_body(belle_sip_channel_t *obj){
 	obj->input_stream.content_length= content_length_header ? belle_sip_header_content_length_get_content_length(content_length_header) : 0;
 
 	expect_body=obj->input_stream.content_length>0;
-	
+
 	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg,belle_http_response_t) || BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg,belle_http_request_t)){
 		/*http chunked mode handling*/
 		if (belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t)!=NULL){
@@ -395,25 +399,29 @@ static int check_body(belle_sip_channel_t *obj){
 	}
 	if (expect_body){
 		belle_sip_body_handler_t *bh;
-		/*should notify the listeners*/
+		// Should notify the listeners
 		BELLE_SIP_CHANNEL_INVOKE_MESSAGE_HEADERS_LISTENERS(obj,msg);
-		/*check if the listener has setup a body handler, otherwise create a default one*/
-		if ((bh=belle_sip_message_get_body_handler(msg))==NULL){
+		// Check if the listener has setup a body handler, otherwise create a default one
+		bh = belle_sip_message_get_body_handler(msg);
+		if (!bh) {
 			belle_sip_header_t *content_encoding = belle_sip_message_get_header(msg, "Content-Encoding");
 			belle_sip_header_content_type_t *content_type = belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t);
-			if (content_encoding != NULL) {
-				belle_sip_message_set_body_handler(msg, (bh = (belle_sip_body_handler_t *)belle_sip_memory_body_handler_new(NULL, NULL)));
-			} else if (content_type
-				&& (strcmp(belle_sip_header_content_type_get_type(content_type), "multipart") == 0)) {
+			if (content_encoding) {
+				bh = (belle_sip_body_handler_t *)belle_sip_memory_body_handler_new(NULL, NULL);
+				belle_sip_body_handler_add_header(bh, content_encoding);
+			} else if (content_type && (strcmp(belle_sip_header_content_type_get_type(content_type), "multipart") == 0)) {
 				const char *unparsed_value = belle_sip_header_get_unparsed_value(BELLE_SIP_HEADER(content_type));
 				const char *boundary = strstr(unparsed_value, ";boundary=");
-				if (boundary != NULL) boundary += 10;
+				if (boundary) boundary += 10;
 				if (boundary[0] == '\0') boundary = NULL;
-				belle_sip_message_set_body_handler(msg, (bh = (belle_sip_body_handler_t *)belle_sip_multipart_body_handler_new(belle_sip_multipart_body_handler_progress_cb, NULL, NULL, boundary)));
-				belle_sip_body_handler_set_size(bh, obj->input_stream.content_length);
+				bh = (belle_sip_body_handler_t *)belle_sip_multipart_body_handler_new(belle_sip_multipart_body_handler_progress_cb, NULL, NULL, boundary);
 			} else {
-				belle_sip_message_set_body_handler(msg,(bh=(belle_sip_body_handler_t*)belle_sip_memory_body_handler_new(NULL,NULL)));
+				bh = (belle_sip_body_handler_t *)belle_sip_memory_body_handler_new(NULL, NULL);
 			}
+			belle_sip_body_handler_set_size(bh, obj->input_stream.content_length);
+			belle_sip_body_handler_add_header(bh, BELLE_SIP_HEADER(content_length_header));
+			belle_sip_body_handler_add_header(bh, BELLE_SIP_HEADER(content_type));
+			belle_sip_message_set_body_handler(msg, bh);
 		}
 		belle_sip_body_handler_begin_recv_transfer(bh);
 	}
@@ -513,7 +521,7 @@ static int acquire_body(belle_sip_channel_t *obj, int end_of_stream){
 
 static void notify_incoming_messages(belle_sip_channel_t *obj){
 	belle_sip_list_t *elem,*l_it;
-	
+
 	belle_sip_list_t *listeners=belle_sip_list_copy_with_data(obj->full_listeners,(void *(*)(void*))belle_sip_object_ref);
 
 	for(l_it=listeners;l_it!=NULL;l_it=l_it->next){
@@ -721,7 +729,7 @@ static void update_inactivity_timer(belle_sip_channel_t *obj, int from_recv){
 			obj->inactivity_timer=belle_sip_main_loop_create_timeout(obj->stack->ml,channel_inactive_timeout,obj,inactive_timeout,"Channel inactivity timer");
 		}else{
 			/*restart the timer for new period*/
-			belle_sip_source_set_timeout(obj->inactivity_timer,inactive_timeout);
+			belle_sip_source_set_timeout_int64(obj->inactivity_timer,inactive_timeout);
 		}
 	}else{
 		if (obj->inactivity_timer){
@@ -739,17 +747,18 @@ static void update_inactivity_timer(belle_sip_channel_t *obj, int from_recv){
  * locaport locaport to use for binding, can be set to 0 if port doesn't matter
  * peer_cname canonical name of remote host, used for TLS verification
  * peername peer's hostname, either ip address or DNS name
- * pee_port peer's port to connect to.
+ * peer_port peer's port to connect to.
  */
-void belle_sip_channel_init(belle_sip_channel_t *obj, belle_sip_stack_t *stack,const char *bindip,int localport,const char *peer_cname, const char *peername, int peer_port){
+void belle_sip_channel_init(belle_sip_channel_t *obj, belle_sip_stack_t *stack,const char *bindip,int localport,const char *peer_cname, const char *peername, int peer_port, int no_srv){
 	/*to initialize our base class:*/
 	belle_sip_channel_set_socket(obj,-1,NULL);
-
+	belle_sip_message("belle_sip_channel_init(); peer_cname=%s", peer_cname);
 	/*then initialize members*/
 	obj->ai_family=AF_INET;
 	obj->peer_cname=peer_cname ? belle_sip_strdup(peer_cname) : NULL;
 	obj->peer_name=belle_sip_strdup(peername);
 	obj->peer_port=peer_port;
+	obj->no_srv = (unsigned char)no_srv;
 	obj->stack=stack;
 	if (bindip){
 		if (strcmp(bindip,"::0")!=0 && strcmp(bindip,"0.0.0.0")!=0)
@@ -780,8 +789,8 @@ void belle_sip_channel_init_with_addr(belle_sip_channel_t *obj, belle_sip_stack_
 	ai.ai_addr=(struct sockaddr*)peer_addr;
 	ai.ai_addrlen=addrlen;
 	bctbx_addrinfo_to_ip_address(&ai,remoteip,sizeof(remoteip),&peer_port);
-	belle_sip_channel_init(obj,stack,bindip,localport,NULL,remoteip,peer_port);
-	obj->peer_list=obj->current_peer=bctbx_ip_address_to_addrinfo(ai.ai_family, ai.ai_socktype, obj->peer_name,obj->peer_port);
+	belle_sip_channel_init(obj,stack,bindip,localport,NULL,remoteip,peer_port, TRUE);
+	obj->peer_list = obj->current_peer = obj->static_peer_list = bctbx_ip_address_to_addrinfo(ai.ai_family, ai.ai_socktype, obj->peer_name,obj->peer_port);
 	obj->ai_family=ai.ai_family;
 }
 
@@ -808,17 +817,19 @@ static void channel_remove_listener(belle_sip_channel_t *obj, belle_sip_channel_
 }
 
 void belle_sip_channel_add_listener(belle_sip_channel_t *obj, belle_sip_channel_listener_t *l){
-	
+
 	if (is_state_only_listener(l)) {
-		obj->state_listeners=belle_sip_list_prepend(obj->state_listeners,
-												   belle_sip_object_weak_ref(l,
-																	   (belle_sip_object_destroy_notify_t)channel_remove_listener,obj));
+		obj->state_listeners=belle_sip_list_prepend(
+			obj->state_listeners,
+			belle_sip_object_weak_ref(l, (belle_sip_object_destroy_notify_t)channel_remove_listener,obj)
+		);
 	} else {
-		obj->full_listeners=belle_sip_list_prepend(obj->full_listeners,
-													belle_sip_object_weak_ref(l,
-																		(belle_sip_object_destroy_notify_t)channel_remove_listener,obj));
+		obj->full_listeners=belle_sip_list_prepend(
+			obj->full_listeners,
+			belle_sip_object_weak_ref(l, (belle_sip_object_destroy_notify_t)channel_remove_listener,obj)
+		);
 	}
-	
+
 }
 
 void belle_sip_channel_remove_listener(belle_sip_channel_t *obj, belle_sip_channel_listener_t *l){
@@ -827,10 +838,20 @@ void belle_sip_channel_remove_listener(belle_sip_channel_t *obj, belle_sip_chann
 }
 
 int belle_sip_channel_matches(const belle_sip_channel_t *obj, const belle_sip_hop_t *hop, const struct addrinfo *addr){
-	if (hop && strcmp(hop->host,obj->peer_name)==0 && (hop->port==obj->peer_port || obj->srv_overrides_port)){
-		if (hop->cname && obj->peer_cname && strcmp(hop->cname,obj->peer_cname)!=0)
-			return 0; /*cname mismatch*/
-		return 1;
+	if (hop){
+		if (obj->current_peer_cname && strcasecmp(hop->host, obj->current_peer_cname)==0 && hop->port==obj->peer_port){
+			/*We are matching a specific node of a SRV record set. */
+			if (hop->cname && obj->peer_cname && strcasecmp(hop->cname,obj->current_peer_cname) != 0)
+				return 0; /*cname mismatch*/
+			return 1;
+		}
+		if (strcmp(hop->host,obj->peer_name)==0 && (hop->port==obj->peer_port || obj->srv_overrides_port)){
+			/*We may be matching the general name of the service, in that case the port doesn't matter.*/ 
+			if (hop->cname && obj->peer_cname && strcasecmp(hop->cname,obj->peer_cname) != 0){
+				return 0; /*cname mismatch*/
+			}
+			return 1;
+		}
 	}
 	if (addr && obj->current_peer)
 		return bctbx_sockaddr_equals(addr->ai_addr,obj->current_peer->ai_addr);
@@ -904,7 +925,6 @@ int belle_sip_channel_recv(belle_sip_channel_t *obj, void *buf, size_t buflen){
 void belle_sip_channel_close(belle_sip_channel_t *obj){
 	if (BELLE_SIP_OBJECT_VPTR(obj,belle_sip_channel_t)->close)
 		BELLE_SIP_OBJECT_VPTR(obj,belle_sip_channel_t)->close(obj); /*udp channel doesn't have close function*/
-	/*removing the source (our base class) will decrement the ref count, this why this code needs to be protected by ref/unref.*/
 	belle_sip_main_loop_remove_source(obj->stack->ml,(belle_sip_source_t*)obj);
 	belle_sip_source_uninit((belle_sip_source_t*)obj);
 }
@@ -992,13 +1012,27 @@ static void channel_connect_next(belle_sip_channel_t *obj){
 	belle_sip_object_unref(obj);
 }
 
+static void channel_set_current_peer(belle_sip_channel_t *obj, const struct addrinfo *ai){
+	if (obj->resolver_results){
+		const belle_sip_dns_srv_t *srv = belle_sip_resolver_results_get_srv_from_addrinfo(obj->resolver_results, ai);
+		obj->current_peer_cname = srv ? belle_sip_dns_srv_get_target(srv) : NULL;
+		if (obj->current_peer_cname){
+			belle_sip_message("channel[%p]: current peer hostname is [%s] on port [%i].", obj, 
+					  obj->current_peer_cname, belle_sip_dns_srv_get_port(srv));
+		}
+	}else{
+		obj->current_peer_cname = NULL;
+	}
+	obj->current_peer = ai;
+}
+
 static void belle_sip_channel_handle_error(belle_sip_channel_t *obj){
 	if (obj->state!=BELLE_SIP_CHANNEL_READY || obj->soft_error){
 		/* Previous connection attempts were failed (channel could not get ready) OR soft error reported*/
 		obj->soft_error = FALSE;
 		/* See if you can retry on an alternate ip address.*/
 		if (obj->current_peer && obj->current_peer->ai_next){ /*obj->current_peer may be null in case of dns error*/
-			obj->current_peer=obj->current_peer->ai_next;
+			channel_set_current_peer(obj, obj->current_peer->ai_next);
 			channel_set_state(obj,BELLE_SIP_CHANNEL_RETRY);
 			belle_sip_channel_close(obj);
 			belle_sip_main_loop_do_later(obj->stack->ml,(belle_sip_callback_t)channel_connect_next,belle_sip_object_ref(obj));
@@ -1015,15 +1049,16 @@ static void belle_sip_channel_handle_error(belle_sip_channel_t *obj){
 }
 
 int belle_sip_channel_notify_timeout(belle_sip_channel_t *obj){
-	const int too_long=60;
-	
+	const uint64_t too_long = 60;
+
 	if (obj->state != BELLE_SIP_CHANNEL_READY){
 		/*no need to notify the timeout if the channel is already in error or retry state*/
 		return FALSE;
 	}
-	
-	if ((int)(belle_sip_time_ms() - obj->last_recv_time) >= (too_long * 1000)){
-		belle_sip_message("A timeout related to this channel occured and no message received during last %i seconds. This channel is suspect, moving to error state",too_long);
+
+	if ((belle_sip_time_ms() - obj->last_recv_time) >= too_long * 1000){
+		belle_sip_message("A timeout related to this channel occured and no message received during last %i seconds. This channel is suspect, moving to error state", (int)too_long);
+		obj->soft_error = TRUE;
 		channel_set_state(obj,BELLE_SIP_CHANNEL_ERROR);
 		return TRUE;
 	}
@@ -1037,8 +1072,12 @@ void belle_sip_channel_notify_server_error(belle_sip_channel_t *obj){
 }
 
 void channel_set_state(belle_sip_channel_t *obj, belle_sip_channel_state_t state) {
-	belle_sip_message("channel %p: state %s",obj,belle_sip_channel_state_to_string(state));
+	belle_sip_message("channel[%p]: entering state %s",obj,belle_sip_channel_state_to_string(state));
 
+	if (obj->state == state) {
+		belle_sip_error("channel_set_state() called twice with the same state. This is a programming mistake.");
+		return;
+	}
 	if (state==BELLE_SIP_CHANNEL_ERROR){
 		belle_sip_channel_handle_error(obj);
 	}else{
@@ -1224,7 +1263,10 @@ static void compress_body_if_required(belle_sip_message_t *msg) {
 		}
 		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(bh, belle_sip_memory_body_handler_t)) {
 			mbh = BELLE_SIP_MEMORY_BODY_HANDLER(bh);
-			belle_sip_memory_body_handler_apply_encoding(mbh, content_encoding);
+			int ret = belle_sip_memory_body_handler_apply_encoding(mbh, content_encoding);
+			/* Remove Content-Encoding header if it could not be applied */
+			if (ret < 0)
+				belle_sip_message_remove_header_from_ptr(msg, ceh);
 		} else {
 			belle_sip_warning("message [%p] has Content-Encoding [%s] that cannot be applied", msg, content_encoding);
 		}
@@ -1248,6 +1290,9 @@ static void _send_message(belle_sip_channel_t *obj){
 			obj->ewouldblock_offset+=sendret;
 			if (obj->ewouldblock_offset==obj->ewouldblock_size){
 				free_ewouldblock_buffer(obj);
+				if (obj->out_state==OUTPUT_STREAM_SENDING_HEADERS)
+					goto done; //to avoid message to be sent twice
+				/*else continue body sending*/
 			}
 			/* continue to expedite the ewouldblock error until we it is completed or get a new ewouldblock*/
 		}else if (belle_sip_error_code_is_would_block(-sendret)) {
@@ -1383,11 +1428,11 @@ static void channel_process_queue(belle_sip_channel_t *obj){
 	belle_sip_message_t *msg;
 	belle_sip_object_ref(obj);/* we need to ref ourself because code below may trigger our destruction*/
 
-	if (obj->out_state!=OUTPUT_STREAM_IDLE) {
+	if (obj->out_state != OUTPUT_STREAM_IDLE) {
 		_send_message(obj);
 	}
 
-	while((msg=channel_pop_outgoing(obj))!=NULL && obj->state==BELLE_SIP_CHANNEL_READY && obj->out_state==OUTPUT_STREAM_IDLE) {
+	while (obj->state == BELLE_SIP_CHANNEL_READY && obj->out_state == OUTPUT_STREAM_IDLE && (msg = channel_pop_outgoing(obj)) != NULL) {
 		send_message(obj, msg);
 		belle_sip_object_unref(msg);
 	}
@@ -1429,60 +1474,95 @@ static int channel_dns_ttl_timeout(void *data, unsigned int event) {
 	return BELLE_SIP_STOP;
 }
 
-static bool_t addrinfo_in_list(const struct addrinfo *ai, const struct addrinfo *list) {
+/* returns the addrinfo from list that matches 'ai' in terms of content*/
+static const struct addrinfo* addrinfo_in_list(const struct addrinfo *ai, const struct addrinfo *list) {
 	const struct addrinfo *item = list;
-	bool_t in_list = FALSE;
 	while (item != NULL) {
 		if ((ai->ai_family == item->ai_family) && (bctbx_sockaddr_equals(ai->ai_addr, item->ai_addr))) {
-			in_list = TRUE;
-			break;
+			return item;
 		}
 		item = item->ai_next;
 	}
-	return in_list;
+	return NULL;
 }
 
-static void channel_res_done(void *data, const char *name, struct addrinfo *ai_list, uint32_t ttl){
+/* returns the first addrinfo from list, if it is matching 'ai' in terms content*/
+static const struct addrinfo * addrinfo_is_first(const struct addrinfo *ai, const struct addrinfo *list) {
+	if (list != NULL && (ai->ai_family == list->ai_family) && (bctbx_sockaddr_equals(ai->ai_addr, list->ai_addr)))
+		return list;
+	return NULL;
+}
+
+static void channel_res_done(void *data, belle_sip_resolver_results_t *results){
 	belle_sip_channel_t *obj=(belle_sip_channel_t*)data;
+	const struct addrinfo *ai_list = NULL;
+	const char *name = NULL;
+	belle_sip_resolver_results_t *prev_dns_results = obj->resolver_results;
+
 	if (obj->resolver_ctx){
 		belle_sip_object_unref(obj->resolver_ctx);
 		obj->resolver_ctx=NULL;
 	}
+	if (results){
+		/*update current resolver_results, peer_list. current_peer still points to an element of the old list, if it exists*/
+		obj->peer_list = ai_list = belle_sip_resolver_results_get_addrinfos(results);
+		belle_sip_object_ref(results);
+		name = belle_sip_resolver_results_get_name(results);
+	}
+	obj->resolver_results = results;
+
 	if (ai_list){
+		int ttl = belle_sip_resolver_results_get_ttl(results);
+		/* setup now our current_peer. It can be initialized for the first time, kept as before, or changed*/
 		if (!obj->current_peer) {
-			obj->peer_list=obj->current_peer=ai_list;
+			/*first time we connect*/
+			channel_set_current_peer(obj, ai_list);
 			channel_set_state(obj,BELLE_SIP_CHANNEL_RES_DONE);
 		} else {
-			if (addrinfo_in_list(obj->current_peer, ai_list)) {
+			const struct addrinfo *existing_peer;
+
+			if (belle_sip_stack_reconnect_to_primary_asap_enabled(obj->stack)) {
+				existing_peer = addrinfo_is_first(obj->current_peer, ai_list);
+			} else {
+				/*find if the current_peer we had is still existing in the new list. If yes, we will keep it*/
+				existing_peer = addrinfo_in_list(obj->current_peer, ai_list);
+			}
+
+			if (existing_peer) {
 				belle_sip_message("channel[%p]: DNS resolution returned the currently used address, continue using it", obj);
-				obj->peer_list = ai_list;
+				channel_set_current_peer(obj, existing_peer);
 				channel_set_state(obj, BELLE_SIP_CHANNEL_READY);
 			} else {
 				belle_sip_message("channel[%p]: DNS resolution returned an address different than the one being used, reconnect to the new address", obj);
-				obj->peer_list = obj->current_peer = ai_list;
+				channel_set_current_peer(obj, ai_list);
 				belle_sip_channel_close(obj);
 				belle_sip_main_loop_do_later(obj->stack->ml, (belle_sip_callback_t)channel_connect_next, belle_sip_object_ref(obj));
 				channel_set_state(obj, BELLE_SIP_CHANNEL_RETRY);
 			}
 		}
 		channel_prepare_continue(obj);
-		if (!obj->dns_ttl_timer ) {
-			obj->dns_ttl_timer = belle_sip_main_loop_create_timeout(obj->stack->ml, channel_dns_ttl_timeout, obj, ttl * 1000, "Channel DNS TTL timer");
-		} else {
-			/* Restart the timer for new period. */
-			belle_sip_source_set_timeout(obj->dns_ttl_timer, ttl * 1000);
-			belle_sip_main_loop_add_source(obj->stack->ml, obj->dns_ttl_timer);
+		if (ttl > 0){
+			if (!obj->dns_ttl_timer ) {
+				obj->dns_ttl_timer = belle_sip_main_loop_create_timeout(obj->stack->ml, channel_dns_ttl_timeout, obj, ttl * 1000, "Channel DNS TTL timer");
+			} else {
+				/* Restart the timer for new period. */
+				belle_sip_source_set_timeout_int64(obj->dns_ttl_timer, ttl * 1000LL);
+				belle_sip_main_loop_add_source(obj->stack->ml, obj->dns_ttl_timer);
+			}
 		}
 	}else{
+		channel_set_current_peer(obj, NULL);
 		belle_sip_error("%s: DNS resolution failed for %s", __FUNCTION__, name);
 		channel_set_state(obj,BELLE_SIP_CHANNEL_ERROR);
 	}
+	/*now free the old resolver results if any, since we don't need the old current_peer*/
+	if (prev_dns_results) belle_sip_object_unref(prev_dns_results);
 }
 
 void belle_sip_channel_resolve(belle_sip_channel_t *obj){
 	belle_sip_message("channel [%p]: starting resolution of %s", obj, obj->peer_name);
 	channel_set_state(obj,BELLE_SIP_CHANNEL_RES_IN_PROGRESS);
-	if (belle_sip_stack_dns_srv_enabled(obj->stack) && obj->lp!=NULL)
+	if (belle_sip_stack_dns_srv_enabled(obj->stack) && obj->lp!=NULL && obj->no_srv == 0)
 		obj->resolver_ctx=belle_sip_stack_resolve(obj->stack, "sip", belle_sip_channel_get_transport_name_lower_case(obj), obj->peer_name, obj->peer_port, obj->ai_family, channel_res_done, obj);
 	else
 		obj->resolver_ctx=belle_sip_stack_resolve_a(obj->stack, obj->peer_name, obj->peer_port, obj->ai_family, channel_res_done, obj);
@@ -1544,6 +1624,7 @@ static void queue_message_delayed(belle_sip_channel_t *obj, belle_sip_message_t
 	delay_send_t *ctx=belle_sip_malloc(sizeof(delay_send_t));
 	ctx->chan=(belle_sip_channel_t*)belle_sip_object_ref(obj);
 	ctx->msg=(belle_sip_message_t*)belle_sip_object_ref(msg);
+
 	belle_sip_main_loop_add_timeout(obj->stack->ml,(belle_sip_source_func_t)on_delayed_send_do,ctx,obj->stack->tx_delay);
 	belle_sip_message("channel %p: message sending delayed by %i ms",obj,obj->stack->tx_delay);
 }
@@ -1566,6 +1647,7 @@ belle_sip_channel_t *belle_sip_channel_find_from_list_with_addrinfo(belle_sip_li
 
 	for(elem=l;elem!=NULL;elem=elem->next){
 		chan=(belle_sip_channel_t*)elem->data;
+		if (chan->state == BELLE_SIP_CHANNEL_DISCONNECTED || chan->state == BELLE_SIP_CHANNEL_ERROR) continue;
 		if (!chan->about_to_be_closed && belle_sip_channel_matches(chan,hop,addr)){
 			return chan;
 		}
@@ -1593,11 +1675,11 @@ void belle_sip_channel_check_dns_reusability(belle_sip_channel_t *obj) {
 #ifdef __ANDROID__
 
 unsigned long belle_sip_begin_background_task(const char *name, belle_sip_background_task_end_callback_t cb, void *data){
-    return wake_lock_acquire(name);
+	return wake_lock_acquire(name);
 }
 
 void belle_sip_end_background_task(unsigned long id){
-    wake_lock_release(id);
+	wake_lock_release(id);
 }
 
 #elif !TARGET_OS_IPHONE && !defined(__APPLE__)
@@ -1612,4 +1694,3 @@ void belle_sip_end_background_task(unsigned long id){
 }
 
 #endif
-
