@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle_sip_internal.h"
 #include <bctoolbox/defs.h>
@@ -55,13 +56,9 @@ static void belle_sip_dialog_uninit(belle_sip_dialog_t *obj){
 }
 
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_dialog_t);
-BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_dialog_t)
-		BELLE_SIP_VPTR_INIT(belle_sip_dialog_t, belle_sip_object_t,TRUE),
-		(belle_sip_object_destroy_t)belle_sip_dialog_uninit,
-		NULL,
-		NULL,
-		BELLE_SIP_DEFAULT_BUFSIZE_HINT
-BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
+
+BELLE_SIP_INSTANCIATE_VPTR(belle_sip_dialog_t, belle_sip_object_t, belle_sip_dialog_uninit, NULL, NULL, TRUE);
+
 
 const char* belle_sip_dialog_state_to_string(const belle_sip_dialog_state_t state) {
 	switch(state) {
@@ -219,11 +216,13 @@ static int belle_sip_dialog_schedule_expiration(belle_sip_dialog_t *dialog, bell
 	}
 	belle_sip_message("belle_sip_dialog_schedule_expiration() dialog=%p expires_value=%i", dialog, expires_value);
 	if (expires_value == 0) return BELLE_SIP_STOP;
+
 	dialog->expiration_timer = belle_sip_main_loop_create_timeout(dialog->provider->stack->ml,
 					(belle_sip_source_func_t) belle_sip_dialog_on_expired,
 					dialog,
 					expires_value * 1000,
 					"Dialog expiration");
+
 	return BELLE_SIP_CONTINUE;
 }
 
@@ -332,8 +331,8 @@ int belle_sip_dialog_check_incoming_request_ordering(belle_sip_dialog_t *obj, be
 static int dialog_on_200Ok_timer(belle_sip_dialog_t *dialog){
 	/*reset the timer */
 	const belle_sip_timer_config_t *cfg=belle_sip_stack_get_timer_config(dialog->provider->stack);
-	unsigned int prev_timeout=belle_sip_source_get_timeout(dialog->timer_200Ok);
-	belle_sip_source_set_timeout(dialog->timer_200Ok,MIN(2*prev_timeout,(unsigned int)cfg->T2));
+	int64_t prev_timeout=belle_sip_source_get_timeout_int64(dialog->timer_200Ok);
+	belle_sip_source_set_timeout_int64(dialog->timer_200Ok, MIN(2*prev_timeout,cfg->T2));
 	belle_sip_message("Dialog sending retransmission of 200Ok");
 	belle_sip_provider_send_response(dialog->provider,dialog->last_200Ok);
 	return BELLE_SIP_CONTINUE;
@@ -358,6 +357,7 @@ static void belle_sip_dialog_init_200Ok_retrans(belle_sip_dialog_t *obj, belle_s
 		belle_sip_error("dialog [%p] already has a 200ok retransmition timer ! skipping",obj);
 		return;
 	}
+
 	obj->timer_200Ok=belle_sip_timeout_source_new((belle_sip_source_func_t)dialog_on_200Ok_timer,obj,cfg->T1);
 	belle_sip_object_set_name((belle_sip_object_t*)obj->timer_200Ok,"dialog_200Ok_timer");
 	belle_sip_main_loop_add_source(obj->provider->stack->ml,obj->timer_200Ok);
@@ -431,6 +431,103 @@ static void belle_sip_dialog_update_remote_target(belle_sip_dialog_t *obj, belle
 	}
 }
 
+static void belle_sip_dialog_update_local_cseq(belle_sip_dialog_t *obj, const char *method){
+	if (obj->local_cseq==0) obj->local_cseq=110;
+	if (strcmp(method,"ACK")!=0) obj->local_cseq++;
+}
+
+static belle_sip_request_t *belle_sip_dialog_create_prack(belle_sip_dialog_t *dialog, unsigned int rseq, unsigned int cseq, const char* method){
+	belle_sip_request_t *prack;
+	belle_sip_request_t *invite = dialog->last_out_invite;
+
+	if (!invite){
+		belle_sip_error("No INVITE to PACK.");
+		return NULL;
+	}
+
+	prack = belle_sip_dialog_create_request(dialog, "PRACK");
+
+	if (prack){
+		//RAck
+		char header[50];
+		snprintf(header,sizeof(header),"%d %d %s", rseq, cseq, method);
+		belle_sip_header_t *rack = belle_sip_header_create("RAck", header);
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(prack), rack);
+
+		//Contact
+		belle_sip_header_from_t* from_header = belle_sip_message_get_header_by_type(prack, belle_sip_header_from_t);
+		belle_sip_uri_t* req_uri = belle_sip_header_address_get_uri((belle_sip_header_address_t*)from_header);
+		belle_sip_header_contact_t* contact_header = belle_sip_header_contact_new();
+		belle_sip_header_address_set_uri((belle_sip_header_address_t*)contact_header, belle_sip_uri_new());
+		belle_sip_uri_set_user(belle_sip_header_address_get_uri((belle_sip_header_address_t*)contact_header),belle_sip_uri_get_user(req_uri));
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(prack),BELLE_SIP_HEADER(contact_header));
+
+	}
+
+	return prack;
+}
+
+static void belle_sip_dialog_send_prack(belle_sip_dialog_t *dialog, belle_sip_request_t *request){
+	belle_sip_provider_send_request(dialog->provider, request);
+}
+
+/*return 0 if message should be delivered to the next listener, otherwise, its a retransmision, just keep it*/
+static int belle_sip_dialog_process_response_100rel(belle_sip_dialog_t *obj, belle_sip_transaction_t* transaction){
+	belle_sip_response_t *resp = belle_sip_transaction_get_response(transaction);
+	belle_sip_message_t* msg = (belle_sip_message_t*)resp;
+	belle_sip_header_cseq_t *header_cseq_resp = belle_sip_message_get_header_by_type(msg, belle_sip_header_cseq_t);
+	int ret = 0;
+
+	if (header_cseq_resp == NULL){
+		belle_sip_message("Message [%p] does not contain CSeq header!", msg);
+	}
+
+	belle_sip_header_require_t* header_require = belle_sip_message_get_header_by_type(msg, belle_sip_header_require_t);
+	if (header_require){
+		belle_sip_list_t* list = belle_sip_header_require_get_require(header_require);
+		int require_100rel = 0;
+		while (list){
+			if (strcmp(list->data, "100rel") == 0){
+				require_100rel = 1;
+				break;
+			}
+			list=list->next;
+		}
+		if (require_100rel){
+			belle_sip_message("Found header Require with value 100rel in message [%p].", msg);
+			belle_sip_header_t *header_rseq = belle_sip_message_get_header(msg, "RSeq");
+			if (header_rseq){
+				const char* header_rseq_value = belle_sip_header_get_unparsed_value(header_rseq);
+				long rseq = strtol(header_rseq_value, NULL, 10);
+				unsigned int cseq_resp = belle_sip_header_cseq_get_seq_number(header_cseq_resp);
+				const char* method_resp = belle_sip_header_cseq_get_method(header_cseq_resp);
+				/*"A response is a retransmission when
+				 its dialog ID, CSeq, and RSeq match the original response". Currently we only store last acknoledged Rseq. This data is used to detect retransmitions*/
+				belle_sip_ict_t * ict = BELLE_SIP_CAST(transaction,belle_sip_ict_t);
+				if (ict->r_cseq > 0 &&  ict->r_cseq >= (unsigned int)rseq) {
+					belle_sip_warning("provisionnal response with sequence number [%ld] already acknoledged, dropping",rseq);
+					ret = -1;
+				} else {
+					/*send PRACK from callee */
+					belle_sip_request_t *prack = belle_sip_dialog_create_prack(obj, rseq, cseq_resp, method_resp);
+					if (prack){
+						/*put auth header*/
+						belle_sip_provider_add_authorization(transaction->provider,prack,transaction->last_response,NULL,NULL,NULL);
+						belle_sip_dialog_send_prack(obj, prack);
+						ict->r_cseq = rseq;
+					} else {
+						belle_sip_message("Failed to create PRACK message!");
+					}
+				}
+			}
+			else {
+				belle_sip_message("Message [%p] does not contain RSeq header! (required be with 100rel)", msg);
+			}
+		}
+	}
+	return ret;
+}
+
 /*
  * return 0 if message should be delivered to the next listener, otherwise, its a retransmision, just keep it
  * */
@@ -479,13 +576,16 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 				belle_sip_dialog_establish(obj,req,resp);
 				if (code<200){
 					set_state(obj,BELLE_SIP_DIALOG_EARLY);
+					if (!as_uas) {
+						belle_sip_dialog_process_response_100rel(obj, transaction);
+					}
 					break;
 				}/* no break  for code >200 because need to call belle_sip_dialog_establish_full*/
 			}
 			BCTBX_NO_BREAK; /*intentionally no break*/
 		case BELLE_SIP_DIALOG_EARLY:
 			/*don't terminate dialog for UPDATE*/
-			if (code>=300 && (is_invite || is_subscribe)) {
+			if (code>=300 && ((is_invite && code!=407) || is_subscribe)) {
 				/*12.3 Termination of a Dialog
 			   	   Independent of the method, if a request outside of a dialog generates
 			   	   a non-2xx final response, any early dialogs created through
@@ -513,6 +613,13 @@ int belle_sip_dialog_update(belle_sip_dialog_t *obj, belle_sip_transaction_t* tr
 				/*no response establishing the dialog, and transaction terminated (transport errors)*/
 				delete_dialog=TRUE;
 			}
+			if ((code > 100 && code < 200) && !as_uas) {
+				if (belle_sip_dialog_process_response_100rel(obj, transaction)) {
+					/*retransmition detected, dropping*/
+					ret = -1;
+					goto end;
+				}
+			}
 			break;
 		case BELLE_SIP_DIALOG_CONFIRMED:
 			if (code==481 && (is_invite || is_subscribe)) {
@@ -608,6 +715,7 @@ belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t){
 	belle_sip_header_to_t *to;
 	const char *to_tag=NULL;
 	belle_sip_header_call_id_t *call_id=belle_sip_message_get_header_by_type(t->request,belle_sip_header_call_id_t);
+	belle_sip_dialog_type_t type;
 
 	from=belle_sip_message_get_header_by_type(t->request,belle_sip_header_from_t);
 	if (from==NULL){
@@ -644,19 +752,29 @@ belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t){
 		}
 		to_tag = belle_sip_header_to_get_tag(to);
 	}
+
+	if (strcmp(belle_sip_request_get_method(t->request),"INVITE") == 0){
+		type = BELLE_SIP_DIALOG_INVITE;
+	}else if (strcmp(belle_sip_request_get_method(t->request),"SUBSCRIBE") == 0){
+		type = BELLE_SIP_DIALOG_SUBSCRIBE_NOTIFY;
+	}else{
+		belle_sip_error("belle_sip_dialog_new(): unsupported request [%s] for creating a dialog.", belle_sip_request_get_method(t->request));
+		return NULL;
+	}
+
+	if (type == BELLE_SIP_DIALOG_SUBSCRIBE_NOTIFY) {
+		belle_sip_header_expires_t *expires = belle_sip_message_get_header_by_type(t->request,belle_sip_header_expires_t);
+		if (expires && belle_sip_header_expires_get_expires(expires) < 1) {
+			belle_sip_error("belle_sip_dialog_new(): cannot create SUBSCRIBE/NOTIFY dialog with expiration <1 for transaction [%p]",t);
+			return NULL;
+		}
+	}
 	obj=belle_sip_object_new(belle_sip_dialog_t);
 	obj->terminate_on_bye=1;
 	obj->provider=t->provider;
 	obj->pending_trans_checking_enabled=1;
 	obj->call_id=(belle_sip_header_call_id_t*)belle_sip_object_ref(call_id);
-
-	if (strcmp(belle_sip_request_get_method(t->request),"INVITE") == 0){
-		obj->type = BELLE_SIP_DIALOG_INVITE;
-	}else if (strcmp(belle_sip_request_get_method(t->request),"SUBSCRIBE") == 0){
-		obj->type = BELLE_SIP_DIALOG_SUBSCRIBE_NOTIFY;
-	}else{
-		belle_sip_error("belle_sip_dialog_new(): unsupported request [%s] for creating a dialog.", belle_sip_request_get_method(t->request));
-	}
+	obj->type=type;
 
 	belle_sip_object_ref(t);
 	obj->last_transaction=t;
@@ -691,20 +809,23 @@ belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t){
 }
 
 belle_sip_request_t *belle_sip_dialog_create_ack(belle_sip_dialog_t *obj, unsigned int cseq){
-	belle_sip_header_cseq_t *cseqh;
 	belle_sip_request_t *invite=obj->last_out_invite;
 	belle_sip_request_t *ack;
+	belle_sip_header_cseq_t* cseqh;
 
 	if (!invite){
 		belle_sip_error("No INVITE to ACK.");
 		return NULL;
 	}
+
 	cseqh=belle_sip_message_get_header_by_type(invite,belle_sip_header_cseq_t);
 	if (belle_sip_header_cseq_get_seq_number(cseqh)!=cseq){
 		belle_sip_error("No INVITE with cseq %i to create ack for.",cseq);
 		return NULL;
 	}
+
 	ack=create_request(obj,"ACK",TRUE);
+	belle_sip_message_set_header(BELLE_SIP_MESSAGE(ack), BELLE_SIP_HEADER(belle_sip_header_cseq_create(cseq, "ACK")));
 /*
 22 Usage of HTTP Authentication
 22.1 Framework
@@ -800,11 +921,6 @@ belle_sip_request_t * belle_sip_dialog_create_queued_request(belle_sip_dialog_t
 	return req;
 }
 
-static void belle_sip_dialog_update_local_cseq(belle_sip_dialog_t *obj, const char *method){
-	if (obj->local_cseq==0) obj->local_cseq=110;
-	if (strcmp(method,"ACK")!=0) obj->local_cseq++;
-}
-
 belle_sip_request_t *belle_sip_dialog_create_request(belle_sip_dialog_t *obj, const char *method){
 	belle_sip_request_t *req;
 
@@ -848,7 +964,6 @@ static void copy_non_system_headers(belle_sip_header_t* header,belle_sip_request
 static belle_sip_request_t *_belle_sip_dialog_create_request_from(belle_sip_dialog_t *obj, const belle_sip_request_t *initial_req, int queued){
 	belle_sip_request_t* req;
 	const char *method=belle_sip_request_get_method(initial_req);
-	belle_sip_header_content_length_t* content_lenth;
 	belle_sip_list_t* headers;
 
 	if (queued) req=belle_sip_dialog_create_queued_request(obj,method);
@@ -856,7 +971,6 @@ static belle_sip_request_t *_belle_sip_dialog_create_request_from(belle_sip_dial
 
 	if (req==NULL) return NULL;
 
-	content_lenth = belle_sip_message_get_header_by_type(initial_req,belle_sip_header_content_length_t);
 	/*first copy non system headers*/
 	headers = belle_sip_message_get_all_headers(BELLE_SIP_MESSAGE(initial_req));
 	belle_sip_list_for_each2(headers,(void (*)(void *, void *))copy_non_system_headers,req);
@@ -869,9 +983,9 @@ static belle_sip_request_t *_belle_sip_dialog_create_request_from(belle_sip_dial
 		belle_sip_parameters_copy_parameters_from(BELLE_SIP_PARAMETERS(new_via),BELLE_SIP_PARAMETERS(orig_via));
 	}
 
-	/*copy body*/
-	if (content_lenth && belle_sip_header_content_length_get_content_length(content_lenth)>0) {
-		belle_sip_message_set_body(BELLE_SIP_MESSAGE(req),belle_sip_message_get_body(BELLE_SIP_MESSAGE(initial_req)),belle_sip_header_content_length_get_content_length(content_lenth));
+	/*copy body handler to save content enconding processing (I.E deflat)*/
+	if (belle_sip_message_get_body_handler(BELLE_SIP_MESSAGE(initial_req))) {
+		belle_sip_message_set_body_handler(BELLE_SIP_MESSAGE(req),belle_sip_message_get_body_handler(BELLE_SIP_MESSAGE(initial_req)));
 	}
 	return req;
 }
@@ -1001,8 +1115,8 @@ int _belle_sip_dialog_match(belle_sip_dialog_t *obj, const char *call_id, const
 	/*Dialog created by notify matching subscription are still in NULL state if (obj->state==BELLE_SIP_DIALOG_NULL) belle_sip_fatal("_belle_sip_dialog_match() must not be used for dialog in null state.");*/
 	dcid=belle_sip_header_call_id_get_call_id(obj->call_id);
 	return strcmp(dcid,call_id)==0
-		&& strcmp(obj->local_tag,local_tag)==0
-		&& obj->remote_tag /* handle 180 without to tag */ && strcmp(obj->remote_tag,remote_tag)==0;
+		&& (!local_tag || strcmp(obj->local_tag,local_tag)==0) //local tag is not checked if not provided
+		&& obj->remote_tag /* handle 180 without to tag */ && remote_tag && strcmp(obj->remote_tag,remote_tag)==0;
 }
 
 void belle_sip_dialog_check_ack_sent(belle_sip_dialog_t*obj){
@@ -1077,17 +1191,25 @@ If an initial SUBSCRIBE is sent on a pre-existing dialog, a matching
    subscription associated with that dialog.
    */
 
-int belle_sip_dialog_is_authorized_transaction(const belle_sip_dialog_t *dialog,const char* method) {
+int belle_sip_dialog_can_accept_request(const belle_sip_dialog_t *dialog, belle_sip_request_t *req) {
+	const char *method = belle_sip_request_get_method(req);
 	if (belle_sip_dialog_request_pending(dialog)){
 		const char* last_transaction_request;
 		if (strcasecmp(method,"BYE")==0)
 			return TRUE; /*don't reject a BYE*/
 
 		last_transaction_request = belle_sip_request_get_method(belle_sip_transaction_get_request(dialog->last_transaction));
-		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(dialog->last_transaction,belle_sip_client_transaction_t)
-			&& strcmp(last_transaction_request,"SUBSCRIBE")==0 && strcmp(method,"NOTIFY")==0){
-			/*stupid as it is, you have to accept a NOTIFY for a SUBSCRIBE for which no answer is received yet...*/
-			return TRUE;
+		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(dialog->last_transaction,belle_sip_client_transaction_t)){
+			if (strcmp(last_transaction_request,"SUBSCRIBE")==0 && strcmp(method,"NOTIFY")==0){
+				/*stupid as it may sound, you have to accept a NOTIFY for a SUBSCRIBE for which no answer is received yet.*/
+				return TRUE;
+			}else if (strcmp(last_transaction_request, "NOTIFY") == 0 && strcmp(method, "SUBSCRIBE") == 0){
+				belle_sip_header_expires_t *expire = belle_sip_message_get_header_by_type((belle_sip_message_t*)req, belle_sip_header_expires_t);
+				if (expire && belle_sip_header_expires_get_expires(expire) == 0){
+					/* Accept to receive an unSUBSCRIBE even if we have an outgoing pending NOTIFY */
+					return TRUE;
+				}
+			}
 		}
 		if (strcmp(last_transaction_request,"INVITE")==0 && (strcmp(method,"PRACK")==0 || strcmp(method,"UPDATE")==0)){
 			/*PRACK /UPDATE needs to be sent or received during reINVITEs.*/
@@ -1110,7 +1232,7 @@ static void _belle_sip_dialog_process_queue(belle_sip_dialog_t* dialog){
 
 	dialog->queued_ct=belle_sip_list_pop_front(dialog->queued_ct,(void**)&tr);
 	if (tr){
-		belle_sip_message("Dialog [%p]: sending queued request.",dialog);
+		belle_sip_message("Dialog [%p]: sending queued request [%p].",dialog,tr);
 		tr->base.sent_by_dialog_queue=TRUE;
 		belle_sip_client_transaction_send_request(tr);
 		belle_sip_object_unref(tr);
