@@ -1,21 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-	Copyright (C) 2013  Belledonne Communications SARL
-
-	This program is free software: you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation, either version 2 of the License, or
-	(at your option) any later version.
-
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
-
-	You should have received a copy of the GNU General Public License
-	along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
-
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <bctoolbox/defs.h>
 
@@ -41,6 +41,7 @@ struct belle_http_provider{
 	belle_sip_list_t *tcp_channels;
 	belle_sip_list_t *tls_channels;
 	belle_tls_crypto_config_t *crypto_config;
+	uint8_t transports; /**< a mask of enabled transports, availables: BELLE_SIP_HTTP_TRANSPORT_TCP and BELLE_SIP_HTTP_TRANSPORT_TLS */
 };
 
 #define BELLE_HTTP_REQUEST_INVOKE_LISTENER(obj,method,arg) \
@@ -53,10 +54,13 @@ static int http_channel_context_handle_authentication(belle_http_channel_context
 	const char *username=NULL;
 	const char *passwd=NULL;
 	const char *ha1=NULL;
-	char computed_ha1[33];
+	const char *algorithm=NULL;
+	char computed_ha1[65];
 	belle_sip_header_www_authenticate_t* authenticate;
-	int ret=0;
+	belle_sip_list_t* authenticate_lst;
+	belle_sip_list_t *it;
 
+	int ret=0;
 
 	if (req->auth_attempt_count>1){
 		req->auth_attempt_count=0;
@@ -66,6 +70,8 @@ static int http_channel_context_handle_authentication(belle_http_channel_context
 		belle_sip_error("Missing response for  req [%p], cannot authenticate", req);
 		return -1;
 	}
+
+	/* check we have at least one authenticate header and discard any proxy authentification */
 	if (!(authenticate = belle_sip_message_get_header_by_type(resp,belle_sip_header_www_authenticate_t))) {
 		if (belle_sip_message_get_header_by_type(resp,belle_sip_header_proxy_authenticate_t)) {
 			belle_sip_error("Proxy authentication not supported yet, cannot authenticate for resp [%p]", resp);
@@ -74,33 +80,63 @@ static int http_channel_context_handle_authentication(belle_http_channel_context
 		return -1;
 	}
 
-
-	if (strcasecmp("Digest",belle_sip_header_www_authenticate_get_scheme(authenticate)) != 0) {
-		belle_sip_error("Unsupported auth scheme [%s] in response  [%p], cannot authenticate", belle_sip_header_www_authenticate_get_scheme(authenticate),resp);
-		return -1;
-	}
-
 	/*find if username, passwd were already supplied in original request uri*/
 	if (req->orig_uri){
 		username=belle_generic_uri_get_user(req->orig_uri);
 		passwd=belle_generic_uri_get_user_password(req->orig_uri);
 	}
 
-	realm = belle_sip_header_www_authenticate_get_realm(authenticate);
-	if (!username || !passwd) {
-		ev=belle_sip_auth_event_create((belle_sip_object_t*)ctx->provider,realm,NULL);
-		BELLE_HTTP_REQUEST_INVOKE_LISTENER(req,process_auth_requested,ev);
-		username=ev->username;
-		passwd=ev->passwd;
-		ha1=ev->ha1;
-	}
-	if (!ha1 && username  && passwd) {
-		belle_sip_auth_helper_compute_ha1(username,realm,passwd, computed_ha1);
-		ha1=computed_ha1;
-	} else if (!ha1){
-		belle_sip_error("No auth info found for request [%p], cannot authenticate",req);
-		ret=-1;
+	// add from_uri to process_auth_requested event data
+	belle_sip_uri_t *from_uri = NULL;
+	belle_sip_header_t *header = belle_sip_message_get_header(BELLE_SIP_MESSAGE(req), "From");
+
+	/* loop on all authenticate headers */
+	authenticate_lst = belle_sip_list_copy(belle_sip_message_get_headers(BELLE_SIP_MESSAGE(resp),BELLE_SIP_WWW_AUTHENTICATE));
+	for (it=authenticate_lst;it!=NULL;it=it->next) {
+		authenticate=BELLE_SIP_HEADER_WWW_AUTHENTICATE(it->data);
+		const char *requested_algorithm = belle_sip_header_www_authenticate_get_algorithm(authenticate);
+		if (requested_algorithm == NULL) { // default algorithm is MD5
+			requested_algorithm = "MD5";
+		}
+		if (strcasecmp("Digest",belle_sip_header_www_authenticate_get_scheme(authenticate)) != 0) {
+			belle_sip_error("Unsupported auth scheme [%s] in response  [%p], cannot authenticate", belle_sip_header_www_authenticate_get_scheme(authenticate),resp);
+			belle_sip_list_free(authenticate_lst);
+			return -1;
+		}
+		
+		if (belle_sip_stack_check_digest_compatibility(ctx->provider->stack, authenticate) == -1) continue;
+
+		realm = belle_sip_header_www_authenticate_get_realm(authenticate);
+
+		if (header) {
+			belle_sip_header_address_t *from_address = belle_sip_header_address_parse(belle_sip_header_get_unparsed_value(header));
+			from_uri = belle_sip_header_address_get_uri(from_address);
+		} else if (username && !passwd) {
+			from_uri = belle_sip_uri_create(username,realm);
+		}
+
+		if (!username || !passwd) {
+			ev=belle_sip_auth_event_create((belle_sip_object_t*)ctx->provider,realm,from_uri);
+			belle_sip_auth_event_set_algorithm(ev, requested_algorithm);
+			BELLE_HTTP_REQUEST_INVOKE_LISTENER(req,process_auth_requested,ev);
+			username= ev->userid ? ev->userid : ev->username;
+			passwd=ev->passwd;
+			ha1=ev->ha1;
+			algorithm=ev->algorithm;
+		}
+
+		if (!ha1) {
+			if (username  && passwd) {
+				belle_sip_auth_helper_compute_ha1_for_algorithm(username,realm,passwd, computed_ha1, belle_sip_auth_define_size(requested_algorithm), requested_algorithm);
+				ha1=computed_ha1;
+				algorithm = requested_algorithm;
+				break;
+			}
+		} else {
+			break;
+		}
 	}
+	belle_sip_list_free(authenticate_lst);
 
 	if (ha1) {
 		belle_http_header_authorization_t* authorization;
@@ -112,6 +148,7 @@ static int http_channel_context_handle_authentication(belle_http_channel_context
 		belle_sip_header_authorization_set_nonce_count(BELLE_SIP_HEADER_AUTHORIZATION(authorization),1); /*we don't store nonce count for now*/
 		belle_sip_header_authorization_set_username(BELLE_SIP_HEADER_AUTHORIZATION(authorization),username);
 		belle_http_header_authorization_set_uri(authorization,belle_http_request_get_uri(req));
+		belle_sip_header_authorization_set_algorithm(BELLE_SIP_HEADER_AUTHORIZATION(authorization), algorithm);
 		if (belle_sip_auth_helper_fill_authorization(BELLE_SIP_HEADER_AUTHORIZATION(authorization),belle_http_request_get_method(req),ha1)) {
 			belle_sip_error("Cannot fill auth header for request [%p]",req);
 			if (authorization) belle_sip_object_unref(authorization);
@@ -122,6 +159,9 @@ static int http_channel_context_handle_authentication(belle_http_channel_context
 			belle_http_provider_send_request(ctx->provider,req,NULL);
 		}
 
+	} else {
+		belle_sip_error("No auth info found for request [%p], cannot authenticate",req);
+		ret=-1;
 	}
 	if (ev) belle_sip_auth_event_destroy(ev);
 	return ret;
@@ -150,6 +190,14 @@ static void http_channel_context_handle_response_headers(belle_http_channel_cont
 	}
 }
 
+static void release_background_task(belle_http_request_t *req){
+	if( req->background_task_id ){
+		belle_sip_message("HTTP request finished: ending bg task id=[%x]", req->background_task_id);
+		belle_sip_end_background_task(req->background_task_id);
+		req->background_task_id = 0;
+	}
+}
+
 static void http_channel_context_handle_response(belle_http_channel_context_t *ctx , belle_sip_channel_t *chan, belle_http_response_t *response){
 	belle_http_request_t *req=NULL;
 	belle_http_response_event_t ev={0};
@@ -179,18 +227,24 @@ static void http_channel_context_handle_response(belle_http_channel_context_t *c
 		ev.request=req;
 		ev.response=response;
 		BELLE_HTTP_REQUEST_INVOKE_LISTENER(req,process_response,&ev);
-		if( req->background_task_id ){
-			belle_sip_warning("HTTP request finished: ending bg task id=[%x]", req->background_task_id);
-			belle_sip_end_background_task(req->background_task_id);
-			req->background_task_id = 0;
-		}
+		release_background_task(req);
 	}
 	belle_sip_object_unref(req);
 }
 
+static void http_channel_notify_io_error(belle_http_channel_context_t *ctx, belle_sip_channel_t *chan, belle_http_request_t *req){
+	belle_sip_io_error_event_t ev={0};
+	/*TODO: would be nice to put the message in the event*/
+	ev.source=(belle_sip_object_t*)ctx->provider;
+	ev.host=chan->peer_cname;
+	ev.port=chan->peer_port;
+	ev.transport=belle_sip_channel_get_transport_name(chan);
+	BELLE_HTTP_REQUEST_INVOKE_LISTENER(req,process_io_error,&ev);
+	release_background_task(req);
+}
+
 static void http_channel_context_handle_io_error(belle_http_channel_context_t *ctx, belle_sip_channel_t *chan){
 	belle_http_request_t *req=NULL;
-	belle_sip_io_error_event_t ev={0};
 	belle_sip_list_t *elem;
 
 	/*if the error happens before attempting to send the message, the pending_requests is empty*/
@@ -199,20 +253,35 @@ static void http_channel_context_handle_io_error(belle_http_channel_context_t *c
 	/*pop the requests for which this error is reported*/
 	for(;elem!=NULL;elem=elem->next){
 		req=(belle_http_request_t *)elem->data;
-		/*else notify the app about the response received*/
-		/*TODO: would be nice to put the message in the event*/
-		ev.source=(belle_sip_object_t*)ctx->provider;
-		ev.host=chan->peer_cname;
-		ev.port=chan->peer_port;
-		ev.transport=belle_sip_channel_get_transport_name(chan);
-		BELLE_HTTP_REQUEST_INVOKE_LISTENER(req,process_io_error,&ev);
-		if( req->background_task_id ){
-			belle_sip_warning("IO Error on HTTP request: ending bg task id=[%x]", req->background_task_id);
-			belle_sip_end_background_task(req->background_task_id);
-			req->background_task_id = 0;
+		http_channel_notify_io_error(ctx, chan, req);
+	}
+}
+
+static void http_channel_context_handle_disconnection(belle_http_channel_context_t *ctx, belle_sip_channel_t *chan){
+	belle_sip_list_t *elem;
+	belle_http_request_t *req=NULL;
+	belle_sip_list_t *to_be_resubmitted = NULL;
+
+	for (elem = chan->outgoing_messages ; elem != NULL ; elem = elem->next){
+		to_be_resubmitted = bctbx_list_append(to_be_resubmitted, elem->data);
+	}
+	for (elem = ctx->pending_requests ; elem != NULL ; elem = elem->next){
+		if (bctbx_list_find(to_be_resubmitted, elem->data) == NULL){
+			to_be_resubmitted = bctbx_list_append(to_be_resubmitted, elem->data);
 		}
 	}
-
+	for (elem = to_be_resubmitted; elem != NULL; elem = elem->next){
+		req = (belle_http_request_t *)elem->data;
+		if (req->resubmitted == 0){
+			req->resubmitted = 1;
+			belle_sip_message("Resubmitting http request.");
+			belle_http_provider_send_request(ctx->provider, req, NULL /*keep the listener as it is already*/);
+		}else{
+			belle_sip_warning("Http request has already been resubmitted after a server disconnection. Treating this as an error now.");
+			http_channel_notify_io_error(ctx, chan, req);
+		}
+	}
+	bctbx_list_free(to_be_resubmitted);
 }
 
 /* we are called here by the channel when receiving a message for which a body is expected.
@@ -268,10 +337,19 @@ static void channel_state_changed(belle_sip_channel_listener_t *obj, belle_sip_c
 			break;
 		case BELLE_SIP_CHANNEL_ERROR:
 			http_channel_context_handle_io_error(ctx, chan);
-			BCTBX_NO_BREAK; /*intentionally no break*/
-		case BELLE_SIP_CHANNEL_DISCONNECTED:
-			if (!chan->force_close) provider_remove_channel(ctx->provider,chan);
+			if (!chan->force_close) {
+				provider_remove_channel(ctx->provider,chan);
+			}
 			break;
+		case BELLE_SIP_CHANNEL_DISCONNECTED:
+			if (!chan->force_close) {
+				/* This may happen while there is a pending requests, typically when the client decides to send a new request while the server
+				* decides to close the connection because of inactivity timeout. Handle this case by re-submitting the pending request(s). */
+				http_channel_context_handle_disconnection(ctx, chan);
+				provider_remove_channel(ctx->provider,chan);
+			}
+			break;
+
 	}
 }
 
@@ -298,7 +376,7 @@ belle_http_channel_context_t * belle_http_channel_context_new(belle_sip_channel_
 
 int belle_http_channel_is_busy(belle_sip_channel_t *obj) {
 	belle_sip_list_t *it;
-	if (obj->incoming_messages != NULL || obj->outgoing_messages != NULL) {
+	if (obj->outgoing_messages != NULL) {
 		return 1;
 	}
 	/*fixme, a litle bit intrusive*/
@@ -335,12 +413,13 @@ static void http_provider_uninit(belle_http_provider_t *obj){
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_http_provider_t);
 BELLE_SIP_INSTANCIATE_VPTR(belle_http_provider_t,belle_sip_object_t,http_provider_uninit,NULL,NULL,FALSE);
 
-belle_http_provider_t *belle_http_provider_new(belle_sip_stack_t *s, const char *bind_ip){
+belle_http_provider_t *belle_http_provider_new(belle_sip_stack_t *s, const char *bind_ip, const uint8_t transports){
 	belle_http_provider_t *p=belle_sip_object_new(belle_http_provider_t);
 	p->stack=s;
 	p->bind_ip=belle_sip_strdup(bind_ip);
 	p->ai_family=strchr(p->bind_ip,':') ? AF_INET6 : AF_INET;
 	p->crypto_config=belle_tls_crypto_config_new();
+	p->transports=transports;
 	return p;
 }
 
@@ -428,9 +507,25 @@ int belle_http_provider_send_request(belle_http_provider_t *obj, belle_http_requ
 	}
 	if (!chan){
 		if (strcasecmp(hop->transport,"tcp")==0){
-			chan=belle_sip_stream_channel_new_client(obj->stack,obj->bind_ip,0,hop->cname,hop->host,hop->port);
+			if ((obj->transports & BELLE_SIP_HTTP_TRANSPORT_TCP)==0) {
+				char *uri_as_string = belle_generic_uri_to_string(req->orig_uri ? req->orig_uri : req->req_uri);
+				belle_sip_error("%s: cannot process request to [%s] as this provider is not configured to process http requests",
+						__FUNCTION__, uri_as_string);
+				belle_sip_free(uri_as_string);
+				belle_sip_object_unref(hop);
+				return -1;
+			}
+			chan=belle_sip_stream_channel_new_client(obj->stack,obj->bind_ip,0,hop->cname,hop->host,hop->port, FALSE);
 		} else if (strcasecmp(hop->transport,"tls")==0){
-			chan=belle_sip_channel_new_tls(obj->stack,obj->crypto_config,obj->bind_ip,0,hop->cname,hop->host,hop->port);
+			if ((obj->transports & BELLE_SIP_HTTP_TRANSPORT_TLS)==0) {
+				char *uri_as_string = belle_generic_uri_to_string(req->orig_uri ? req->orig_uri : req->req_uri);
+				belle_sip_error("%s: cannot process request to [%s] as this provider is not configured to process https requests",
+						__FUNCTION__, uri_as_string);
+				belle_sip_free(uri_as_string);
+				belle_sip_object_unref(hop);
+				return -1;
+			}
+			chan=belle_sip_channel_new_tls(obj->stack,obj->crypto_config,obj->bind_ip,0,hop->cname,hop->host,hop->port, FALSE);
 		}
 
 		if (!chan){
