@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle_sip_internal.h"
 
@@ -71,9 +72,9 @@ static void transaction_background_task_ended(belle_sip_transaction_t *obj){
 static void transaction_begin_background_task(belle_sip_transaction_t *obj){
 	if (obj->bg_task_id==0){
 		char *transaction = bctbx_strdup_printf("belle-sip transaction(%p)", obj);
-        obj->bg_task_id=belle_sip_begin_background_task(transaction,(void (*)(void*))transaction_background_task_ended, obj);
-        if (obj->bg_task_id) belle_sip_message("transaction [%p]: starting transaction background task with id=[%lx].",obj,obj->bg_task_id);
-        bctbx_free(transaction);
+		obj->bg_task_id=belle_sip_begin_background_task(transaction,(void (*)(void*))transaction_background_task_ended, obj);
+		if (obj->bg_task_id) belle_sip_message("transaction [%p]: starting transaction background task with id=[%lx].",obj,obj->bg_task_id);
+		bctbx_free(transaction);
 	}
 }
 
@@ -116,10 +117,56 @@ static int client_transaction_on_call_repair_timer(belle_sip_transaction_t *t) {
 	return BELLE_SIP_STOP;
 }
 
+static void transaction_handle_channel_error(belle_sip_transaction_t *t, belle_sip_channel_t *chan){
+	belle_sip_io_error_event_t ev;
+	belle_sip_transaction_state_t tr_state=belle_sip_transaction_get_state((belle_sip_transaction_t*)t);
+	const belle_sip_timer_config_t *timercfg = NULL;
+
+	belle_sip_object_ref(t);  /*take a ref in the case where the app calls belle_sip_transaction_terminate() within the listener*/
+
+	ev.transport=belle_sip_channel_get_transport_name(chan);
+	ev.source=BELLE_SIP_OBJECT(t);
+	ev.port=chan->peer_port;
+	ev.host=chan->peer_name;
+
+	if ( tr_state!=BELLE_SIP_TRANSACTION_COMPLETED
+		&& tr_state!=BELLE_SIP_TRANSACTION_CONFIRMED
+		&& tr_state!=BELLE_SIP_TRANSACTION_ACCEPTED
+		&& tr_state!=BELLE_SIP_TRANSACTION_TERMINATED) {
+		BELLE_SIP_PROVIDER_INVOKE_LISTENERS_FOR_TRANSACTION(((belle_sip_transaction_t*)t),process_io_error,&ev);
+	}
+	if (t->timed_out) {
+		notify_timeout((belle_sip_transaction_t*)t);
+	} else {
+		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ist_t) || BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ict_t)) {
+			timercfg = belle_sip_transaction_get_timer_config(t);
+			if (t->call_repair_timer) {
+				belle_sip_transaction_stop_timer(t, t->call_repair_timer);
+				belle_sip_object_unref(t->call_repair_timer);
+				t->call_repair_timer = NULL;
+			}
+		}
+	}
+	if (!t->timed_out && belle_sip_transaction_state_is_transient(t->state) && BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ist_t)) {
+		t->call_repair_timer = belle_sip_timeout_source_new((belle_sip_source_func_t)server_transaction_on_call_repair_timer, t, 32 * timercfg->T1);
+		belle_sip_transaction_start_timer(t, t->call_repair_timer);
+	} else if (!t->timed_out && belle_sip_transaction_state_is_transient(t->state) && BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ict_t)) {
+		t->call_repair_timer = belle_sip_timeout_source_new((belle_sip_source_func_t)client_transaction_on_call_repair_timer, t, 32 * timercfg->T1);
+		belle_sip_transaction_start_timer(t, t->call_repair_timer);
+	} else {
+		belle_sip_transaction_terminate(t);
+	}
+
+	if (t->channel){
+		belle_sip_channel_remove_listener(t->channel, (belle_sip_channel_listener_t*)t);
+		belle_sip_object_unref(t->channel);
+		t->channel = NULL;
+	}
+	belle_sip_object_unref(t);
+}
+
 static void on_channel_state_changed(belle_sip_channel_listener_t *l, belle_sip_channel_t *chan, belle_sip_channel_state_t state){
 	belle_sip_transaction_t *t=(belle_sip_transaction_t*)l;
-	belle_sip_io_error_event_t ev;
-	const belle_sip_timer_config_t *timercfg = NULL;
 	belle_sip_transaction_state_t tr_state=belle_sip_transaction_get_state((belle_sip_transaction_t*)t);
 
 	belle_sip_message("transaction [%p] channel state changed to [%s]"
@@ -134,51 +181,20 @@ static void on_channel_state_changed(belle_sip_channel_listener_t *l, belle_sip_
 		break;
 		case BELLE_SIP_CHANNEL_DISCONNECTED:
 		case BELLE_SIP_CHANNEL_ERROR:
-			belle_sip_object_ref(t);  /*take a ref in the case where the app calls belle_sip_transaction_terminate() within the listener*/
-			ev.transport=belle_sip_channel_get_transport_name(chan);
-			ev.source=BELLE_SIP_OBJECT(t);
-			ev.port=chan->peer_port;
-			ev.host=chan->peer_name;
-			if ( tr_state!=BELLE_SIP_TRANSACTION_COMPLETED
-				&& tr_state!=BELLE_SIP_TRANSACTION_CONFIRMED
-				&& tr_state!=BELLE_SIP_TRANSACTION_ACCEPTED
-				&& tr_state!=BELLE_SIP_TRANSACTION_TERMINATED) {
-				BELLE_SIP_PROVIDER_INVOKE_LISTENERS_FOR_TRANSACTION(((belle_sip_transaction_t*)t),process_io_error,&ev);
-			}
-			if (t->timed_out) {
-				notify_timeout((belle_sip_transaction_t*)t);
-			} else {
-				if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ist_t) || BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ict_t)) {
-					timercfg = belle_sip_transaction_get_timer_config(t);
-					if (t->call_repair_timer) {
-						belle_sip_transaction_stop_timer(t, t->call_repair_timer);
-						belle_sip_object_unref(t->call_repair_timer);
-						t->call_repair_timer = NULL;
-					}
-				}
-			}
-
-			if (!t->timed_out && belle_sip_transaction_state_is_transient(t->state) && BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ist_t)) {
-				t->call_repair_timer = belle_sip_timeout_source_new((belle_sip_source_func_t)server_transaction_on_call_repair_timer, t, 32 * timercfg->T1);
-				belle_sip_transaction_start_timer(t, t->call_repair_timer);
-			} else if (!t->timed_out && belle_sip_transaction_state_is_transient(t->state) && BELLE_SIP_OBJECT_IS_INSTANCE_OF(t, belle_sip_ict_t)) {
-				t->call_repair_timer = belle_sip_timeout_source_new((belle_sip_source_func_t)client_transaction_on_call_repair_timer, t, 32 * timercfg->T1);
-				belle_sip_transaction_start_timer(t, t->call_repair_timer);
-			} else {
-				belle_sip_transaction_terminate(t);
-			}
-			if (t->channel){
-				belle_sip_channel_remove_listener(t->channel, l);
-				belle_sip_object_unref(t->channel);
-				t->channel = NULL;
-			}
-			belle_sip_object_unref(t);
-
+			transaction_handle_channel_error(t, chan);
+		break;
+		case BELLE_SIP_CHANNEL_RETRY:
+			/* Going to the retry state means that the transaction will loose its transport, and get a new one.
+			 * While it has no matter if the request was not yet sent, it is a fatal error otherwise.
+			 */
+			if (tr_state != BELLE_SIP_TRANSACTION_INIT)
+				transaction_handle_channel_error(t, chan);
 		break;
 		default:
 			/*ignored*/
 		break;
 	}
+	/* NO MORE INSTRUCTIONS - transaction_handle_channel_error() may delete both the channel and the transaction.*/
 }
 
 BELLE_SIP_IMPLEMENT_INTERFACE_BEGIN(belle_sip_transaction_t,belle_sip_channel_listener_t)
@@ -207,12 +223,12 @@ void *belle_sip_transaction_get_application_data_internal(const belle_sip_transa
 }
 
 void *belle_sip_transaction_get_application_data(const belle_sip_transaction_t *t){
-	if (t->is_internal) {
-		belle_sip_error("belle_sip_transaction_get_application_data should not be used on internal transaction [%p]",t);
-		return NULL;
-	} else {
+	//if (t->is_internal) {
+	//	belle_sip_error("belle_sip_transaction_get_application_data should not be used on internal transaction [%p]",t);
+	//	return NULL;
+	//} else {
 		return belle_sip_transaction_get_application_data_internal(t);
-	};
+	//};
 }
 
 void belle_sip_transaction_set_application_data(belle_sip_transaction_t *t, void *data){
@@ -272,20 +288,20 @@ belle_sip_response_t *belle_sip_transaction_get_response(const belle_sip_transac
 }
 
 void belle_sip_transaction_notify_timeout(belle_sip_transaction_t *t){
-	/*report the channel as possibly dead. If an alternate IP can be tryied, the channel will notify us with the RETRY state.
+	/* Report the channel as possibly dead. If an alternate IP can be tryied, the channel will notify us with the RETRY state.
 	 * Otherwise it will report the error.
 	 * We limit this dead channel reporting to REGISTER transactions, who are unlikely to be unresponded.
 	**/
-	belle_sip_object_ref(t);  /*take a ref in the case where the app calls belle_sip_transaction_terminate() within the timeout listener*/
-	if (strcmp(belle_sip_request_get_method(t->request),"REGISTER")==0){
-		if ( belle_sip_channel_notify_timeout(t->channel)==TRUE){
+	if (strcmp(belle_sip_request_get_method(t->request), "REGISTER") == 0){
+		if (belle_sip_channel_notify_timeout(t->channel)==TRUE){
 			belle_sip_warning("Transaction [%p] has timeout, reported to channel.",t);
-			t->timed_out=TRUE;
+			t->timed_out = TRUE;
+			return;
 		}
-	}else {
-		notify_timeout(t);
-		belle_sip_transaction_terminate(t);
 	}
+	belle_sip_object_ref(t);  /*take a ref in the case where the app calls belle_sip_transaction_terminate() within the timeout listener*/
+	notify_timeout(t);
+	belle_sip_transaction_terminate(t);
 	belle_sip_object_unref(t);
 }
 
@@ -321,6 +337,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_server_transaction_t)
 			(belle_sip_object_destroy_t) server_transaction_destroy,
 			NULL,
 			NULL,
+			(belle_sip_object_on_first_ref_t) NULL,
+			(belle_sip_object_on_last_ref_t) NULL,
 			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
 		NULL
@@ -366,11 +384,11 @@ void belle_sip_server_transaction_send_response(belle_sip_server_transaction_t *
 			//add a random to tag
 			belle_sip_header_to_set_tag(to,t->to_tag);
 		}
-		/*12.1 Creation of a Dialog
-		   Dialogs are created through the generation of non-failure responses
-		   to requests with specific methods.  Within this specification, only
-		   2xx and 101-199 responses with a To tag, where the request was
-		   INVITE, will establish a dialog.*/
+		// 12.1 Creation of a Dialog
+		// Dialogs are created through the generation of non-failure responses
+		// to requests with specific methods.  Within this specification, only
+		// 2xx and 101-199 responses with a To tag, where the request was
+		// INVITE, will establish a dialog.
 		if (dialog && status_code>100 && status_code<300){
 			belle_sip_response_fill_for_dialog(resp,base->request);
 		}
@@ -432,21 +450,21 @@ belle_sip_request_t * belle_sip_client_transaction_create_cancel(belle_sip_clien
 	}
 	if ((t->base.state != BELLE_SIP_TRANSACTION_PROCEEDING) && (t->base.state != BELLE_SIP_TRANSACTION_CALLING)) {
 		belle_sip_error("belle_sip_client_transaction_create_cancel() can only be used in state PROCEEDING or CALLING"
-		               " but current transaction state is %s",belle_sip_transaction_state_to_string(t->base.state));
+									 " but current transaction state is %s",belle_sip_transaction_state_to_string(t->base.state));
 		return NULL;
 	}
 	req=belle_sip_request_new();
 	belle_sip_request_set_method(req,"CANCEL");
-/*	9.1 Client Behavior
-	   Since requests other than INVITE are responded to immediately,
-	   sending a CANCEL for a non-INVITE request would always create a
-	   race condition.
-	   The following procedures are used to construct a CANCEL request.  The
-	   Request-URI, Call-ID, To, the numeric part of CSeq, and From header
-	   fields in the CANCEL request MUST be identical to those in the
-	   request being cancelled, including tags.  A CANCEL constructed by a
-	   client MUST have only a single Via header field value matching the
-	   top Via value in the request being cancelled.*/
+	// 9.1 Client Behavior
+	// Since requests other than INVITE are responded to immediately,
+	// sending a CANCEL for a non-INVITE request would always create a
+	// race condition.
+	// The following procedures are used to construct a CANCEL request.  The
+	// Request-URI, Call-ID, To, the numeric part of CSeq, and From header
+	// fields in the CANCEL request MUST be identical to those in the
+	// request being cancelled, including tags.  A CANCEL constructed by a
+	// client MUST have only a single Via header field value matching the
+	// top Via value in the request being cancelled.
 	belle_sip_request_set_uri(req,(belle_sip_uri_t*)belle_sip_object_clone((belle_sip_object_t*)belle_sip_request_get_uri((belle_sip_request_t*)orig)));
 	belle_sip_util_copy_headers(orig,(belle_sip_message_t*)req,"via",FALSE);
 	belle_sip_util_copy_headers(orig,(belle_sip_message_t*)req,"call-id",FALSE);
@@ -454,10 +472,13 @@ belle_sip_request_t * belle_sip_client_transaction_create_cancel(belle_sip_clien
 	belle_sip_util_copy_headers(orig,(belle_sip_message_t*)req,"to",FALSE);
 	belle_sip_util_copy_headers(orig,(belle_sip_message_t*)req,"route",TRUE);
 	belle_sip_util_copy_headers(orig,(belle_sip_message_t*)req,BELLE_SIP_MAX_FORWARDS,FALSE);
-	belle_sip_message_add_header((belle_sip_message_t*)req,
+	belle_sip_message_add_header(
+		(belle_sip_message_t*)req,
 		(belle_sip_header_t*)belle_sip_header_cseq_create(
 			belle_sip_header_cseq_get_seq_number((belle_sip_header_cseq_t*)belle_sip_message_get_header(orig,"cseq")),
-		    "CANCEL"));
+			"CANCEL"
+		)
+	);
 	return req;
 }
 
@@ -553,11 +574,17 @@ int belle_sip_client_transaction_send_request_to(belle_sip_client_transaction_t
 	return result;
 }
 
-static unsigned int should_dialog_be_created(belle_sip_client_transaction_t *t, belle_sip_response_t *resp){
+/**
+ * The client transaction may create a dialog in NULL state upon receiving their first provisionnal response (ie without to-tag defined) for convenience of upper layer.
+ * The dialog will transition to early state as soon as a new provisional response with to-tag is received.
+**/
+static unsigned int should_dialog_be_created(belle_sip_client_transaction_t *t, belle_sip_response_t *resp, int to_tag_required){
 	belle_sip_request_t* req = belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(t));
 	const char* method = belle_sip_request_get_method(req);
 	int status_code = belle_sip_response_get_status_code(resp);
-	return status_code>=101 && status_code<300 && (strcmp(method,"INVITE")==0 || strcmp(method,"SUBSCRIBE")==0);
+	belle_sip_header_to_t *to = belle_sip_message_get_header_by_type((belle_sip_message_t*)resp, belle_sip_header_to_t);
+	return status_code>=101 && status_code<300 && (strcmp(method,"INVITE")==0 || strcmp(method,"SUBSCRIBE")==0)
+		&& to && (!to_tag_required || belle_sip_header_to_get_tag(to));
 }
 
 void belle_sip_client_transaction_notify_response(belle_sip_client_transaction_t *t, belle_sip_response_t *resp){
@@ -578,13 +605,13 @@ void belle_sip_client_transaction_notify_response(belle_sip_client_transaction_t
 			/*make sure this response matches the current dialog, or creates a new one*/
 			if (!belle_sip_dialog_match(dialog,(belle_sip_message_t*)resp,FALSE)){
 				dialog=belle_sip_provider_find_dialog_from_message(t->base.provider,(belle_sip_message_t*)resp,FALSE);
-				if (!dialog){
+				if (!dialog && should_dialog_be_created(t, resp, TRUE)){
 					dialog=belle_sip_provider_create_dialog_internal(t->base.provider,BELLE_SIP_TRANSACTION(t),FALSE);/*belle_sip_dialog_new(base);*/
-					belle_sip_message("Handling response creating a new dialog !");
+					belle_sip_message("Handling response creating a new dialog!");
 				}
 			}
 		}
-	} else if (should_dialog_be_created(t,resp)) {
+	} else if (should_dialog_be_created(t, resp, FALSE)) {
 		dialog=belle_sip_provider_create_dialog_internal(t->base.provider,BELLE_SIP_TRANSACTION(t),FALSE);
 	}
 
@@ -603,9 +630,21 @@ void belle_sip_client_transaction_notify_response(belle_sip_client_transaction_t
 	if (dialog && status_code>=200 && status_code<300 && strcmp(method,"INVITE")==0){
 		belle_sip_dialog_check_ack_sent(dialog);
 	}
-	/*report a server having internal errors for REGISTER to the channel, in order to go to a fallback IP*/
-	if (status_code == 500 && strcmp(method,"REGISTER") == 0){
-		belle_sip_channel_notify_server_error(base->channel);
+	/*
+	 * Report a server having internal errors for REGISTER to the channel, in order to go to a fallback IP.
+	 * Why for only REGISTER ? because we are sure that the response comes from the server we are connected to.
+	 * For others (INVITE, SUBSCRIBE), the response may come from an intermediary server, or the final client.
+	 * It would be a bad idea to close the connection and fallback to another node in these cases.
+	 */
+	if (strcmp(method,"REGISTER") == 0){
+		switch (status_code){
+			case 500: /* Internal error */
+			case 503: /* Service unavailable */
+				belle_sip_channel_notify_server_error(base->channel);
+			break;
+			default:
+			break;
+		}
 	}
 }
 
@@ -633,11 +672,14 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_client_transaction_t)
 			(belle_sip_object_destroy_t)client_transaction_destroy,
 			NULL,
 			NULL,
+			(belle_sip_object_on_first_ref_t) NULL,
+			(belle_sip_object_on_last_ref_t) NULL,
 			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
 		NULL
 	},
 	NULL,
+	NULL,
 	NULL
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
@@ -700,8 +742,10 @@ belle_sip_request_t* belle_sip_client_transaction_create_authenticated_request(b
 
  */
 
-int belle_sip_client_transaction_is_notify_matching_pending_subscribe(belle_sip_client_transaction_t *trans
-																	  , belle_sip_request_t *notify) {
+int belle_sip_client_transaction_is_notify_matching_pending_subscribe(
+	belle_sip_client_transaction_t *trans,
+	belle_sip_request_t *notify
+) {
 	belle_sip_request_t *subscription;
 	belle_sip_header_event_t *sub_event, *notif_event;
 	belle_sip_header_call_id_t *sub_call_id, *notif_call_id;
@@ -712,7 +756,8 @@ int belle_sip_client_transaction_is_notify_matching_pending_subscribe(belle_sip_
 
 
 	if (strcmp("NOTIFY",belle_sip_request_get_method(notify)) != 0) {
-		belle_sip_error("belle_sip_client_transaction_is_notify_matching_pending_subscribe for dialog [%p], requires a notify request",notify);
+		belle_sip_error("belle_sip_client_transaction_is_notify_matching_pending_subscribe for request [%p], requires a notify request",notify);
+		return 0;
 	}
 
 	subscription = belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(trans));
@@ -732,6 +777,8 @@ int belle_sip_client_transaction_is_notify_matching_pending_subscribe(belle_sip_
 	return strcmp(belle_sip_header_call_id_get_call_id(sub_call_id),belle_sip_header_call_id_get_call_id(notif_call_id))==0
 	&& sub_from_tag && notif_to_tag && strcmp(sub_from_tag,notif_to_tag)==0
 	&& strcasecmp(belle_sip_header_event_get_package_name(sub_event),belle_sip_header_event_get_package_name(notif_event))==0;
-
 }
 
+void belle_sip_client_transaction_stop_retransmissions(belle_sip_client_transaction_t *t){
+	BELLE_SIP_OBJECT_VPTR(t,belle_sip_client_transaction_t)->stop_retransmissions(t);
+}
