@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle_sip_internal.h"
 #include "belle-sip/mainloop.h"
@@ -93,7 +94,7 @@ void stream_channel_close(belle_sip_stream_channel_t *obj){
 #if TARGET_OS_IPHONE
 static void stream_channel_enable_ios_background_mode(belle_sip_stream_channel_t *obj){
 	int sock=belle_sip_source_get_socket((belle_sip_source_t*)obj);
-	
+
 	CFStreamCreatePairWithSocket(kCFAllocatorDefault, sock, &obj->read_stream, &obj->write_stream);
 	if (obj->read_stream){
 		if (!CFReadStreamSetProperty (obj->read_stream, kCFStreamNetworkServiceType, kCFStreamNetworkServiceTypeVoIP)){
@@ -105,11 +106,11 @@ static void stream_channel_enable_ios_background_mode(belle_sip_stream_channel_t
 			belle_sip_warning("CFReadStreamSetProperty() could not set VoIP service type on write stream.");
 		}
 	}else belle_sip_warning("CFStreamCreatePairWithSocket() could not create the write stream.");
-	
+
 	if (!CFReadStreamOpen (obj->read_stream)) {
 		belle_sip_warning("CFReadStreamOpen() failed.");
 	}
-	
+
 	if (!CFWriteStreamOpen (obj->write_stream)) {
 		belle_sip_warning("CFWriteStreamOpen() failed.");
 	}
@@ -121,25 +122,44 @@ int stream_channel_connect(belle_sip_stream_channel_t *obj, const struct addrinf
 	int err;
 	int tmp;
 	belle_sip_socket_t sock;
+	belle_sip_stack_t *stack = obj->base.stack;
 	tmp=1;
-	
+
 	obj->base.ai_family=ai->ai_family;
 	sock=bctbx_socket(ai->ai_family, SOCK_STREAM, IPPROTO_TCP);
-	
+
 	if (sock==(belle_sip_socket_t)-1){
 		belle_sip_error("Could not create socket: %s",belle_sip_get_socket_error_string());
 		return -1;
 	}
-	
+	tmp = 1;
+	err=bctbx_setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,(char*)&tmp,sizeof(tmp));
+	if (err!=0){
+		belle_sip_error("bctbx_setsockopt SO_REUSEADDR failed: [%s]",belle_sip_get_socket_error_string());
+	}
+
+	if (stack->test_bind_port != 0){
+		struct addrinfo *bind_ai = bctbx_ip_address_to_addrinfo(ai->ai_family, SOCK_STREAM, ai->ai_family == AF_INET6 ? "::0" : "0.0.0.0", stack->test_bind_port);
+
+		err = bctbx_bind(sock, bind_ai->ai_addr, (socklen_t)bind_ai->ai_addrlen);
+		if (err != 0){
+			belle_sip_error("bctbx_bind failed: [%s]",belle_sip_get_socket_error_string());
+			belle_sip_close_socket(sock);
+			return -1;
+		}else bctbx_message("bind() on port [%i] successful", stack->test_bind_port);
+		bctbx_freeaddrinfo(bind_ai);
+	}
+
+	tmp = 1;
 	err=bctbx_setsockopt(sock, IPPROTO_TCP, TCP_NODELAY,(char*)&tmp,sizeof(tmp));
 	if (err!=0){
 		belle_sip_error("bctbx_setsockopt TCP_NODELAY failed: [%s]",belle_sip_get_socket_error_string());
 	}
 	belle_sip_socket_set_nonblocking(sock);
-	if (ai->ai_family==AF_INET6){
+	if (ai->ai_family==AF_INET6 && stack->test_bind_port == 0){
 		belle_sip_socket_enable_dual_stack(sock);
 	}
-	
+
 	err = bctbx_connect(sock,ai->ai_addr,(socklen_t)ai->ai_addrlen);
 	if (err != 0 && get_socket_error()!=BELLESIP_EINPROGRESS && get_socket_error()!=BELLESIP_EWOULDBLOCK) {
 		belle_sip_error("stream connect failed %s",belle_sip_get_socket_error_string());
@@ -148,7 +168,7 @@ int stream_channel_connect(belle_sip_stream_channel_t *obj, const struct addrinf
 	}
 	belle_sip_channel_set_socket((belle_sip_channel_t*)obj,sock,(belle_sip_source_func_t)stream_channel_process_data);
 	belle_sip_source_set_events((belle_sip_source_t*)obj,BELLE_SIP_EVENT_READ|BELLE_SIP_EVENT_WRITE|BELLE_SIP_EVENT_ERROR);
-	belle_sip_source_set_timeout((belle_sip_source_t*)obj,belle_sip_stack_get_transport_timeout(obj->base.stack));
+	belle_sip_source_set_timeout_int64((belle_sip_source_t*)obj,belle_sip_stack_get_transport_timeout(obj->base.stack));
 	belle_sip_main_loop_add_source(obj->base.stack->ml,(belle_sip_source_t*)obj);
 	return 0;
 }
@@ -162,6 +182,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_stream_channel_t)
 			(belle_sip_object_destroy_t)stream_channel_uninit,
 			NULL,
 			NULL,
+			(belle_sip_object_on_first_ref_t) NULL,
+			(belle_sip_object_on_last_ref_t) NULL,
 			BELLE_SIP_DEFAULT_BUFSIZE_HINT
 		},
 		"TCP",
@@ -177,7 +199,7 @@ int finalize_stream_connection(belle_sip_stream_channel_t *obj, unsigned int rev
 	int err, errnum;
 	socklen_t optlen=sizeof(errnum);
 	belle_sip_socket_t sock=belle_sip_source_get_socket((belle_sip_source_t*)obj);
-	
+
 	if (revents==BELLE_SIP_EVENT_TIMEOUT){
 		belle_sip_warning("channel [%p]: user-defined transport timeout.",obj);
 		return -1;
@@ -186,7 +208,7 @@ int finalize_stream_connection(belle_sip_stream_channel_t *obj, unsigned int rev
 		belle_sip_warning("channel [%p]: getting unexpected event while connecting",obj);
 		return -1;
 	}
-	
+
 	err=bctbx_getsockopt(sock,SOL_SOCKET,SO_ERROR,(void*)&errnum,&optlen);
 	if (err!=0){
 		belle_sip_error("Failed to retrieve connection status for fd [%i]: cause [%s]",sock,belle_sip_get_socket_error_string());
@@ -200,7 +222,9 @@ int finalize_stream_connection(belle_sip_stream_channel_t *obj, unsigned int rev
 				return -1;
 			}
 #if TARGET_OS_IPHONE
-			stream_channel_enable_ios_background_mode(obj);
+			if (belle_sip_get_ios_device_major_version() < 16) { // Causes crash on app built for iOS16 with Xcode 14
+				stream_channel_enable_ios_background_mode(obj);
+			}
 #endif
 			if (obj->base.stack->dscp && obj->base.lp){
 				/*apply dscp only to channel belonging to a SIP listening point*/
@@ -222,7 +246,7 @@ static int stream_channel_process_data(belle_sip_stream_channel_t *obj,unsigned
 	belle_sip_channel_t *base=(belle_sip_channel_t*)obj;
 
 	/*belle_sip_message("TCP channel process_data");*/
-	
+
 	if (state == BELLE_SIP_CHANNEL_CONNECTING ) {
 		if (finalize_stream_connection(obj,revents,(struct sockaddr*)&ss,&addrlen)) {
 			belle_sip_error("Cannot connect to [%s://%s:%i]",belle_sip_channel_get_transport_name(base),base->peer_name,base->peer_port);
@@ -230,26 +254,28 @@ static int stream_channel_process_data(belle_sip_stream_channel_t *obj,unsigned
 			return BELLE_SIP_STOP;
 		}
 		belle_sip_source_set_events((belle_sip_source_t*)obj,BELLE_SIP_EVENT_READ|BELLE_SIP_EVENT_ERROR);
-		belle_sip_source_set_timeout((belle_sip_source_t*)obj,-1);
+		belle_sip_source_set_timeout_int64((belle_sip_source_t*)obj,-1);
 		belle_sip_channel_set_ready(base,(struct sockaddr*)&ss,addrlen);
 		return BELLE_SIP_CONTINUE;
-	} else if (state == BELLE_SIP_CHANNEL_READY) {
+	} else if (state == BELLE_SIP_CHANNEL_READY || state == BELLE_SIP_CHANNEL_RES_IN_PROGRESS) {
+		/* Because of DNS TTL timeout, the channel may enter the RES_IN_PROGRESS state temporarily while being connected.*/
 		return belle_sip_channel_process_data(base,revents);
 	} else {
-		belle_sip_warning("Unexpected event [%i], in state [%s] for channel [%p]",revents,belle_sip_channel_state_to_string(state),obj);
+		belle_sip_error("Unexpected event [%i], in state [%s] for channel [%p]",revents,belle_sip_channel_state_to_string(state),obj);
+		channel_set_state(base,BELLE_SIP_CHANNEL_ERROR);
 		return BELLE_SIP_STOP;
 	}
 	return BELLE_SIP_CONTINUE;
 }
 
-void belle_sip_stream_channel_init_client(belle_sip_stream_channel_t *obj, belle_sip_stack_t *stack, const char *bindip, int localport, const char *peer_cname, const char *dest, int port){
+void belle_sip_stream_channel_init_client(belle_sip_stream_channel_t *obj, belle_sip_stack_t *stack, const char *bindip, int localport, const char *peer_cname, const char *dest, int port, int no_srv){
 	belle_sip_channel_init((belle_sip_channel_t*)obj, stack
-					,bindip,localport,peer_cname,dest,port);
+					,bindip,localport,peer_cname,dest,port, no_srv);
 }
 
-belle_sip_channel_t * belle_sip_stream_channel_new_client(belle_sip_stack_t *stack,const char *bindip, int localport, const char *peer_cname, const char *dest, int port){
+belle_sip_channel_t * belle_sip_stream_channel_new_client(belle_sip_stack_t *stack,const char *bindip, int localport, const char *peer_cname, const char *dest, int port, int no_srv){
 	belle_sip_stream_channel_t *obj=belle_sip_object_new(belle_sip_stream_channel_t);
-	belle_sip_stream_channel_init_client(obj,stack,bindip,localport,peer_cname,dest,port);
+	belle_sip_stream_channel_init_client(obj,stack,bindip,localport,peer_cname,dest,port, no_srv);
 	return (belle_sip_channel_t*)obj;
 }
 
@@ -260,20 +286,20 @@ belle_sip_channel_t * belle_sip_stream_channel_new_child(belle_sip_stack_t *stac
 	belle_sip_stream_channel_t *obj;
 	int err;
 	int optval=1;
-	
+
 	err = bctbx_setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
 			(char*)&optval, sizeof (optval));
 	if (err == -1){
 		belle_sip_warning ("Fail to set SIP/TCP address reusable: %s.", belle_sip_get_socket_error_string());
 	}
-	
+
 	set_tcp_nodelay(sock);
-	
+
 	if (bctbx_getsockname(sock,(struct sockaddr*)&localaddr,&local_len)==-1){
 		belle_sip_error("bctbx_getsockname() failed: %s",belle_sip_get_socket_error_string());
 		return NULL;
 	}
-	
+
 	obj=belle_sip_object_new(belle_sip_stream_channel_t);
 	belle_sip_channel_init_with_addr((belle_sip_channel_t*)obj,stack,NULL,0,remote_addr,slen);
 	belle_sip_socket_set_nonblocking(sock);
@@ -283,5 +309,3 @@ belle_sip_channel_t * belle_sip_stream_channel_new_child(belle_sip_stack_t *stac
 	belle_sip_main_loop_add_source(stack->ml,(belle_sip_source_t*)obj);
 	return (belle_sip_channel_t*)obj;
 }
-
-
