@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2013  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle_sip_internal.h"
 #include "stream_channel.h"
@@ -279,7 +280,6 @@ int belle_sip_generate_self_signed_certificate(const char* path, const char *sub
 
 		if ( (fd = fopen(name_with_path, "w") ) == NULL) {
 			belle_sip_error("Certificate generation can't open/create file %s", name_with_path);
-			free(name_with_path);
 			belle_sip_object_unref(*pkey);
 			belle_sip_object_unref(*certificate);
 			*pkey = NULL;
@@ -329,6 +329,7 @@ char *belle_sip_certificates_chain_get_fingerprint(belle_sip_certificates_chain_
 // length - length of certificate DER data
 // depth - position of certificate in cert chain, ending at 0 = root or top
 // flags - verification state for CURRENT certificate only
+// deprecated
 typedef int (*verify_cb_error_cb_t)(unsigned char* der, int length, int depth, uint32_t* flags);
 static verify_cb_error_cb_t tls_verify_cb_error_cb = NULL;
 
@@ -339,6 +340,7 @@ struct belle_sip_tls_channel{
 	bctbx_ssl_context_t *sslctx;
 	bctbx_ssl_config_t *sslcfg;
 	bctbx_x509_certificate_t *root_ca;
+	bctbx_rng_context_t *rng;
 
 	struct sockaddr_storage ss;
 	socklen_t socklen;
@@ -366,6 +368,10 @@ static void tls_channel_uninit(belle_sip_tls_channel_t *obj){
 	if (sock!=(belle_sip_socket_t)-1)
 		tls_channel_close(obj);
 	belle_sip_tls_channel_deinit_bctbx_ssl(obj);
+	if (obj->rng) {
+		bctbx_rng_context_free(obj->rng);
+		obj->rng=NULL;
+	}
 
 	if (obj->cur_debug_msg)
 		belle_sip_free(obj->cur_debug_msg);
@@ -402,9 +408,9 @@ static int tls_channel_recv(belle_sip_channel_t *obj, void *buf, size_t buflen){
 
 static int tls_channel_connect_to(belle_sip_channel_t *obj, const struct addrinfo *ai){
 	int err;
-	
+
 	if (belle_sip_tls_channel_init_bctbx_ssl((belle_sip_tls_channel_t*)obj)==-1) return -1;
-	
+
 	err= stream_channel_connect((belle_sip_stream_channel_t*)obj,ai);
 	if (err==0){
 		belle_sip_source_set_notify((belle_sip_source_t *)obj, (belle_sip_source_func_t)tls_process_data);
@@ -413,17 +419,18 @@ static int tls_channel_connect_to(belle_sip_channel_t *obj, const struct addrinf
 	return -1;
 }
 
-static void http_proxy_res_done(void *data, const char *name, struct addrinfo *ai_list, uint32_t ttl){
+static void http_proxy_res_done(void *data, belle_sip_resolver_results_t *results){
 	belle_sip_tls_channel_t *obj=(belle_sip_tls_channel_t*)data;
+	const struct addrinfo *ai_list;
 	if (obj->http_proxy_resolver_ctx){
 		belle_sip_object_unref(obj->http_proxy_resolver_ctx);
 		obj->http_proxy_resolver_ctx=NULL;
 	}
+	ai_list = belle_sip_resolver_results_get_addrinfos(results);
 	if (ai_list){
 		tls_channel_connect_to((belle_sip_channel_t *)obj,ai_list);
-		bctbx_freeaddrinfo(ai_list);
 	}else{
-		belle_sip_error("%s: DNS resolution failed for %s", __FUNCTION__, name);
+		belle_sip_error("%s: DNS resolution failed for %s", __FUNCTION__, belle_sip_resolver_results_get_name(results));
 		channel_set_state((belle_sip_channel_t*)obj,BELLE_SIP_CHANNEL_ERROR);
 	}
 }
@@ -454,6 +461,8 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_tls_channel_t)
 				(belle_sip_object_destroy_t)tls_channel_uninit,
 				NULL,
 				NULL,
+				(belle_sip_object_on_first_ref_t) NULL,
+				(belle_sip_object_on_last_ref_t) NULL,
 				BELLE_SIP_DEFAULT_BUFSIZE_HINT
 			},
 			"TLS",
@@ -466,7 +475,7 @@ BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_BEGIN(belle_sip_tls_channel_t)
 	}
 BELLE_SIP_INSTANCIATE_CUSTOM_VPTR_END
 
-static int belle_sip_client_certificate_request_callback(void *data, bctbx_ssl_context_t *ssl_ctx, unsigned char *dn, size_t dn_length) {
+static int belle_sip_client_certificate_request_callback(void *data, bctbx_ssl_context_t *ssl_ctx, const bctbx_list_t *names) {
 	belle_sip_tls_channel_t *channel = (belle_sip_tls_channel_t *)data;
 
 	/* ask certificate */
@@ -474,7 +483,7 @@ static int belle_sip_client_certificate_request_callback(void *data, bctbx_ssl_c
 			,belle_sip_channel_listener_t
 			,on_auth_requested
 			,&channel->base.base
-			,(char *)dn);
+			,(names==NULL)?NULL:(char *)names->data); // forward only the first name of the list, this functionnality is not used for now anyway
 
 	/* if we got one, set it in the ssl handshake context */
 	if (channel->client_cert_chain && channel->client_cert_key) {
@@ -492,24 +501,47 @@ static int belle_sip_client_certificate_request_callback(void *data, bctbx_ssl_c
 		return 0;
 	}
 
-	belle_sip_warning("Channel [%p] cannot get client certificate to answer server request for dn [%s]", channel, (dn==NULL)?"null":(char *)dn);
+	belle_sip_warning("Channel [%p] cannot get client certificate to answer server request", channel);
 
 	return 0; /* we couldn't find any certificate, just keep on going, server may decide to abort the handshake */
 }
 
+static int tls_handle_postcheck(belle_sip_tls_channel_t* channel){
+	if (channel->crypto_config && channel->crypto_config->postcheck_cb){
+		const bctbx_x509_certificate_t *cert = bctbx_ssl_get_peer_certificate(channel->sslctx);
+		if (!cert){
+			belle_sip_error("tls_handle_postcheck(): no peer certificate, this should not happen");
+			return -1;
+		}
+		return channel->crypto_config->postcheck_cb(channel->crypto_config->postcheck_cb_data, cert);
+	}
+	return 0;
+}
+
 static int tls_process_handshake(belle_sip_channel_t *obj){
 	belle_sip_tls_channel_t* channel=(belle_sip_tls_channel_t*)obj;
+	char tmp[128];
 	int err=bctbx_ssl_handshake(channel->sslctx);
-	if (err==0){
+
+	memset(tmp, '\0', sizeof(tmp));
+	if (err == 0){
 		belle_sip_message("Channel [%p]: SSL handshake finished, SSL version is [%s], selected ciphersuite is [%s]",obj,
 				  bctbx_ssl_get_version(channel->sslctx), bctbx_ssl_get_ciphersuite(channel->sslctx));
-		belle_sip_source_set_timeout((belle_sip_source_t*)obj,-1);
+		err = tls_handle_postcheck(channel);
+		if (err != 0) {
+			snprintf(tmp, sizeof(tmp)-1, "%s", "application level post-check failed.");
+		}
+	}
+
+	if (err==0){
+		belle_sip_source_set_timeout_int64((belle_sip_source_t*)obj,-1);
 		belle_sip_channel_set_ready(obj,(struct sockaddr*)&channel->ss,channel->socklen);
 	}else if (err==BCTBX_ERROR_NET_WANT_READ || err==BCTBX_ERROR_NET_WANT_WRITE){
 		belle_sip_message("Channel [%p]: SSL handshake in progress...",obj);
 	}else{
-		char tmp[128];
-		bctbx_strerror(err,tmp,sizeof(tmp));
+		if (tmp[0] == '\0'){
+			bctbx_strerror(err,tmp,sizeof(tmp));
+		}
 		belle_sip_error("Channel [%p]: SSL handshake failed : %s",obj,tmp);
 		return -1;
 	}
@@ -524,10 +556,18 @@ static int tls_process_http_connect(belle_sip_tls_channel_t *obj) {
 	char *host_ip;
 	char url_ipport[64];
 	int port;
-	bctbx_addrinfo_to_printable_ip_address(channel->current_peer,url_ipport,sizeof(url_ipport));
-	bctbx_addrinfo_to_ip_address(channel->current_peer,ip,sizeof(ip),&port);
+	struct sockaddr_storage connect_addr;
+	socklen_t connect_addr_len = sizeof(connect_addr);
 	
-	if (channel->current_peer->ai_family == AF_INET6) {
+	/* In case the connection address is a V4MAPPED, transform it as a pure v4 address.
+	 * Indeed, some old http proxy implementations don't support IPv6 targets, so don't attempt
+	 * to use it if it is not strictly necessary. */
+	bctbx_sockaddr_remove_v4_mapping(channel->current_peer->ai_addr, (struct sockaddr*) &connect_addr, &connect_addr_len);
+	
+	bctbx_sockaddr_to_printable_ip_address((struct sockaddr*)&connect_addr, connect_addr_len, url_ipport,sizeof(url_ipport));
+	bctbx_sockaddr_to_ip_address((struct sockaddr*)&connect_addr, connect_addr_len, ip, sizeof(ip), &port);
+
+	if (connect_addr.ss_family == AF_INET6) {
 		host_ip = belle_sip_strdup_printf("[%s]",ip);
 	} else {
 		host_ip = belle_sip_strdup_printf("%s",ip); /*just to simplify code*/
@@ -536,7 +576,7 @@ static int tls_process_http_connect(belle_sip_tls_channel_t *obj) {
 	request = belle_sip_strdup_printf("CONNECT %s HTTP/1.1\r\nProxy-Connection: keep-alive\r\nConnection: keep-alive\r\nHost: %s\r\nUser-Agent: Mozilla/5.0\r\n"
 									  ,url_ipport
 									  ,host_ip);
-	
+	belle_sip_message("HTTP request is\n%s", request);
 	belle_sip_free(host_ip);
 
 	if (channel->stack->http_proxy_username && channel->stack->http_proxy_passwd) {
@@ -576,18 +616,18 @@ static int tls_process_data(belle_sip_channel_t *obj,unsigned int revents){
 
 			channel->socket_connected=1;
 			belle_sip_source_set_events((belle_sip_source_t*)channel,BELLE_SIP_EVENT_READ|BELLE_SIP_EVENT_ERROR);
-			belle_sip_source_set_timeout((belle_sip_source_t*)obj,belle_sip_stack_get_transport_timeout(obj->stack));
+			belle_sip_source_set_timeout_int64((belle_sip_source_t*)obj,belle_sip_stack_get_transport_timeout(obj->stack));
 			if (obj->stack->http_proxy_host) {
 				belle_sip_message("Channel [%p]: Connected at TCP level, now doing http proxy connect",obj);
 				if (tls_process_http_connect(channel)) goto process_error;
 			} else {
-				belle_sip_message("Channel [%p]: Connected at TCP level, now doing TLS handshake with cname=%s",obj, 
+				belle_sip_message("Channel [%p]: Connected at TCP level, now doing TLS handshake with cname=%s",obj,
 						  obj->peer_cname ? obj->peer_cname : obj->peer_name);
 				if (tls_process_handshake(obj)==-1) goto process_error;
 			}
 		} else if (obj->stack->http_proxy_host && !channel->http_proxy_connected) {
-			char response[256];
-			err = stream_channel_recv((belle_sip_stream_channel_t*)obj,response,sizeof(response));
+			char response[256]={0};
+			err = stream_channel_recv((belle_sip_stream_channel_t*)obj,response,sizeof(response)-1);
 			if (err<0 ){
 				belle_sip_error("Channel [%p]: connection refused by http proxy [%s:%i] status [%s]"
 								,channel
@@ -596,7 +636,7 @@ static int tls_process_data(belle_sip_channel_t *obj,unsigned int revents){
 								,strerror(errno));
 				goto process_error;
 			} else if (strstr(response,"407")) {
-				belle_sip_error("Channel [%p]: auth requested, provide user/passwd by http proxy [%s:%i]"
+				belle_sip_error("Channel [%p]: auth requested, provide user/passwd for http proxy [%s:%i]"
 								,channel
 								,obj->stack->http_proxy_host
 								,obj->stack->http_proxy_port);
@@ -626,10 +666,12 @@ static int tls_process_data(belle_sip_channel_t *obj,unsigned int revents){
 				belle_sip_warning("channel [%p]: unexpected event [%i] during TLS handshake.",obj,revents);
 			}
 		}
-	} else if ( obj->state == BELLE_SIP_CHANNEL_READY) {
+	} else if (obj->state == BELLE_SIP_CHANNEL_READY || obj->state == BELLE_SIP_CHANNEL_RES_IN_PROGRESS) {
+		/* Because of DNS TTL timeout, the channel may enter the RES_IN_PROGRESS state temporarily while being connected.*/
 		return belle_sip_channel_process_data(obj,revents);
 	} else {
-		belle_sip_warning("Unexpected event [%i], for channel [%p]",revents,channel);
+		belle_sip_error("Unexpected event [%i], for channel [%p] in state [%s]", revents, channel, belle_sip_channel_state_to_string(obj->state));
+		channel_set_state(obj,BELLE_SIP_CHANNEL_ERROR);
 		return BELLE_SIP_STOP;
 	}
 	return BELLE_SIP_CONTINUE;
@@ -669,13 +711,14 @@ static int tls_callback_write(void * ctx, const unsigned char *buf, size_t len )
 }
 
 static int random_generator(void *ctx, unsigned char *ptr, size_t size){
-	belle_sip_random_bytes(ptr, size);
-	return 0;
+	bctbx_rng_context_t *rng = (bctbx_rng_context_t *)ctx;
+	return bctbx_rng_get(rng, ptr, size);
 }
 
 
 // shim the default certificate handling by adding an external callback
 // see "verify_cb_error_cb_t" for the function signature
+/*deprecated*/
 int belle_sip_tls_set_verify_error_cb(void * callback) {
 
 	if (callback) {
@@ -688,6 +731,7 @@ int belle_sip_tls_set_verify_error_cb(void * callback) {
 	return 0;
 }
 
+
 //
 // Augment certificate verification with certificates stored outside rootca.pem
 // PolarSSL calls the verify_cb with each cert in the chain; flags apply to the
@@ -755,6 +799,9 @@ static int belle_sip_ssl_verify(void *data , bctbx_x509_certificate_t *cert , in
 		bctbx_x509_certificate_unset_flag(flags, BCTBX_CERTIFICATE_VERIFY_BADCERT_CN_MISMATCH);
 	}
 
+	if (crypto_config->verify_cb){
+		crypto_config->verify_cb(crypto_config->verify_cb_data, cert, depth, flags);
+	}
 	ret = belle_sip_verify_cb_error_wrapper(cert, depth, flags);
 
 	belle_sip_free(flags_str);
@@ -823,13 +870,13 @@ static void belle_sip_tls_channel_deinit_bctbx_ssl(belle_sip_tls_channel_t *obj)
 		bctbx_x509_certificate_free(obj->root_ca);
 		obj->root_ca = NULL;
 	}
-	
+
 }
 
 static int belle_sip_tls_channel_init_bctbx_ssl(belle_sip_tls_channel_t *obj){
 	belle_sip_stream_channel_t* super=(belle_sip_stream_channel_t*)obj;
 	belle_tls_crypto_config_t *crypto_config = obj->crypto_config;
-	
+
 	/* create and initialise ssl context and configuration */
 	obj->sslctx = bctbx_ssl_context_new();
 	obj->sslcfg = bctbx_ssl_config_new();
@@ -846,7 +893,7 @@ static int belle_sip_tls_channel_init_bctbx_ssl(belle_sip_tls_channel_t *obj){
 		belle_sip_message("Use externally provided SSL configuration when creating TLS channel [%p]", obj);
 	}
 
-	bctbx_ssl_config_set_rng(obj->sslcfg, random_generator, NULL);
+	bctbx_ssl_config_set_rng(obj->sslcfg, random_generator, obj->rng);
 	bctbx_ssl_set_io_callbacks(obj->sslctx, obj, tls_callback_write, tls_callback_read);
 	if ((crypto_config->root_ca_data && belle_sip_tls_channel_load_root_ca_from_buffer(obj, crypto_config->root_ca_data) == 0)
 		||  (crypto_config->root_ca && belle_sip_tls_channel_load_root_ca(obj, crypto_config->root_ca) == 0)){
@@ -860,15 +907,16 @@ static int belle_sip_tls_channel_init_bctbx_ssl(belle_sip_tls_channel_t *obj){
 	return 0;
 }
 
-belle_sip_channel_t * belle_sip_channel_new_tls(belle_sip_stack_t *stack, belle_tls_crypto_config_t *crypto_config, const char *bindip, int localport, const char *peer_cname, const char *dest, int port) {
+belle_sip_channel_t * belle_sip_channel_new_tls(belle_sip_stack_t *stack, belle_tls_crypto_config_t *crypto_config, const char *bindip, int localport, const char *peer_cname, const char *dest, int port, int no_srv) {
 	belle_sip_tls_channel_t *obj=belle_sip_object_new(belle_sip_tls_channel_t);
 	belle_sip_stream_channel_t* super=(belle_sip_stream_channel_t*)obj;
 
 	belle_sip_stream_channel_init_client(super
 					,stack
-					,bindip,localport,peer_cname,dest,port);
+					,bindip,localport,peer_cname,dest,port, no_srv);
 
 	obj->crypto_config=(belle_tls_crypto_config_t*)belle_sip_object_ref(crypto_config);
+	obj->rng = bctbx_rng_context_new();
 	return (belle_sip_channel_t*)obj;
 }
 
