@@ -1,20 +1,21 @@
 /*
-	belle-sdp - SDP (RFC) library.
-    Copyright (C) 2011  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle-sip/belle-sip.h"
 #include "belle_sip_tester.h"
@@ -22,7 +23,7 @@
 
 
 //v=0
-//o=jehan-mac 1239 1239 IN IP4 192.168.0.18
+//o=jehan-mac 1239 1239 IN IP4 192.168.0.19
 //s=Talk
 //c=IN IP4 192.168.0.18
 //t=0 0
@@ -73,64 +74,319 @@ static void test_attribute_2(void) {
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_value(lAttribute), "1 1 : e2br+9PL Eu1qGlQ9 10.211.55.3 8988");
 	BC_ASSERT_TRUE(belle_sdp_attribute_has_value(lAttribute));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
+}
 
+static void test_label_attribute(void) {
+	const char* line = "a=label:2";
+
+	belle_sdp_label_attribute_t* lAttribute;
+
+	lAttribute = belle_sdp_label_attribute_parse(line);
+	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "label");
+	char * obj_string = belle_sip_object_to_string(BELLE_SIP_OBJECT(lAttribute));
+	BC_ASSERT_STRING_EQUAL(obj_string, line);
+	belle_sip_free(obj_string);
+
+	belle_sdp_label_attribute_t* clone = BELLE_SDP_LABEL_ATTRIBUTE(
+		belle_sip_object_clone(BELLE_SIP_OBJECT(lAttribute))
+	);
+	char * clone_obj_string = belle_sip_object_to_string(clone);
+	BC_ASSERT_STRING_EQUAL(clone_obj_string, line);
+	belle_sip_free(clone_obj_string);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(clone));
+
+	BC_ASSERT_STRING_EQUAL(belle_sdp_label_attribute_get_pointer(lAttribute), "2");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
+}
+
+static void test_content_attribute(void) {
+	belle_sdp_content_attribute_t* lAttribute;
+	belle_sip_list_t* list;
+	int i = 0;
+	const char* fmt[] = {"slides","speaker","sl","main"};
+	const char* line = "a=content:slides,speaker,sl,main";
+
+	lAttribute = belle_sdp_content_attribute_parse(line);
+	char * obj_string = belle_sip_object_to_string(BELLE_SIP_OBJECT(lAttribute));
+	BC_ASSERT_STRING_EQUAL(obj_string, line);
+	belle_sip_free(obj_string);
+
+	belle_sdp_content_attribute_t* clone = BELLE_SDP_CONTENT_ATTRIBUTE(
+		belle_sip_object_clone(BELLE_SIP_OBJECT(lAttribute))
+	);
+	char * clone_obj_string = belle_sip_object_to_string(clone);
+	BC_ASSERT_STRING_EQUAL(clone_obj_string, line);
+	belle_sip_free(clone_obj_string);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(clone));
+
+	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "content");
+
+	list = belle_sdp_content_attribute_get_media_tags(lAttribute);
+	BC_ASSERT_PTR_NOT_NULL(list);
+	for(; list!=NULL; list=list->next){
+		BC_ASSERT_STRING_EQUAL(list->data, fmt[i++]);
+	}
+
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
+}
+
+static void test_csup_attribute(void) {
+	belle_sdp_csup_attribute_t* lAttribute;
+	belle_sip_list_t* list;
+	int i = 0;
+	const char* fmt[] = {"cap-v0","foo","bar"};
+	const char* line = "a=csup:cap-v0,foo,bar";
+
+	lAttribute = belle_sdp_csup_attribute_parse(line);
+	char * obj_string = belle_sip_object_to_string(BELLE_SIP_OBJECT(lAttribute));
+	BC_ASSERT_STRING_EQUAL(obj_string, line);
+	belle_sip_free(obj_string);
+
+	belle_sdp_csup_attribute_t* clone = BELLE_SDP_CSUP_ATTRIBUTE(
+		belle_sip_object_clone(BELLE_SIP_OBJECT(lAttribute))
+	);
+	char * clone_obj_string = belle_sip_object_to_string(clone);
+	BC_ASSERT_STRING_EQUAL(clone_obj_string, line);
+	belle_sip_free(clone_obj_string);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(clone));
+
+	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "csup");
+
+	list = belle_sdp_csup_attribute_get_option_tags(lAttribute);
+	BC_ASSERT_PTR_NOT_NULL(list);
+	for(; list!=NULL; list=list->next){
+		BC_ASSERT_STRING_EQUAL(list->data, fmt[i++]);
+	}
+
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
+}
+
+static void test_creq_attribute(void) {
+	belle_sdp_creq_attribute_t* lAttribute;
+	belle_sip_list_t* list;
+	int i = 0;
+	const char* fmt[] = {"cap-v0","foo","bar"};
+	const char* line = "a=creq:cap-v0,foo,bar";
+
+	lAttribute = belle_sdp_creq_attribute_parse(line);
+	char * obj_string = belle_sip_object_to_string(BELLE_SIP_OBJECT(lAttribute));
+	BC_ASSERT_STRING_EQUAL(obj_string, line);
+	belle_sip_free(obj_string);
+
+	belle_sdp_creq_attribute_t* clone = BELLE_SDP_CREQ_ATTRIBUTE(
+		belle_sip_object_clone(BELLE_SIP_OBJECT(lAttribute))
+	);
+	char * clone_obj_string = belle_sip_object_to_string(clone);
+	BC_ASSERT_STRING_EQUAL(clone_obj_string, line);
+	belle_sip_free(clone_obj_string);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(clone));
+
+	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "creq");
+
+	list = belle_sdp_creq_attribute_get_option_tags(lAttribute);
+	BC_ASSERT_PTR_NOT_NULL(list);
+	for(; list!=NULL; list=list->next){
+		BC_ASSERT_STRING_EQUAL(list->data, fmt[i++]);
+	}
+
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
+}
+
+static void test_tcap_attribute(void) {
+	belle_sdp_tcap_attribute_t* lAttribute;
+	belle_sip_list_t* list;
+	int i = 0;
+	const char* protos[] = {"RTP/SAVP","RTP/SAVPF"};
+	const char* line = "a=tcap:5 RTP/SAVP RTP/SAVPF";
+
+	lAttribute = belle_sdp_tcap_attribute_parse(line);
+	char * obj_string = belle_sip_object_to_string(BELLE_SIP_OBJECT(lAttribute));
+	BC_ASSERT_STRING_EQUAL(obj_string, line);
+	belle_sip_free(obj_string);
+
+	belle_sdp_tcap_attribute_t* clone = BELLE_SDP_TCAP_ATTRIBUTE(
+		belle_sip_object_clone(BELLE_SIP_OBJECT(lAttribute))
+	);
+	char * clone_obj_string = belle_sip_object_to_string(clone);
+	BC_ASSERT_STRING_EQUAL(clone_obj_string, line);
+	belle_sip_free(clone_obj_string);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(clone));
+
+	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "tcap");
+	BC_ASSERT_EQUAL(belle_sdp_tcap_attribute_get_id(lAttribute), 5, int, "%d");
+
+	list = belle_sdp_tcap_attribute_get_protos(lAttribute);
+	BC_ASSERT_PTR_NOT_NULL(list);
+	for(; list!=NULL; list=list->next){
+		BC_ASSERT_STRING_EQUAL(list->data, protos[i++]);
+	}
+
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
+}
+
+static void test_acap_attribute_base(const char* line, int id, const char * name, const char * value) {
+	belle_sdp_acap_attribute_t* lAttribute;
+
+	lAttribute = belle_sdp_acap_attribute_parse(line);
+	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "acap");
+	char * obj_string = belle_sip_object_to_string(BELLE_SIP_OBJECT(lAttribute));
+	BC_ASSERT_STRING_EQUAL(obj_string, line);
+	belle_sip_free(obj_string);
+
+	belle_sdp_acap_attribute_t* clone = BELLE_SDP_ACAP_ATTRIBUTE(
+		belle_sip_object_clone(BELLE_SIP_OBJECT(lAttribute))
+	);
+	char * clone_obj_string = belle_sip_object_to_string(clone);
+	BC_ASSERT_STRING_EQUAL(clone_obj_string, line);
+	belle_sip_free(clone_obj_string);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(clone));
+
+	BC_ASSERT_EQUAL(belle_sdp_acap_attribute_get_id(lAttribute), id, int, "%d");
+	BC_ASSERT_STRING_EQUAL(belle_sdp_acap_attribute_get_name(lAttribute), name);
+	BC_ASSERT_STRING_EQUAL(belle_sdp_acap_attribute_get_value(lAttribute), value);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
+}
+
+static void test_acap_attribute(void) {
+	const char* line = "a=acap:3 key-mgmt:mikey AQAFgM";
+	test_acap_attribute_base(line, 3, "key-mgmt", "mikey AQAFgM");
+}
+
+static void test_simple_acap_attribute(void) {
+	const char* line = "a=acap:20 ptime:30";
+	test_acap_attribute_base(line, 20, "ptime", "30");
+}
+
+static void test_long_acap_attribute(void) {
+	const char* line = "a=acap:10021 crypto:1 AES_CM_256_HMAC_SHA1_80 inline:WVNfX19zZW1jdGwgKCkgewkyMjA7fQp9CnVubGVz|2^20|1:4";
+	test_acap_attribute_base(line, 10021, "crypto", "1 AES_CM_256_HMAC_SHA1_80 inline:WVNfX19zZW1jdGwgKCkgewkyMjA7fQp9CnVubGVz|2^20|1:4");
+}
+
+static void test_acfg_attribute(void) {
+	belle_sdp_acfg_attribute_t* lAttribute;
+	belle_sip_list_t* list;
+	int i = 0;
+	const char* line = "a=acfg:1 t=3 a=[2]";
+	const char* configs[] = {"t=3","a=[2]"};
+
+	lAttribute = belle_sdp_acfg_attribute_parse(line);
+	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "acfg");
+	char * obj_string = belle_sip_object_to_string(BELLE_SIP_OBJECT(lAttribute));
+	BC_ASSERT_STRING_EQUAL(obj_string, line);
+	belle_sip_free(obj_string);
+
+	belle_sdp_acfg_attribute_t* clone = BELLE_SDP_ACFG_ATTRIBUTE(
+		belle_sip_object_clone(BELLE_SIP_OBJECT(lAttribute))
+	);
+	char * clone_obj_string = belle_sip_object_to_string(clone);
+	BC_ASSERT_STRING_EQUAL(clone_obj_string, line);
+	belle_sip_free(clone_obj_string);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(clone));
+
+	BC_ASSERT_EQUAL(belle_sdp_acfg_attribute_get_id(lAttribute), 1, int, "%d");
+
+	list = belle_sdp_acfg_attribute_get_configs(lAttribute);
+	BC_ASSERT_PTR_NOT_NULL(list);
+	for(; list!=NULL; list=list->next){
+		BC_ASSERT_STRING_EQUAL(list->data, configs[i++]);
+	}
+
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
+}
+static void test_pcfg_attribute(void) {
+	belle_sdp_pcfg_attribute_t* lAttribute;
+	belle_sip_list_t* list;
+	int i = 0;
+	const char* line = "a=pcfg:1 a=-m:1,2,[3,4]|1,7,[5] a=[2]";
+	const char* configs[] = {"a=-m:1,2,[3,4]|1,7,[5]","a=[2]"};
+
+	lAttribute = belle_sdp_pcfg_attribute_parse(line);
+	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "pcfg");
+	char * obj_string = belle_sip_object_to_string(BELLE_SIP_OBJECT(lAttribute));
+	BC_ASSERT_STRING_EQUAL(obj_string, line);
+	belle_sip_free(obj_string);
+
+	belle_sdp_pcfg_attribute_t* clone = BELLE_SDP_PCFG_ATTRIBUTE(
+		belle_sip_object_clone(BELLE_SIP_OBJECT(lAttribute))
+	);
+	char * clone_obj_string = belle_sip_object_to_string(clone);
+	BC_ASSERT_STRING_EQUAL(clone_obj_string, line);
+	belle_sip_free(clone_obj_string);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(clone));
+
+	BC_ASSERT_EQUAL(belle_sdp_pcfg_attribute_get_id(lAttribute), 1, int, "%d");
+
+	list = belle_sdp_pcfg_attribute_get_configs(lAttribute);
+	BC_ASSERT_PTR_NOT_NULL(list);
+	for(; list!=NULL; list=list->next){
+		BC_ASSERT_STRING_EQUAL(list->data, configs[i++]);
+	}
+
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
 }
 
 static void test_rtcp_fb_attribute(void) {
 	belle_sdp_rtcp_fb_attribute_t* lAttribute;
 
-	lAttribute = BELLE_SDP_RTCP_FB_ATTRIBUTE(attribute_parse_marshall_parse_clone("a=rtcp-fb:* ack"));
+	lAttribute = belle_sdp_rtcp_fb_attribute_parse("a=rtcp-fb:* ack");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-fb");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_id(lAttribute), -1, int, "%d");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_type(lAttribute), BELLE_SDP_RTCP_FB_ACK, int, "%d");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_param(lAttribute), BELLE_SDP_RTCP_FB_NONE, int, "%d");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
 
-	lAttribute = BELLE_SDP_RTCP_FB_ATTRIBUTE(attribute_parse_marshall_parse_clone("a=rtcp-fb:98 nack rpsi"));
+	lAttribute = belle_sdp_rtcp_fb_attribute_parse("a=rtcp-fb:98 nack rpsi");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-fb");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_id(lAttribute), 98, int, "%d");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_type(lAttribute), BELLE_SDP_RTCP_FB_NACK, int, "%d");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_param(lAttribute), BELLE_SDP_RTCP_FB_RPSI, int, "%d");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
 
-	lAttribute = BELLE_SDP_RTCP_FB_ATTRIBUTE(attribute_parse_marshall_parse_clone("a=rtcp-fb:* trr-int 3"));
+	lAttribute = belle_sdp_rtcp_fb_attribute_parse("a=rtcp-fb:* trr-int 3");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-fb");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_id(lAttribute), -1, int, "%d");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_type(lAttribute), BELLE_SDP_RTCP_FB_TRR_INT, int, "%d");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_trr_int(lAttribute), 3, int, "%d");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
 
-	lAttribute = BELLE_SDP_RTCP_FB_ATTRIBUTE(attribute_parse_marshall_parse_clone("a=rtcp-fb:103 ccm fir"));
+	lAttribute = belle_sdp_rtcp_fb_attribute_parse("a=rtcp-fb:103 ccm fir");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-fb");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_id(lAttribute), 103, int, "%d");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_type(lAttribute), BELLE_SDP_RTCP_FB_CCM, int, "%d");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_param(lAttribute), BELLE_SDP_RTCP_FB_FIR, int, "%d");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
+
+	lAttribute = belle_sdp_rtcp_fb_attribute_parse("a=rtcp-fb:* ccm tmmbr smaxpr=120");
+	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-fb");
+	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_id(lAttribute), -1, int, "%d");
+	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_type(lAttribute), BELLE_SDP_RTCP_FB_CCM, int, "%d");
+	BC_ASSERT_EQUAL(belle_sdp_rtcp_fb_attribute_get_param(lAttribute), BELLE_SDP_RTCP_FB_TMMBR, int, "%d");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
 }
 
 static void test_rtcp_xr_attribute(void) {
 	belle_sdp_rtcp_xr_attribute_t* lAttribute;
 
-	lAttribute = BELLE_SDP_RTCP_XR_ATTRIBUTE(attribute_parse_marshall_parse_clone("a=rtcp-xr"));
+	lAttribute = belle_sdp_rtcp_xr_attribute_parse("a=rtcp-xr");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-xr");
 	BC_ASSERT_FALSE(belle_sdp_rtcp_xr_attribute_has_stat_summary(lAttribute));
 	BC_ASSERT_FALSE(belle_sdp_rtcp_xr_attribute_has_voip_metrics(lAttribute));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
 
-	lAttribute = BELLE_SDP_RTCP_XR_ATTRIBUTE(attribute_parse_marshall_parse_clone("a=rtcp-xr:rcvr-rtt=all:10"));
+	lAttribute = belle_sdp_rtcp_xr_attribute_parse("a=rtcp-xr:rcvr-rtt=all:10");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-xr");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_rtcp_xr_attribute_get_rcvr_rtt_mode(lAttribute), "all");
 	BC_ASSERT_EQUAL(belle_sdp_rtcp_xr_attribute_get_rcvr_rtt_max_size(lAttribute), 10, int, "%d");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
 
-	lAttribute = BELLE_SDP_RTCP_XR_ATTRIBUTE(attribute_parse_marshall_parse_clone("a=rtcp-xr:stat-summary"));
+	lAttribute = belle_sdp_rtcp_xr_attribute_parse("a=rtcp-xr:stat-summary");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-xr");
 	BC_ASSERT_PTR_NULL(belle_sdp_rtcp_xr_attribute_get_rcvr_rtt_mode(lAttribute));
 	BC_ASSERT_TRUE(belle_sdp_rtcp_xr_attribute_has_stat_summary(lAttribute));
 	BC_ASSERT_FALSE(belle_sdp_rtcp_xr_attribute_has_voip_metrics(lAttribute));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
 
-	lAttribute = BELLE_SDP_RTCP_XR_ATTRIBUTE(attribute_parse_marshall_parse_clone("a=rtcp-xr:stat-summary=loss,jitt"));
+	lAttribute = belle_sdp_rtcp_xr_attribute_parse("a=rtcp-xr:stat-summary=loss,jitt");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-xr");
 	BC_ASSERT_TRUE(belle_sdp_rtcp_xr_attribute_has_stat_summary(lAttribute));
 	BC_ASSERT_PTR_NOT_NULL(belle_sip_list_find_custom(belle_sdp_rtcp_xr_attribute_get_stat_summary_flags(lAttribute), (belle_sip_compare_func)strcasecmp, "loss"));
@@ -138,13 +394,13 @@ static void test_rtcp_xr_attribute(void) {
 	BC_ASSERT_PTR_NULL(belle_sip_list_find_custom(belle_sdp_rtcp_xr_attribute_get_stat_summary_flags(lAttribute), (belle_sip_compare_func)strcasecmp, "HL"));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
 
-	lAttribute = BELLE_SDP_RTCP_XR_ATTRIBUTE(attribute_parse_marshall_parse_clone("a=rtcp-xr:voip-metrics"));
+	lAttribute = belle_sdp_rtcp_xr_attribute_parse("a=rtcp-xr:voip-metrics");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-xr");
 	BC_ASSERT_FALSE(belle_sdp_rtcp_xr_attribute_has_stat_summary(lAttribute));
 	BC_ASSERT_TRUE(belle_sdp_rtcp_xr_attribute_has_voip_metrics(lAttribute));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lAttribute));
 
-	lAttribute = BELLE_SDP_RTCP_XR_ATTRIBUTE(attribute_parse_marshall_parse_clone("a=rtcp-xr:rcvr-rtt=sender stat-summary=loss,dup,jitt,TTL voip-metrics"));
+	lAttribute = belle_sdp_rtcp_xr_attribute_parse("a=rtcp-xr:rcvr-rtt=sender stat-summary=loss,dup,jitt,TTL voip-metrics");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_attribute_get_name(BELLE_SDP_ATTRIBUTE(lAttribute)), "rtcp-xr");
 	BC_ASSERT_STRING_EQUAL(belle_sdp_rtcp_xr_attribute_get_rcvr_rtt_mode(lAttribute), "sender");
 	BC_ASSERT_TRUE(belle_sdp_rtcp_xr_attribute_has_stat_summary(lAttribute));
@@ -185,7 +441,6 @@ static void test_origin(void) {
 	belle_sip_free(l_raw_origin);
 }
 
-
 static void test_malformed_origin(void) {
 	belle_sdp_origin_t* lOrigin = belle_sdp_origin_parse("o=Jehan Monnier 3800 2558 IN IP4 192.168.0.165");
 	BC_ASSERT_PTR_NULL(lOrigin);
@@ -269,7 +524,6 @@ static void test_connection_multicast(void) {
 
 }
 
-
 static void test_email(void) {
 	belle_sdp_email_t* lTmp;
 	belle_sdp_email_t* l_email = belle_sdp_email_parse("e= jehan <jehan@linphone.org>");
@@ -320,12 +574,43 @@ static void test_media(void) {
 	belle_sip_free(l_raw_media);
 }
 
-static void test_media_description_base(belle_sdp_media_description_t* media_description) {
-	const char* attr[] ={"99 MP4V-ES/90000"
-				,"99 profile-level-id=3"
-				,"97 theora/90000"
-				,"98 H263-1998/90000"
-				,"98 CIF=1;QCIF=1"};
+const char* media_description_attr[] ={"2"
+			,"speaker"
+			,"98 nack rpsi"
+			,"rcvr-rtt=all:10"
+			,"4 key-mgmt:mikey AQAFgM"
+			,"2147483647 key-mgmt:mikey YjKBgNn"
+			,"20 ptime:30"
+			,"10021 crypto:1 AES_CM_256_HMAC_SHA1_80 inline:WVNfX19zZW1jdGwgKCkgewkyMjA7fQp9CnVubGVz|2^20|1:4"
+			,"6 RTP/SAVP RTP/SAVPF"
+			,"99 RTP/AVP RTP/AVPF"
+			,"99 MP4V-ES/90000"
+			,"99 profile-level-id=3"
+			,"97 theora/90000"
+			,"98 H263-1998/90000"
+			,"1 t=99 a=[2]"
+			,"2 t=6 a=2147483647"
+			,"98 CIF=1;QCIF=1"};
+
+const char* media_description_attr_2[] ={"2"
+			,"speaker"
+			,"98 nack rpsi"
+			,"2 t=6 a=2147483647"
+			,"rcvr-rtt=all:10"
+			,"4 key-mgmt:mikey AQAFgM"
+			,"2147483647 key-mgmt:mikey YjKBgNn"
+			,"20 ptime:30"
+			,"10021 crypto:1 AES_CM_256_HMAC_SHA1_80 inline:WVNfX19zZW1jdGwgKCkgewkyMjA7fQp9CnVubGVz|2^20|1:4"
+			,"6 RTP/SAVP RTP/SAVPF"
+			,"99 RTP/AVP RTP/AVPF"
+			,"99 MP4V-ES/90000"
+			,"99 profile-level-id=3"
+			,"97 theora/90000"
+			,"98 H263-1998/90000"
+			,"1 t=99 a=[2]"
+			,"98 CIF=1;QCIF=1"};
+
+static void test_media_description_base(const char** attr, belle_sdp_media_description_t* media_description) {
 	belle_sdp_connection_t* lConnection;
 	belle_sdp_media_description_t* l_media_description=media_description;
 	belle_sdp_media_t* l_media = belle_sdp_media_description_get_media(l_media_description);
@@ -348,7 +633,6 @@ static void test_media_description_base(belle_sdp_media_description_t* media_des
 	BC_ASSERT_STRING_EQUAL(belle_sdp_connection_get_network_type(lConnection), "IN");
 
 	/*bandwidth*/
-
 	BC_ASSERT_EQUAL(belle_sdp_media_description_get_bandwidth(l_media_description,"AS"),380, int, "%d");
 
 	/*attributes*/
@@ -363,12 +647,25 @@ static void test_media_description_base(belle_sdp_media_description_t* media_des
 
 static void test_media_description(void) {
 	const char* l_src = "m=video 8078 RTP/AVP 99 97 98\r\n"\
+						"i=Hey\r\n"\
 						"c=IN IP4 192.168.0.18\r\n"\
 						"b=AS:380\r\n"\
+						"a=label:2\r\n"\
+						"a=content:speaker\r\n"\
+						"a=rtcp-fb:98 nack rpsi\r\n"\
+						"a=rtcp-xr:rcvr-rtt=all:10\r\n"\
+						"a=acap:4 key-mgmt:mikey AQAFgM\r\n"\
+						"a=acap:2147483647 key-mgmt:mikey YjKBgNn\r\n"\
+						"a=acap:20 ptime:30\r\n"\
+						"a=acap:10021 crypto:1 AES_CM_256_HMAC_SHA1_80 inline:WVNfX19zZW1jdGwgKCkgewkyMjA7fQp9CnVubGVz|2^20|1:4\r\n"\
+						"a=tcap:6 RTP/SAVP RTP/SAVPF\r\n"\
+						"a=tcap:99 RTP/AVP RTP/AVPF\r\n"\
 						"a=rtpmap:99 MP4V-ES/90000\r\n"\
 						"a=fmtp:99 profile-level-id=3\r\n"\
 						"a=rtpmap:97 theora/90000\r\n"\
 						"a=rtpmap:98 H263-1998/90000\r\n"\
+						"a=acfg:1 t=99 a=[2]\r\n"\
+						"a=acfg:2 t=6 a=2147483647\r\n"\
 						"a=fmtp:98 CIF=1;QCIF=1\r\n";
 
 	belle_sdp_media_description_t* lTmp;
@@ -378,7 +675,7 @@ static void test_media_description(void) {
 	lTmp = belle_sdp_media_description_parse(l_raw_media_description);
 	l_media_description = BELLE_SDP_MEDIA_DESCRIPTION(belle_sip_object_clone(BELLE_SIP_OBJECT(lTmp)));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(lTmp));
-	test_media_description_base(l_media_description);
+	test_media_description_base(media_description_attr, l_media_description);
 	belle_sip_object_unref(BELLE_SIP_OBJECT(l_media_description));
 	belle_sip_free(l_raw_media_description);
 	return;
@@ -390,8 +687,12 @@ static void test_simple_session_description(void) {
 						"s=Talk\r\n"\
 						"c=IN IP4 192.168.0.18\r\n"\
 						"t=0 0\r\n"\
+						"a=acfg:1 t=3 a=[2]\r\n"\
+						"a=tcap:5 RTP/SAVP RTP/SAVPF\r\n"\
 						"m=audio 7078 RTP/AVP 111 110 3 0 8 101\r\n"\
 						"a=alt:1 1 : e2br+9PL Eu1qGlQ9 10.211.55.3 8988\r\n"\
+						"a=acap:3 key-mgmt:mikey AQAFgM\r\n"\
+						"a=tcap:2 RTP/SAVP RTP/SAVPF\r\n"\
 						"a=rtpmap:111 speex/16000\r\n"\
 						"a=fmtp:111 vbr=on\r\n"\
 						"a=rtpmap:110 speex/8000\r\n"\
@@ -401,16 +702,29 @@ static void test_simple_session_description(void) {
 						"m=video 8078 RTP/AVP 99 97 98\r\n"\
 						"c=IN IP4 192.168.0.18\r\n"\
 						"b=AS:380\r\n"\
+						"a=label:2\r\n"\
+						"a=content:speaker\r\n"\
+						"a=rtcp-fb:98 nack rpsi\r\n"\
+						"a=rtcp-xr:rcvr-rtt=all:10\r\n"\
+						"a=acap:4 key-mgmt:mikey AQAFgM\r\n"\
+						"a=acap:2147483647 key-mgmt:mikey YjKBgNn\r\n"\
+						"a=acap:20 ptime:30\r\n"\
+						"a=acap:10021 crypto:1 AES_CM_256_HMAC_SHA1_80 inline:WVNfX19zZW1jdGwgKCkgewkyMjA7fQp9CnVubGVz|2^20|1:4\r\n"\
+						"a=tcap:6 RTP/SAVP RTP/SAVPF\r\n"\
+						"a=tcap:99 RTP/AVP RTP/AVPF\r\n"\
 						"a=rtpmap:99 MP4V-ES/90000\r\n"\
 						"a=fmtp:99 profile-level-id=3\r\n"\
 						"a=rtpmap:97 theora/90000\r\n"\
 						"a=rtpmap:98 H263-1998/90000\r\n"\
+						"a=acfg:1 t=99 a=[2]\r\n"\
+						"a=acfg:2 t=6 a=2147483647\r\n"\
 						"a=fmtp:98 CIF=1;QCIF=1\r\n";
 	belle_sdp_origin_t* l_origin;
 	belle_sip_list_t* media_descriptions;
 	belle_sdp_session_description_t* lTmp;
 	belle_sdp_session_description_t* l_session_description = belle_sdp_session_description_parse(l_src);
 	char* l_raw_session_description = belle_sip_object_to_string(BELLE_SIP_OBJECT(l_session_description));
+
 	belle_sip_object_unref(BELLE_SIP_OBJECT(l_session_description));
 	lTmp = belle_sdp_session_description_parse(l_raw_session_description);
 	belle_sip_free(l_raw_session_description);
@@ -438,11 +752,98 @@ static void test_simple_session_description(void) {
 
 	media_descriptions = belle_sdp_session_description_get_media_descriptions(l_session_description);
 	BC_ASSERT_PTR_NOT_NULL(media_descriptions);
-	BC_ASSERT_STRING_EQUAL (belle_sdp_media_get_media_type(belle_sdp_media_description_get_media((belle_sdp_media_description_t*)(media_descriptions->data))),"audio");
-	media_descriptions=media_descriptions->next;
-	BC_ASSERT_PTR_NOT_NULL(media_descriptions);
+	if (media_descriptions) {
+		BC_ASSERT_STRING_EQUAL (belle_sdp_media_get_media_type(belle_sdp_media_description_get_media((belle_sdp_media_description_t*)(media_descriptions->data))),"audio");
+		media_descriptions=media_descriptions->next;
+		BC_ASSERT_PTR_NOT_NULL(media_descriptions);
+		if (media_descriptions) {
+			test_media_description_base(media_description_attr, (belle_sdp_media_description_t*)(media_descriptions->data));
+		}
+	}
+	belle_sip_object_unref(l_session_description);
+	return;
+}
 
-	test_media_description_base((belle_sdp_media_description_t*)(media_descriptions->data));
+static void test_session_description_with_capability_referenced_before_definition(void) {
+	const char* l_src = "v=0\r\n"\
+						"o=jehan-mac 2463217870 2463217870 IN IP4 192.168.0.19\r\n"\
+						"s=Talk\r\n"\
+						"c=IN IP4 192.168.0.18\r\n"\
+						"t=0 0\r\n"\
+						"a=acfg:1 t=3 a=[2]\r\n"\
+						"a=tcap:5 RTP/SAVP RTP/SAVPF\r\n"\
+						"m=audio 7078 RTP/AVP 111 110 3 0 8 101\r\n"\
+						"a=alt:1 1 : e2br+9PL Eu1qGlQ9 10.211.55.3 8988\r\n"\
+						"a=acap:3 key-mgmt:mikey AQAFgM\r\n"\
+						"a=tcap:2 RTP/SAVP RTP/SAVPF\r\n"\
+						"a=rtpmap:111 speex/16000\r\n"\
+						"a=fmtp:111 vbr=on\r\n"\
+						"a=rtpmap:110 speex/8000\r\n"\
+						"a=fmtp:110 vbr=on\r\n"\
+						"a=rtpmap:101 telephone-event/8000\r\n"\
+						"a=fmtp:101 0-11\r\n"\
+						"m=video 8078 RTP/AVP 99 97 98\r\n"\
+						"c=IN IP4 192.168.0.18\r\n"\
+						"b=AS:380\r\n"\
+						"a=label:2\r\n"\
+						"a=content:speaker\r\n"\
+						"a=rtcp-fb:98 nack rpsi\r\n"\
+						"a=acfg:2 t=6 a=2147483647\r\n"\
+						"a=rtcp-xr:rcvr-rtt=all:10\r\n"\
+						"a=acap:4 key-mgmt:mikey AQAFgM\r\n"\
+						"a=acap:2147483647 key-mgmt:mikey YjKBgNn\r\n"\
+						"a=acap:20 ptime:30\r\n"\
+						"a=acap:10021 crypto:1 AES_CM_256_HMAC_SHA1_80 inline:WVNfX19zZW1jdGwgKCkgewkyMjA7fQp9CnVubGVz|2^20|1:4\r\n"\
+						"a=tcap:6 RTP/SAVP RTP/SAVPF\r\n"\
+						"a=tcap:99 RTP/AVP RTP/AVPF\r\n"\
+						"a=rtpmap:99 MP4V-ES/90000\r\n"\
+						"a=fmtp:99 profile-level-id=3\r\n"\
+						"a=rtpmap:97 theora/90000\r\n"\
+						"a=rtpmap:98 H263-1998/90000\r\n"\
+						"a=acfg:1 t=99 a=[2]\r\n"\
+						"a=fmtp:98 CIF=1;QCIF=1\r\n";
+
+	belle_sdp_origin_t* l_origin;
+	belle_sip_list_t* media_descriptions;
+	belle_sdp_session_description_t* lTmp;
+	belle_sdp_session_description_t* l_session_description = belle_sdp_session_description_parse(l_src);
+	char* l_raw_session_description = belle_sip_object_to_string(BELLE_SIP_OBJECT(l_session_description));
+
+	belle_sip_object_unref(BELLE_SIP_OBJECT(l_session_description));
+	lTmp = belle_sdp_session_description_parse(l_raw_session_description);
+	belle_sip_free(l_raw_session_description);
+	l_session_description = BELLE_SDP_SESSION_DESCRIPTION(belle_sip_object_clone(BELLE_SIP_OBJECT(lTmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lTmp));
+
+	BC_ASSERT_PTR_NOT_NULL(belle_sdp_session_description_get_version(l_session_description));
+	BC_ASSERT_EQUAL(belle_sdp_version_get_version(belle_sdp_session_description_get_version(l_session_description)),0, int, "%d");
+
+	l_origin = belle_sdp_session_description_get_origin(l_session_description);
+	BC_ASSERT_PTR_NOT_NULL(l_origin);
+	BC_ASSERT_STRING_EQUAL(belle_sdp_origin_get_address(l_origin),"192.168.0.19");
+	BC_ASSERT_STRING_EQUAL(belle_sdp_origin_get_address_type(l_origin),"IP4");
+	BC_ASSERT_STRING_EQUAL(belle_sdp_origin_get_network_type(l_origin),"IN");
+	BC_ASSERT_EQUAL(belle_sdp_origin_get_session_id(l_origin), 2463217870U, unsigned, "%u");
+	BC_ASSERT_EQUAL(belle_sdp_origin_get_session_version(l_origin), 2463217870U, unsigned, "%u");
+
+	BC_ASSERT_PTR_NOT_NULL(belle_sdp_session_description_get_session_name(l_session_description));
+	BC_ASSERT_STRING_EQUAL(belle_sdp_session_name_get_value(belle_sdp_session_description_get_session_name(l_session_description)),"Talk");
+
+	BC_ASSERT_PTR_NOT_NULL(belle_sdp_session_description_get_connection(l_session_description));
+	BC_ASSERT_PTR_NOT_NULL(belle_sdp_session_description_get_time_descriptions(l_session_description));
+	BC_ASSERT_EQUAL(belle_sdp_time_get_start(belle_sdp_time_description_get_time((belle_sdp_time_description_t*)(belle_sdp_session_description_get_time_descriptions(l_session_description)->data))),0, int, "%d");
+	BC_ASSERT_EQUAL(belle_sdp_time_get_stop(belle_sdp_time_description_get_time((belle_sdp_time_description_t*)(belle_sdp_session_description_get_time_descriptions(l_session_description)->data))),0, int, "%d");
+
+	media_descriptions = belle_sdp_session_description_get_media_descriptions(l_session_description);
+	BC_ASSERT_PTR_NOT_NULL(media_descriptions);
+	if (media_descriptions) {
+		BC_ASSERT_STRING_EQUAL (belle_sdp_media_get_media_type(belle_sdp_media_description_get_media((belle_sdp_media_description_t*)(media_descriptions->data))),"audio");
+		media_descriptions=media_descriptions->next;
+		BC_ASSERT_PTR_NOT_NULL(media_descriptions);
+		if (media_descriptions) {
+			test_media_description_base(media_description_attr_2, (belle_sdp_media_description_t*)(media_descriptions->data));
+		}
+	}
 	belle_sip_object_unref(l_session_description);
 	return;
 }
@@ -472,6 +873,10 @@ static const char* big_sdp = "v=0\r\n"\
 						"b=AS:380\r\n"\
 						"t=0 0\r\n"\
 						"a=ice-pwd:31ec21eb38b2ec6d36e8dc7b\r\n"\
+						"a=acap:1 key-mgmt:mikey AQAFgM\r\n"\
+						"a=acap:2 key-mgmt:mikey YjKBgNn\r\n"\
+						"a=tcap:1 RTP/SAVP RTP/SAVPF\r\n"\
+						"a=tcap:2 RTP/AVP RTP/AVPF\r\n"\
 						"m=audio 7078 RTP/AVP 111 110 3 0 8 101\r\n"\
 						"a=rtpmap:111 speex/16000\r\n"\
 						"a=fmtp:111 vbr=on\r\n"\
@@ -482,10 +887,22 @@ static const char* big_sdp = "v=0\r\n"\
 						"m=video 8078 RTP/AVP 99 97 98\r\n"\
 						"c=IN IP4 192.168.0.18\r\n"\
 						"b=AS:380\r\n"\
+						"a=label:2\r\n"\
+						"a=content:speaker\r\n"\
+						"a=rtcp-fb:98 nack rpsi\r\n"\
+						"a=rtcp-xr:rcvr-rtt=all:10\r\n"\
+						"a=acap:4 key-mgmt:mikey AQAFgM\r\n"\
+						"a=acap:2147483647 key-mgmt:mikey YjKBgNn\r\n"\
+						"a=acap:20 ptime:30\r\n"\
+						"a=acap:10021 crypto:1 AES_CM_256_HMAC_SHA1_80 inline:WVNfX19zZW1jdGwgKCkgewkyMjA7fQp9CnVubGVz|2^20|1:4\r\n"\
+						"a=tcap:6 RTP/SAVP RTP/SAVPF\r\n"\
+						"a=tcap:99 RTP/AVP RTP/AVPF\r\n"\
 						"a=rtpmap:99 MP4V-ES/90000\r\n"\
 						"a=fmtp:99 profile-level-id=3\r\n"\
 						"a=rtpmap:97 theora/90000\r\n"\
 						"a=rtpmap:98 H263-1998/90000\r\n"\
+						"a=acfg:1 t=99 a=[2]\r\n"\
+						"a=acfg:2 t=6 a=2147483647\r\n"\
 						"a=fmtp:98 CIF=1;QCIF=1\r\n";
 
 static void test_session_description(void) {
@@ -516,6 +933,8 @@ static void test_session_description(void) {
 	BC_ASSERT_STRING_EQUAL(belle_sdp_session_name_get_value(belle_sdp_session_description_get_session_name(l_session_description)),"SIP Talk");
 
 	BC_ASSERT_PTR_NOT_NULL(belle_sdp_session_description_get_connection(l_session_description));
+	BC_ASSERT_EQUAL(belle_sdp_session_description_get_bandwidth(l_session_description,"AS"), 380, int, "%d");
+
 	BC_ASSERT_PTR_NOT_NULL(belle_sdp_session_description_get_time_descriptions(l_session_description));
 	BC_ASSERT_EQUAL(belle_sdp_time_get_start(belle_sdp_time_description_get_time((belle_sdp_time_description_t*)(belle_sdp_session_description_get_time_descriptions(l_session_description)->data))),0, int, "%d");
 	BC_ASSERT_EQUAL(belle_sdp_time_get_stop(belle_sdp_time_description_get_time((belle_sdp_time_description_t*)(belle_sdp_session_description_get_time_descriptions(l_session_description)->data))),0, int, "%d");
@@ -526,11 +945,35 @@ static void test_session_description(void) {
 	media_descriptions=media_descriptions->next;
 	BC_ASSERT_PTR_NOT_NULL(media_descriptions);
 
-	test_media_description_base((belle_sdp_media_description_t*)(media_descriptions->data));
+	test_media_description_base(media_description_attr, (belle_sdp_media_description_t*)(media_descriptions->data));
 	belle_sip_object_unref(l_session_description);
 	return;
 }
 
+static const char* sdp_with_invalid_rtcp_fb = "v=0\r\n"\
+						"o=jehan-mac 1239 1239 IN IP6 2a01:e35:1387:1020:6233:4bff:fe0b:5663\r\n"\
+						"s=SIP Talk\r\n"\
+						"c=IN IP4 192.168.0.18\r\n"\
+						"b=AS:380\r\n"\
+						"t=0 0\r\n"\
+						"a=ice-pwd:31ec21eb38b2ec6d36e8dc7b\r\n"\
+						"m=audio 7078 RTP/AVP 111 110 3 0 8 101\r\n"\
+						"a=rtcp-fb\r\n"\
+						"a=rtpmap:111 speex/16000\r\n";
+
+static void test_session_description_with_invalid_rtcp_fb(void) {
+	const char* l_src = sdp_with_invalid_rtcp_fb;
+	belle_sip_list_t* media_descriptions;
+	belle_sdp_media_description_t *media_description;
+	belle_sdp_session_description_t* l_session_description = belle_sdp_session_description_parse(l_src);
+	/* make sure that the invalid rtcp-fb is not parsed as a raw attribute. */
+	media_descriptions = belle_sdp_session_description_get_media_descriptions(l_session_description);
+	media_description = (belle_sdp_media_description_t*) media_descriptions->data;
+	BC_ASSERT_PTR_NULL(belle_sdp_media_description_get_attribute(media_description, "rtcp-fb"));
+	belle_sip_object_unref(l_session_description);
+}
+
+
 static void test_overflow(void){
 	belle_sdp_session_description_t* sdp;
 	belle_sip_list_t *mds;
@@ -606,8 +1049,9 @@ static void test_mime_parameter(void) {
 	belle_sip_list_t* mime_parameter_list;
 	belle_sip_list_t* mime_parameter_list_iterator;
 	belle_sdp_media_description_t* l_media_description_tmp = belle_sdp_media_description_parse(l_src);
-
-	belle_sdp_media_description_t* l_media_description = belle_sdp_media_description_parse(belle_sip_object_to_string(l_media_description_tmp));
+	char* sdp = belle_sip_object_to_string(l_media_description_tmp);
+	belle_sdp_media_description_t* l_media_description = belle_sdp_media_description_parse(sdp);
+	belle_sip_free(sdp);
 	belle_sip_object_unref(l_media_description_tmp);
 
 	mime_parameter_list = belle_sdp_media_description_build_mime_parameters(l_media_description);
@@ -628,9 +1072,10 @@ static void test_mime_parameter(void) {
 
 	/*marshal/unmarshal again*/
 	l_media_description_tmp = l_media_description;
-	l_media_description= belle_sdp_media_description_parse(belle_sip_object_to_string(l_media_description));
+	char* media_desc_string = belle_sip_object_to_string(l_media_description);
+	l_media_description= belle_sdp_media_description_parse(media_desc_string);
+	belle_sip_free(media_desc_string);
 	belle_sip_object_unref(l_media_description_tmp);
-	/*belle_sip_message("%s",belle_sip_object_to_string(l_media_description));*/
 	{
 		belle_sip_list_t* attributes=belle_sdp_media_description_get_attributes(l_media_description);
 #ifdef	BELLE_SDP_FORCE_RTP_MAP
@@ -681,12 +1126,21 @@ static void test_mime_parameter(void) {
 	belle_sip_list_free_with_data(mime_parameter_list, (void (*)(void*))belle_sip_object_unref);
 }
 
-
 test_t sdp_tests[] = {
+	TEST_NO_TAG("a=rtcp-fb", test_rtcp_fb_attribute),
 	TEST_NO_TAG("a= (attribute)", test_attribute),
 	TEST_NO_TAG("a= (attribute) 2", test_attribute_2),
-	TEST_NO_TAG("a=rtcp-fb", test_rtcp_fb_attribute),
 	TEST_NO_TAG("a=rtcp-xr", test_rtcp_xr_attribute),
+	TEST_NO_TAG("a= (label)", test_label_attribute),
+	TEST_NO_TAG("a= (content)", test_content_attribute),
+	TEST_NO_TAG("a= (csup)", test_csup_attribute),
+	TEST_NO_TAG("a= (creq)", test_creq_attribute),
+	TEST_NO_TAG("a= (tcap)", test_tcap_attribute),
+	TEST_NO_TAG("a= (acap)", test_acap_attribute),
+	TEST_NO_TAG("a= (simple acap)", test_simple_acap_attribute),
+	TEST_NO_TAG("a= (long acap)", test_long_acap_attribute),
+	TEST_NO_TAG("a= (acfg)", test_acfg_attribute),
+	TEST_NO_TAG("a= (pcfg)", test_pcfg_attribute),
 	TEST_NO_TAG("b= (bandwidth)", test_bandwidth),
 	TEST_NO_TAG("o= (IPv4 origin)", test_origin),
 	TEST_NO_TAG("o= (malformed origin)", test_malformed_origin),
@@ -699,9 +1153,11 @@ test_t sdp_tests[] = {
 	TEST_NO_TAG("mime parameter", test_mime_parameter),
 	TEST_NO_TAG("Media description", test_media_description),
 	TEST_NO_TAG("Simple session description", test_simple_session_description),
+	TEST_NO_TAG("Session description with capability reference before definition", test_session_description_with_capability_referenced_before_definition),
 	TEST_NO_TAG("Session description", test_session_description),
 	TEST_NO_TAG("Session description for fax", test_image_mline),
-	TEST_NO_TAG("Marshal buffer overflow", test_overflow)
+	TEST_NO_TAG("Marshal buffer overflow", test_overflow),
+	TEST_NO_TAG("Invalid specialized attribute not parsed as raw attribute", test_session_description_with_invalid_rtcp_fb)
 };
 
 test_suite_t sdp_test_suite = {"SDP", NULL, NULL, belle_sip_tester_before_each, belle_sip_tester_after_each,
