@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle-sip/belle-sip.h"
 #include "belle_sip_tester.h"
@@ -77,7 +78,22 @@ static void testIPV6URI(void) {
 	testIPV6URI_base("fe80::1");
 	testIPV6URI_base("2a01:e35:1387:1020:6233:4bff:fe0b:5663");
 	testIPV6URI_base("2a01:e35:1387:1020:6233::5663");
+// Examples from RFC4291 (Keep for testing one digit '0' in address)
+	testIPV6URI_base("2001:DB8:0:0:8:800:200C:417A");//  a unicast address
+	testIPV6URI_base("FF01:0:0:0:0:0:0:101");//  a multicast address
+	testIPV6URI_base("0:0:0:0:0:0:0:1");	// the loopback address
+	testIPV6URI_base("0:0:0:0:0:0:0:0");	// the unspecified address
+// Alternative representations
+	testIPV6URI_base("2001:DB8::8:800:200C:417A");
+	testIPV6URI_base("FF01::101");
 	testIPV6URI_base("::1");
+	testIPV6URI_base("::");
+	testIPV6URI_base("0:0:0:0:0:0:13.1.68.3");
+	testIPV6URI_base("0:0:0:0:0:FFFF:129.144.52.38");
+// Compressed form with IPV4
+	testIPV6URI_base("::13.1.68.3");
+	testIPV6URI_base("::FFFF:129.144.52.38");
+	testIPV6URI_base("2a01:e35:1387:1020::192.168.1.1");
 }
 
 static void testSIPSURI(void) {
@@ -351,7 +367,7 @@ static void test_uri_equals(void) {
 	BC_ASSERT_FALSE(belle_sip_uri_equals(a,b));
 	belle_sip_object_unref(a);
 	belle_sip_object_unref(b);
-	
+
 	a = belle_sip_uri_parse("sip:bob@biloxi.com");
 	if (!BC_ASSERT_PTR_NOT_NULL(a)) return;
 	b = belle_sip_uri_parse("sip:boba@biloxi.com");
@@ -439,12 +455,31 @@ static void test_escaping_bad_chars(void){
 	char bad_uri[13] = { 'h', 'e', 'l', 'l', 'o', (char)0xa0, (char)0xc8, 'w', 'o', 'r', 'l', 'd', 0x0 };
 	char *escaped = belle_sip_uri_to_escaped_username(bad_uri);
 	const char *expected="hello%a0%c8world";
-
 	BC_ASSERT_STRING_EQUAL(escaped, expected);
+	belle_sip_free(escaped);
 
+	char *bad_uri2 = "François";
+	escaped = belle_sip_uri_to_escaped_username(bad_uri2);
+	const char *expected2="Fran%c3%a7ois";
+	BC_ASSERT_STRING_EQUAL(escaped, expected2);
 	belle_sip_free(escaped);
 }
 
+static void test_unescaping_good_chars(void){
+	char phone_number[13] = { (char)0x2b, '3', '3', '6', '4', '5', '4', '9', '9', '9', '4', '3', 0x0 };
+	char *unescaped = belle_sip_username_unescape_unnecessary_characters(phone_number);
+	// '+' can be unescaped
+	const char *expected="+33645499943";
+	BC_ASSERT_STRING_EQUAL(unescaped, expected);
+	belle_sip_free(unescaped);
+
+	char username[10] = { 'F', 'r', 'a', 'n', (char)0xc3, (char)0xa7, 'o', 'i', 's', 0x0 };
+	unescaped = belle_sip_username_unescape_unnecessary_characters(username);
+	// 'ç' needs to be kept escaped
+	const char *expected2="Fran%c3%a7ois";
+	BC_ASSERT_STRING_EQUAL(unescaped, expected2);
+	belle_sip_free(unescaped);
+}
 
 static belle_sip_header_address_t* test_header_address_parsing(const char* address, int expect_fail){
 	belle_sip_header_address_t* header_address = belle_sip_header_address_parse(address);
@@ -484,6 +519,7 @@ static test_t uri_tests[] = {
 	TEST_NO_TAG("Complex URI", testCOMPLEXURI),
 	TEST_NO_TAG("Escaped username", test_escaped_username),
 	TEST_NO_TAG("Escaped username with bad chars", test_escaping_bad_chars),
+	TEST_NO_TAG("Unescape username", test_unescaping_good_chars),
 	TEST_NO_TAG("Escaped parameter", test_escaped_parameter),
 	TEST_NO_TAG("Escaped passwd", test_escaped_passwd),
 	TEST_NO_TAG("User passwd", test_user_passwd),
@@ -500,4 +536,3 @@ static test_t uri_tests[] = {
 	TEST_NO_TAG("URI components", testUriComponentsChecker),
 	TEST_NO_TAG("Empty password", test_empty_password),
 };
-
