@@ -1,20 +1,21 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 
 #include "belle-sip/belle-sip.h"
@@ -120,8 +121,39 @@ static void test_from_header(void) {
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_from_get_tag(L_from),"dlfjklcn6545614XX");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_from));
 
+	// Test Domain name RFC1123-2.1
+	L_from = belle_sip_header_from_parse("f:<sip:123jeff;transport=tcp>;tag=dlfjklcn6545614XX");
+	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_from));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_from));
+	L_tmp = belle_sip_header_from_parse(l_raw_header);
+	L_from = BELLE_SIP_HEADER_FROM(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	belle_sip_free(l_raw_header);
 
-	/*test factory*/
+	L_uri = belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_from));
+
+	BC_ASSERT_PTR_NULL(belle_sip_uri_get_user(L_uri));
+	BC_ASSERT_STRING_EQUAL(belle_sip_uri_get_host(L_uri), "123jeff");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_from_get_tag(L_from),"dlfjklcn6545614XX");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_from));
+
+	L_from = belle_sip_header_from_parse("f:<sip:12345678987;transport=tcp>;tag=dlfjklcn6545614XX");
+	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_from));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_from));
+	L_tmp = belle_sip_header_from_parse(l_raw_header);
+	L_from = BELLE_SIP_HEADER_FROM(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	belle_sip_free(l_raw_header);
+
+	L_uri = belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_from));
+
+	BC_ASSERT_PTR_NULL(belle_sip_uri_get_user(L_uri));
+	BC_ASSERT_STRING_EQUAL(belle_sip_uri_get_host(L_uri), "12345678987");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_from_get_tag(L_from),"dlfjklcn6545614XX");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_from));
+
+
+	// Test factory
 	L_from = belle_sip_header_from_create2("super <sip:titi.com:5060;lr;ttl=1;method=INVITE;maddr=192.168.0.1;transport=tcp?header=123>","12345-abc");
 	if (!BC_ASSERT_PTR_NOT_NULL(L_from)) return;
 	L_uri = belle_sip_header_address_get_uri(BELLE_SIP_HEADER_ADDRESS(L_from));
@@ -156,7 +188,9 @@ static void test_from_header_with_paramless_address_spec(void) {
 	BC_ASSERT_STRING_EQUAL(belle_generic_uri_get_scheme(L_absoluteUri), "tel");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_from_get_tag(L_from),"dlfjklcn6545614XX");
 	belle_sip_object_unref(L_from);
-
+	BC_ASSERT_PTR_NULL(belle_sip_header_from_parse("F: x:xx%"));
+	BC_ASSERT_PTR_NULL(belle_sip_header_from_parse("F\"\\"));
+	
 }
 
 static void test_to_header_with_paramless_address_spec(void) {
@@ -265,6 +299,19 @@ static void test_via_header(void) {
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_via_get_branch(L_via),"z9hG4bK368560724");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_via));
 
+	// Test IPv6 with delimiter received in via. See: https://tools.ietf.org/html/rfc5118#section-4.5
+	L_via = belle_sip_header_via_parse("v: SIP/2.0/UDP [::1]:5062;rport;received=[::1];branch=z9hG4bK368560724");
+
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_via_get_protocol(L_via), "SIP/2.0");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_via_get_transport(L_via), "UDP");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_via_get_host(L_via), "::1");
+	BC_ASSERT_EQUAL(belle_sip_header_via_get_port(L_via),5062,int,"%d");
+
+	BC_ASSERT_TRUE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(L_via),"rport"));
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_via_get_received(L_via),"::1");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_via_get_branch(L_via),"z9hG4bK368560724");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_via));
+
 	L_via = belle_sip_header_via_parse("Via: SIP/2.0/UDP 192.168.0.19:5062;received=192.169.0.4;rport=1234;branch=z9hG4bK368560724, SIP/2.0/UDP 192.168.0.19:5062");
 
 	l_next = belle_sip_header_get_next(BELLE_SIP_HEADER(L_via));
@@ -452,6 +499,23 @@ static void test_route_header(void) {
 
 }
 
+static void test_rseq_header(void) {
+	belle_sip_header_t* L_tmp;
+	belle_sip_header_t* L_rseq;
+	char* l_raw_header = NULL;
+
+	L_rseq = belle_sip_header_parse("RSeq: 3344");
+	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_rseq));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_rseq));
+	L_tmp = belle_sip_header_parse(l_raw_header);
+	L_rseq = BELLE_SIP_HEADER(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	belle_sip_free(l_raw_header);
+
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_get_unparsed_value(L_rseq), "3344");
+	belle_sip_object_unref(L_rseq);
+}
+
 static void test_service_route_header(void) {
 	belle_sip_header_service_route_t* L_service_route = belle_sip_header_service_route_parse("Service-Route: <sip:orig@scscf.ims.linphone.com:6060;lr>");
 	belle_sip_uri_t* L_uri;
@@ -482,6 +546,20 @@ static void test_content_length_header(void) {
 	BC_ASSERT_PTR_NULL(belle_sip_header_content_length_parse("nimportequoi"));
 }
 
+static void test_retry_after_header(void) {
+	belle_sip_header_retry_after_t* L_tmp;
+	belle_sip_header_retry_after_t* L_retry_after = belle_sip_header_retry_after_parse("Retry-After: 120");
+	char* l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_retry_after));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_retry_after));
+	L_tmp = belle_sip_header_retry_after_parse(l_raw_header);
+	L_retry_after = BELLE_SIP_HEADER_RETRY_AFTER(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	belle_sip_free(l_raw_header);
+	BC_ASSERT_EQUAL((unsigned int)belle_sip_header_retry_after_get_retry_after(L_retry_after), 120, int, "%i");
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_retry_after));
+	BC_ASSERT_PTR_NULL(belle_sip_header_retry_after_parse("nimportequoi"));
+}
+
 static belle_sip_header_t* test_header_extension(const char* name,const char* value) {
 	belle_sip_header_t* L_tmp;
 	belle_sip_header_t* L_extension;
@@ -560,6 +638,23 @@ static void test_proxy_authorization_header(void) {
 	BC_ASSERT_PTR_NULL(belle_sip_header_proxy_authorization_parse("nimportequoi"));
 }
 
+static void test_rack_header(void ) {
+	belle_sip_header_t* L_tmp;
+	belle_sip_header_t* L_rseq;
+	char* l_raw_header = NULL;
+
+	L_rseq = belle_sip_header_parse("RAck: 3344 101 INVITE");
+	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_rseq));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_rseq));
+	L_tmp = belle_sip_header_parse(l_raw_header);
+	L_rseq = BELLE_SIP_HEADER(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+	belle_sip_free(l_raw_header);
+
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_get_unparsed_value(L_rseq), "3344 101 INVITE");
+	belle_sip_object_unref(L_rseq);
+}
+
 static void check_header_authenticate(belle_sip_header_www_authenticate_t* authenticate) {
 	belle_sip_list_t* qop;
 	BC_ASSERT_PTR_NOT_NULL(authenticate);
@@ -718,7 +813,7 @@ static void test_expires_header(void) {
 
 static void test_allow_header(void) {
 	belle_sip_header_allow_t* L_tmp;
-	belle_sip_header_allow_t* L_allow = belle_sip_header_allow_parse("Allow:INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO");
+	belle_sip_header_allow_t* L_allow = belle_sip_header_allow_parse("Allow:INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO, PRACK");
 	char* l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_allow));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_allow));
 	L_tmp = belle_sip_header_allow_parse(l_raw_header);
@@ -726,7 +821,7 @@ static void test_allow_header(void) {
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
 	belle_sip_free(l_raw_header);
 
-	BC_ASSERT_STRING_EQUAL(belle_sip_header_allow_get_method(L_allow), "INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO");
+	BC_ASSERT_STRING_EQUAL(belle_sip_header_allow_get_method(L_allow), "INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO, PRACK");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_allow));
 	BC_ASSERT_PTR_NULL(belle_sip_header_allow_parse("nimportequoi"));
 }
@@ -741,11 +836,14 @@ static void test_address_with_error_header(void) {
 static void test_address_header(void) {
 	belle_sip_uri_t* L_uri;
 	char* L_raw;
+	belle_sip_header_address_t* lclonedaddr;
+
 	belle_sip_header_address_t* laddress = belle_sip_header_address_fast_parse("\"toto\" <sip:liblinphone_tester@81.56.11.2:5060>");
 	BC_ASSERT_PTR_NOT_NULL(laddress);
-	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(laddress));
+	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(lclonedaddr=belle_sip_header_address_clone(laddress)));
 	BC_ASSERT_PTR_NOT_NULL(L_raw);
 	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lclonedaddr));
 	laddress = belle_sip_header_address_parse(L_raw);
 	belle_sip_free(L_raw);
 
@@ -779,6 +877,32 @@ static void test_address_header(void) {
 	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
 
 
+}
+static void test_address_header_with_params(void) {
+	belle_sip_uri_t* L_uri;
+	char* L_raw;
+	belle_sip_header_address_t* lclonedaddr;
+
+	belle_sip_header_address_t* laddress = belle_sip_header_address_fast_parse("\"toto\" <sip:liblinphone_tester@81.56.11.2:5060> ; toto=\"titi\";tutu");
+	BC_ASSERT_PTR_NOT_NULL(laddress);
+	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(lclonedaddr=belle_sip_header_address_clone(laddress)));
+	BC_ASSERT_PTR_NOT_NULL(L_raw);
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lclonedaddr));
+	laddress = belle_sip_header_address_parse(L_raw);
+	belle_sip_free(L_raw);
+
+	BC_ASSERT_STRING_EQUAL("toto",belle_sip_header_address_get_displayname(laddress));
+	L_uri = belle_sip_header_address_get_uri(laddress);
+
+	BC_ASSERT_PTR_NOT_NULL(belle_sip_uri_get_user(L_uri));
+	BC_ASSERT_STRING_EQUAL(belle_sip_uri_get_host(L_uri), "81.56.11.2");
+	BC_ASSERT_STRING_EQUAL(belle_sip_uri_get_user(L_uri), "liblinphone_tester");
+	BC_ASSERT_EQUAL(belle_sip_uri_get_port(L_uri), 5060,int,"%d");
+	BC_ASSERT_STRING_EQUAL(belle_sip_parameters_get_parameter(BELLE_SIP_PARAMETERS(laddress), "toto"),"\"titi\"");
+	BC_ASSERT_TRUE(belle_sip_parameters_has_parameter(BELLE_SIP_PARAMETERS(laddress), "toto"));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
+
 }
 
 static void test_address_header_with_tel_uri(void) {
@@ -800,6 +924,34 @@ static void test_address_header_with_tel_uri(void) {
 	belle_sip_object_unref(BELLE_SIP_OBJECT(laddress));
 }
 
+static void test_address_header_with_session_expires(void) {
+	char* L_raw;
+
+	belle_sip_header_session_expires_t* L_session_expires_tmp = belle_sip_header_session_expires_parse("Session-Expires: 4000;refresher=uac");
+	belle_sip_header_session_expires_t* L_session_expires = BELLE_SIP_HEADER_SESSION_EXPIRES(belle_sip_object_clone(BELLE_SIP_OBJECT(L_session_expires_tmp)));
+
+	BC_ASSERT_EQUAL(belle_sip_header_session_expires_get_delta(L_session_expires), 4000, int, "%d");
+	BC_ASSERT_EQUAL(belle_sip_header_session_expires_get_refresher_value(L_session_expires), BELLE_SIP_HEADER_SESSION_EXPIRES_UAC, int,"%d");
+
+	L_raw = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_session_expires));
+	BC_ASSERT_PTR_NOT_NULL(L_raw);
+
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_session_expires));
+	L_session_expires = belle_sip_header_session_expires_parse(L_raw);
+
+	BC_ASSERT_EQUAL(belle_sip_header_session_expires_get_delta(L_session_expires), 4000, int, "%d");
+	belle_sip_header_session_expires_set_delta(L_session_expires, 123);
+	BC_ASSERT_EQUAL(belle_sip_header_session_expires_get_delta(L_session_expires), 123, int, "%d");
+
+	BC_ASSERT_EQUAL(belle_sip_header_session_expires_get_refresher_value(L_session_expires), BELLE_SIP_HEADER_SESSION_EXPIRES_UAC, int,"%d");
+	belle_sip_header_session_expires_set_refresher_value(L_session_expires, BELLE_SIP_HEADER_SESSION_EXPIRES_UAS);
+	BC_ASSERT_EQUAL(belle_sip_header_session_expires_get_refresher_value(L_session_expires), BELLE_SIP_HEADER_SESSION_EXPIRES_UAS, int, "%d");
+
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_session_expires));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_session_expires_tmp));
+	belle_sip_free(L_raw);
+}
+
 static void test_address_header_with_urn(void) {
 	belle_generic_uri_t* L_uri;
 	char* L_raw;
@@ -954,6 +1106,41 @@ static void test_replaces_escaped_header(void) {
 
 }
 
+static void test_require(const char* header_name, const char * header_value, const char* values[],size_t number_values){
+	belle_sip_list_t* list;
+	belle_sip_header_require_t* L_tmp;
+	belle_sip_header_require_t* L_require = BELLE_SIP_HEADER_REQUIRE(belle_sip_header_create(header_name,header_value));
+	char* l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_require));
+	size_t i=0;
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_require));
+	L_tmp = belle_sip_header_require_parse(l_raw_header);
+	L_require = BELLE_SIP_HEADER_REQUIRE(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
+
+	belle_sip_free(l_raw_header);
+
+	list = belle_sip_header_require_get_require(L_require);
+
+	for(i=0;i<number_values;i++){
+		BC_ASSERT_PTR_NOT_NULL(list);
+		if (list) {
+			BC_ASSERT_STRING_EQUAL((const char *)(list->data),values[i]);
+			list=list->next;
+		}
+	}
+	belle_sip_object_unref(BELLE_SIP_OBJECT(L_require));
+	BC_ASSERT_PTR_NULL(belle_sip_header_require_parse("nimportequoi"));
+}
+
+static void test_header_require(void) {
+	const char* value1[] ={"user","critical"};
+	const char* value2[] ={"id"};
+	const char* value3[] ={"100rel"};
+	test_require("Require","user, critical",value1,2);
+	test_require("Require", "id",value2,1);
+	test_require("Require", "100rel", value3, 1);
+}
+
 #ifndef _WIN32
 static void _test_date_header(void){
 	belle_sip_header_date_t *date,*date2;
@@ -1065,6 +1252,15 @@ static void test_event_header(void) {
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_event_get_package_name(L_event), "presence");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_event_get_id(L_event), "blabla1");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_event));
+
+	L_event = belle_sip_header_event_parse("o: presence;id=blabla1");
+	BC_ASSERT_PTR_NOT_NULL(L_event);
+	if (L_event) {
+		BC_ASSERT_STRING_EQUAL(belle_sip_header_event_get_package_name(L_event), "presence");
+		BC_ASSERT_STRING_EQUAL(belle_sip_header_event_get_id(L_event), "blabla1");
+		belle_sip_object_unref(BELLE_SIP_OBJECT(L_event));
+	}
+
 	BC_ASSERT_PTR_NULL(belle_sip_header_event_parse("nimportequoi"));
 }
 
@@ -1093,11 +1289,14 @@ static void test_supported(const char* header_name, const char * header_value, c
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_supported));
 	BC_ASSERT_PTR_NULL(belle_sip_header_supported_parse("nimportequoi"));
 }
+
 static void test_supported_header(void) {
 	const char* value1[] ={"user","critical"};
 	const char* value2[] ={"id"};
+    const char* value3[] ={"100rel"};
 	test_supported("Supported","user, critical",value1,2);
 	test_supported("Supported", "id",value2,1);
+	test_supported("Supported", "100rel", value3, 1);
 }
 
 static void test_content_disposition_header(void) {
@@ -1167,16 +1366,16 @@ static void test_reason_header(void) {
 	belle_sip_header_reason_t *L_tmp, *l_next;
 	belle_sip_header_reason_t* L_reason = BELLE_SIP_HEADER_REASON(belle_sip_header_create("Reason", "Q.850 ;cause=16 ;text=\"Busy Everywhere\""));
 	char* l_raw_header;
-	
+
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_reason_get_text(L_reason),"Busy Everywhere");
 	belle_sip_header_reason_set_text(L_reason, "Terminated");
 	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_reason));
-	
+
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_reason));
 	L_tmp = belle_sip_header_reason_parse(l_raw_header);
 	L_reason = BELLE_SIP_HEADER_REASON(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
-	
+
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_reason_get_text(L_reason),"Terminated");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_reason_get_protocol(L_reason),"Q.850");
 	BC_ASSERT_EQUAL(belle_sip_header_reason_get_cause(L_reason),16 , int,"%d");
@@ -1189,7 +1388,7 @@ static void test_reason_header(void) {
 	BC_ASSERT_EQUAL(belle_sip_header_reason_get_cause(l_next),580 , int,"%d");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_reason));
 
-	
+
 	BC_ASSERT_PTR_NULL(belle_sip_header_reason_parse("nimportequoi"));
 }
 static void test_authentication_info_header(void) {
@@ -1198,43 +1397,46 @@ static void test_authentication_info_header(void) {
 																													, "nextnonce=\"47364c23432d2e131a5fb210812c\""
 																														", qop=auth"));
 	char* l_raw_header;
-	
+
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_authentication_info_get_next_nonce(L_authentication_info),"47364c23432d2e131a5fb210812c");
 	belle_sip_header_authentication_info_set_next_nonce(L_authentication_info, "31a5fb210812c47364c23432d2e1");
 	l_raw_header = belle_sip_object_to_string(BELLE_SIP_OBJECT(L_authentication_info));
-	
+
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_authentication_info));
 	L_tmp = belle_sip_header_authentication_info_parse(l_raw_header);
 	L_authentication_info = BELLE_SIP_HEADER_AUTHENTICATION_INFO(belle_sip_object_clone(BELLE_SIP_OBJECT(L_tmp)));
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_tmp));
-	
+
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_authentication_info_get_next_nonce(L_authentication_info),"31a5fb210812c47364c23432d2e1");
 	BC_ASSERT_STRING_EQUAL(belle_sip_header_authentication_info_get_qop(L_authentication_info),"auth");
-	
+
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_authentication_info));
 	belle_sip_free(l_raw_header);
-	
-	
+
+
 	L_authentication_info = belle_sip_header_authentication_info_parse("Authentication-Info:nextnonce=\"WRSE5VkUg7kOjnP5sBZZzsxrsOQ8Eyp6ZNttXIA=\", qop=auth , rspauth=\"5e8518291c1320989df6966405e0b3d1\", cnonce=\"05bc00e63a7794182f9425da9266d00d\", nc=00000001");
 	BC_ASSERT_EQUAL(belle_sip_header_authentication_info_get_nonce_count(L_authentication_info),1, int, "%i");
 	belle_sip_object_unref(BELLE_SIP_OBJECT(L_authentication_info));
-	
+
 	BC_ASSERT_PTR_NULL(belle_sip_header_authentication_info_parse("nimportequoi"));
 }
 
 test_t headers_tests[] = {
 	TEST_NO_TAG("Address", test_address_header),
+	TEST_NO_TAG("Address with params",test_address_header_with_params),
 	TEST_NO_TAG("Address tel uri", test_address_header_with_tel_uri),
 	TEST_NO_TAG("Address urn", test_address_header_with_urn),
 	TEST_NO_TAG("Header address (very long)", test_very_long_address_header),
 	TEST_NO_TAG("Address with error", test_address_with_error_header),
 	TEST_NO_TAG("Allow", test_allow_header),
 	TEST_NO_TAG("Authorization", test_authorization_header),
+	TEST_NO_TAG("Session-Expires", test_address_header_with_session_expires),
 	TEST_NO_TAG("Call-ID", test_call_id_header),
 	TEST_NO_TAG("Contact (Simple)", test_simple_contact_header),
 	TEST_NO_TAG("Contact (Complex)", test_complex_contact_header),
 	TEST_NO_TAG("Contact (Param-less address spec)", test_contact_header_with_paramless_address_spec),
 	TEST_NO_TAG("Content-Length", test_content_length_header),
+	TEST_NO_TAG("Retry-After", test_retry_after_header),
 	TEST_NO_TAG("Content-Type", test_content_type_header),
 	TEST_NO_TAG("CSeq", test_cseq_header),
 	TEST_NO_TAG("Date", test_date_header),
@@ -1246,12 +1448,15 @@ test_t headers_tests[] = {
 	TEST_NO_TAG("P-Preferred-Identity", test_p_preferred_identity_header),
 	TEST_NO_TAG("Proxy-Authenticate", test_proxy_authenticate_header),
 	TEST_NO_TAG("Proxy-Authorization", test_proxy_authorization_header),
+	TEST_NO_TAG("RAck", test_rack_header),
 	TEST_NO_TAG("Record-Route", test_record_route_header),
 	TEST_NO_TAG("Refer-To", test_refer_to_header),
 	TEST_NO_TAG("Referred-By", test_referred_by_header),
 	TEST_NO_TAG("Replaces", test_replaces_header),
 	TEST_NO_TAG("Replaces (Escaped)", test_replaces_escaped_header),
+	TEST_NO_TAG("Require", test_header_require),
 	TEST_NO_TAG("Route", test_route_header),
+	TEST_NO_TAG("RSeq", test_rseq_header),
 	TEST_NO_TAG("Service-Route", test_service_route_header),
 	TEST_NO_TAG("Subscription-State", test_subscription_state_header),
 	TEST_NO_TAG("To", test_to_header),
@@ -1259,14 +1464,14 @@ test_t headers_tests[] = {
 	TEST_NO_TAG("User-Agent", test_user_agent_header),
 	TEST_NO_TAG("Via", test_via_header),
 	TEST_NO_TAG("WWW-Authenticate", test_www_authenticate_header),
-	TEST_NO_TAG("Header extension", test_header_extension_1),
-	TEST_NO_TAG("Header extension 2", test_header_extension_2),
-	TEST_NO_TAG("Header event", test_event_header),
-	TEST_NO_TAG("Header Supported", test_supported_header),
-	TEST_NO_TAG("Header Content-Disposition", test_content_disposition_header),
-	TEST_NO_TAG("Header Accept", test_accept_header),
-	TEST_NO_TAG("Header Reason", test_reason_header),
-	TEST_NO_TAG("Header Authentication-Info", test_authentication_info_header)
+	TEST_NO_TAG("Extension", test_header_extension_1),
+	TEST_NO_TAG("Extension 2", test_header_extension_2),
+	TEST_NO_TAG("Event", test_event_header),
+	TEST_NO_TAG("Supported", test_supported_header),
+	TEST_NO_TAG("Content-Disposition", test_content_disposition_header),
+	TEST_NO_TAG("Accept", test_accept_header),
+	TEST_NO_TAG("Reason", test_reason_header),
+	TEST_NO_TAG("Authentication-Info", test_authentication_info_header)
 };
 
 test_suite_t headers_test_suite = {"Headers", NULL, NULL, belle_sip_tester_before_each, belle_sip_tester_after_each,
