@@ -1,23 +1,25 @@
 /*
-	belle-sip - SIP (RFC3261) library.
-    Copyright (C) 2010  Belledonne Communications SARL
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ * Copyright (c) 2012-2019 Belledonne Communications SARL.
+ *
+ * This file is part of belle-sip.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "belle-sip/belle-sip.h"
 #include "belle_sip_tester.h"
+#include "belle_sip_internal.h"
 
 
 static void check_uri_and_headers(belle_sip_message_t* message) {
@@ -97,9 +99,10 @@ static void testInviteMessage(void) {
 							"From: \"Benjamin Cheong\" <sip:bcheong@sip.linphone.org>;tag=7326e5f6\r\n"\
 							"Call-ID: Y2NlNzg0ODc0ZGIxODU1MWI5MzhkNDVkNDZhOTQ4YWU.\r\n"\
 							"CSeq: 1 INVITE\r\n"\
-							"Allow: INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO\r\n"\
+							"Allow: INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO, PRACK\r\n"\
 							"c: application/sdp\r\n"\
 							"Supported: replaces\r\n"\
+							"Supported: 100rel\r\n"\
 							"Authorization: Digest username=\"003332176\", realm=\"sip.ovh.net\", nonce=\"24212965507cde726e8bc37e04686459\", uri=\"sip:sip.ovh.net\", response=\"896e786e9c0525ca3085322c7f1bce7b\", algorithm=MD5, opaque=\"241b9fb347752f2\"\r\n"\
 							"User-Agent: X-Lite 4 release 4.0 stamp 58832\r\n"\
 							"Content-Length: 230\r\n\r\n";
@@ -113,6 +116,24 @@ static void testInviteMessage(void) {
 	BC_ASSERT_PTR_NOT_NULL(belle_sip_message_get_header(message,"Contact"));
 	BC_ASSERT_PTR_NOT_NULL(belle_sip_message_get_header(message,"Authorization"));
 	BC_ASSERT_PTR_NOT_NULL(belle_sip_message_get_header(message,"Content-Type"));
+
+	belle_sip_header_supported_t* header_support;
+	BC_ASSERT_PTR_NOT_NULL(header_support=BELLE_SIP_HEADER_SUPPORTED(belle_sip_message_get_header(message,"Supported")));
+	if (header_support) {
+		belle_sip_list_t* list;
+		list = belle_sip_header_supported_get_supported(header_support);
+		BC_ASSERT_PTR_NOT_NULL(list);
+		const char* value[] = {"replaces", "100rel"};
+		int num_supported = 2;
+		int i;
+		for(i = 0; i < num_supported; i++){
+			if (list) {
+				BC_ASSERT_STRING_EQUAL((const char *)(list->data), value[i]);
+				list=list->next;
+			}
+		}
+	}
+
 	check_uri_and_headers(message);
 	belle_sip_object_unref(message);
 	belle_sip_free(encoded_message);
@@ -388,7 +409,7 @@ static void testMalformedMessageWithWrongStart(void) {
 	belle_sip_message_t* message = belle_sip_message_parse(raw_message);
 	BC_ASSERT_PTR_NULL(message);
 }
-#include "belle_sip_internal.h"
+
 
 void channel_parser_tester_recovery_from_error_base (const char* prelude,const char* raw_message) {
 
@@ -398,7 +419,7 @@ void channel_parser_tester_recovery_from_error_base (const char* prelude,const c
 																	, 45421
 																	, NULL
 																	, "127.0.0.1"
-																	, 45421);
+																	, 45421, TRUE);
 
 
 	belle_sip_request_t* request;
@@ -547,7 +568,7 @@ static void testMalformedFrom(void){
 
 	const char* raw_message = "INVITE sip:us2@172.1.1.1 SIP/2.0\r\n"
 			"Via: SIP/2.0/TCP " LISTENING_POINT_HOSTPORT ";branch=z9hG4bK-edx-U_1zoIkaq72GJPqpSmDpJQ-ouBelFuLODzf9oS5J9MeFUA;rport\r\n"
-			"From: c\x8e test <sip:00_12_34_56_78_90@us2>;tag=klsk+kwDc\r\n" /** 'c test' should be enclosed in double quotes */
+			"From: c\x8e test <sip:00_12_34_56_78_90@us2>;tag=klsk+kwDc\r\n" /** 'c? test' should be enclosed in double quotes */
 			"To: <sip:us2@172.1.1.1;transport=tcp>\r\n"
 			"Contact: <sip:00_12_34_56_78_90@172.2.2.2>\r\n"
 			"Call-ID: 2b6fb0320-1384-179494-426025-23b6b0-2e3303331@172.16.42.1\r\n"
@@ -581,6 +602,7 @@ static void testMalformedFrom(void){
 
 	int called_times = 0;
 
+	belle_sip_provider_enable_response_integrity_checking(provider, FALSE);
 	listener_cbs.process_response_event = testMalformedFrom_process_response_cb;
 	listener = belle_sip_listener_create_from_callbacks(&listener_cbs, &called_times);
 	belle_sip_provider_add_sip_listener(provider, listener);
@@ -601,6 +623,185 @@ static void testMalformedFrom(void){
 
 }
 
+static void testMalformedFrom2(void){
+	belle_sip_stack_t*        stack = belle_sip_stack_new(NULL);
+	belle_sip_listening_point_t* lp = belle_sip_stack_create_listening_point(stack,
+																			 "127.0.0.1",
+																			 LISTENING_POINT_PORT,
+																			 "tcp");
+	belle_sip_provider_t* provider = belle_sip_provider_new(stack,lp);
+	belle_sip_listener_callbacks_t listener_cbs = {0};
+
+	const char* raw_message = "INVITE sip:us2@172.1.1.1 SIP/2.0\r\n"
+			"Via: SIP/2.0/TCP " LISTENING_POINT_HOSTPORT ";branch=z9hG4bK-edx-U_1zoIkaq72GJPqpSmDpJQ-ouBelFuLODzf9oS5J9MeFUA;rport\r\n"
+			"From: RottenFrom <sip:@somehost>;tag=klsk+kwDc\r\n" /** The userpart is empty before '@', this is not allowed */
+			"To: <sip:us2@172.1.1.1;transport=tcp>\r\n"
+			"Contact: <sip:00_12_34_56_78_90@172.2.2.2>\r\n"
+			"Call-ID: 2b6fb0320-1384-179494-426025-23b6b0-2e3303331@172.16.42.1\r\n"
+			"Content-Type: application/sdp\r\n"
+			"Content-Length: 389\r\n"
+			"CSeq: 1 INVITE\r\n"
+			"Allow: INVITE, ACK, BYE, CANCEL, OPTIONS, INFO, UPDATE, REGISTER, MESSAGE, REFER, SUBSCRIBE, PRACK\r\n"
+			"Accept: application/sdp, application/dtmf-relay\r\n"
+			"Max-Forwards: 69\r\n"
+			"\r\n"
+			"v=0\r\n"
+			"o=- 1826 1826 IN IP4 172.16.42.1\r\n"
+			"s=Pouetpouet\r\n"
+			"c=IN IP4 172.16.42.1\r\n"
+			"t=0 0\r\n"
+			"m=audio 20506 RTP/AVP 0 8 13 101\r\n"
+			"a=rtpmap:0 PCMU/8000\r\n"
+			"a=rtpmap:8 PCMA/8000\r\n"
+			"a=rtpmap:13 CN/8000\r\n"
+			"a=rtpmap:101 telephone-event/8000\r\n"
+			"a=fmtp:101 0-15\r\n"
+			"m=video 24194 RTP/AVP 105 104\r\n"
+			"a=sendonly\r\n"
+			"a=rtpmap:105 H264/90000\r\n"
+			"a=fmtp:105 packetization-mode=0\r\n"
+			"a=rtpmap:104 H263-1998/90000\r\n"
+			"a=fmtp:104 CIF=1;J=1\r\n";
+
+	belle_sip_message_t* message = belle_sip_message_parse(raw_message);
+	belle_sip_listener_t* listener = NULL;
+
+	int called_times = 0;
+
+	belle_sip_provider_enable_response_integrity_checking(provider, FALSE);
+	listener_cbs.process_response_event = testMalformedFrom_process_response_cb;
+	listener = belle_sip_listener_create_from_callbacks(&listener_cbs, &called_times);
+	belle_sip_provider_add_sip_listener(provider, listener);
+
+	belle_sip_object_ref(message);
+	belle_sip_object_ref(message); /* double ref: originally the message is created with 0 refcount, and dispatch_message will unref() it.*/
+	belle_sip_provider_dispatch_message(provider, message);
+	// we expect the stack to send a 400 error
+	belle_sip_stack_sleep(stack,1000);
+
+	BC_ASSERT_EQUAL(called_times,1,int,"%d");
+	belle_sip_provider_remove_sip_listener(provider,listener);
+
+	belle_sip_object_unref(listener);
+	belle_sip_object_unref(provider);
+	belle_sip_object_unref(stack);
+	belle_sip_object_unref(message);
+}
+
+
+static void testMalformedFrom3(void) {
+	belle_sip_stack_t*        stack = belle_sip_stack_new(NULL);
+	belle_sip_listening_point_t* lp = belle_sip_stack_create_listening_point(stack,
+							"127.0.0.1",
+							LISTENING_POINT_PORT,
+							"tcp");
+	belle_sip_provider_t* provider = belle_sip_provider_new(stack,lp);
+	belle_sip_listener_callbacks_t listener_cbs = {0};
+
+	const char* raw_message = "INVITE sip:us2@172.1.1.1 SIP/2.0\r\n"
+			"Via: SIP/2.0/TCP " LISTENING_POINT_HOSTPORT ";branch=z9hG4bK-edx-U_1zoIkaq72GJPqpSmDpJQ-ouBelFuLODzf9oS5J9MeFUA;rport\r\n"
+			"From: </coucou>\r\n" 
+			"To: <sip:us2@172.1.1.1;transport=tcp>\r\n"
+			"Contact: <sip:00_12_34_56_78_90@172.2.2.2>\r\n"
+			"Call-ID: 2b6fb0320-1384-179494-426025-23b6b0-2e3303331@172.16.42.1\r\n"
+			"Content-Type: application/sdp\r\n"
+			"Content-Length: 389\r\n"
+			"CSeq: 1 INVITE\r\n"
+			"Allow: INVITE, ACK, BYE, CANCEL, OPTIONS, INFO, UPDATE, REGISTER, MESSAGE, REFER, SUBSCRIBE, PRACK\r\n"
+			"Accept: application/sdp, application/dtmf-relay\r\n"
+			"Max-Forwards: 69\r\n"
+			"\r\n"
+			"v=0\r\n"
+			"o=- 1826 1826 IN IP4 172.16.42.1\r\n"
+			"s=Pouetpouet\r\n"
+			"c=IN IP4 172.16.42.1\r\n"
+			"t=0 0\r\n"
+			"m=audio 20506 RTP/AVP 0 8 13 101\r\n"
+			"a=rtpmap:0 PCMU/8000\r\n"
+			"a=rtpmap:8 PCMA/8000\r\n"
+			"a=rtpmap:13 CN/8000\r\n"
+			"a=rtpmap:101 telephone-event/8000\r\n"
+			"a=fmtp:101 0-15\r\n"
+			"m=video 24194 RTP/AVP 105 104\r\n"
+			"a=sendonly\r\n"
+			"a=rtpmap:105 H264/90000\r\n"
+			"a=fmtp:105 packetization-mode=0\r\n"
+			"a=rtpmap:104 H263-1998/90000\r\n"
+			"a=fmtp:104 CIF=1;J=1\r\n";
+
+	belle_sip_message_t* message = belle_sip_message_parse(raw_message);
+	belle_sip_listener_t* listener = NULL;
+
+	int called_times = 0;
+	
+	belle_sip_provider_enable_response_integrity_checking(provider, FALSE);
+	listener_cbs.process_response_event = testMalformedFrom_process_response_cb;
+	listener = belle_sip_listener_create_from_callbacks(&listener_cbs, &called_times);
+	belle_sip_provider_add_sip_listener(provider, listener);
+
+	belle_sip_object_ref(message);
+	belle_sip_object_ref(message); /* double ref: originally the message is created with 0 refcount, and dispatch_message will unref() it.*/
+	belle_sip_provider_dispatch_message(provider, message);
+	// we expect the stack to send a 400 error
+	belle_sip_stack_sleep(stack,1000);
+
+	BC_ASSERT_EQUAL(called_times,1,int,"%d");
+	belle_sip_provider_remove_sip_listener(provider,listener);
+
+	belle_sip_object_unref(listener);
+	belle_sip_object_unref(provider);
+	belle_sip_object_unref(stack);
+	belle_sip_object_unref(message);
+}
+
+
+static void testFromWithoutTag(void) {
+	belle_sip_stack_t*        stack = belle_sip_stack_new(NULL);
+	belle_sip_listening_point_t* lp = belle_sip_stack_create_listening_point(stack,
+							"127.0.0.1",
+							LISTENING_POINT_PORT,
+							"tcp");
+	belle_sip_provider_t* provider = belle_sip_provider_new(stack,lp);
+	belle_sip_listener_callbacks_t listener_cbs = {0};
+
+	const char* raw_message = 	"INVITE sip:100@192.168.122.168 SIP/2.0\r\n"
+					"Via: SIP/2.0/TCP " LISTENING_POINT_HOSTPORT ";branch=z9hG4bK7pqwjtzswe\r\n"
+					"Max-Forwards: 70\r\n"
+					"To: <sip:100@192.168.122.168>\r\n"
+					"From: <sip:200@192.168.122.1>\r\n"
+					"Contact: <sip:200@192.168.56.1:40709;transport=TCP>\r\n"
+					"Call-ID: 38tn4shategtc3i9gmspx38ds94qb2st\r\n"
+					"CSeq: 1 INVITE\r\n"
+					"User-Agent: WireBug\r\n"
+					"Expires: 600\r\n"
+					"Content-Length: 0\r\n\r\n";
+	
+	belle_sip_message_t* message = belle_sip_message_parse(raw_message);
+	belle_sip_listener_t* listener = NULL;
+
+	int called_times = 0;
+	
+	belle_sip_provider_enable_response_integrity_checking(provider, FALSE);
+
+	listener_cbs.process_response_event = testMalformedFrom_process_response_cb;
+	listener = belle_sip_listener_create_from_callbacks(&listener_cbs, &called_times);
+	belle_sip_provider_add_sip_listener(provider, listener);
+
+	belle_sip_object_ref(message);
+	belle_sip_object_ref(message); /* double ref: originally the message is created with 0 refcount, and dispatch_message will unref() it.*/
+	belle_sip_provider_dispatch_message(provider, message);
+	// we expect the stack to send a 400 error
+	belle_sip_stack_sleep(stack,1000);
+
+	BC_ASSERT_EQUAL(called_times,1,int,"%d");
+	belle_sip_provider_remove_sip_listener(provider,listener);
+
+	belle_sip_object_unref(listener);
+	belle_sip_object_unref(provider);
+	belle_sip_object_unref(stack);
+	belle_sip_object_unref(message);
+}
+
 
 static void testMalformedMandatoryField(void){
 	belle_sip_stack_t*        stack = belle_sip_stack_new(NULL);
@@ -914,7 +1115,7 @@ void channel_parser_http_response(void) {
 																	, 45421
 																	, NULL
 																	, "127.0.0.1"
-																	, 45421);
+																	, 45421, TRUE);
 
 	const char * raw_message=	"HTTP/1.1 200 OK\r\n"
 								"Cache-Control: private\r\n"
@@ -986,7 +1187,7 @@ void testGetBody(void) {
 																	, LISTENING_POINT_PORT
 																	, NULL
 																	, "127.0.0.1"
-																	, LISTENING_POINT_PORT);
+																	, LISTENING_POINT_PORT, TRUE);
 
 
 	belle_sip_message_t* message;
@@ -1031,9 +1232,12 @@ test_t message_tests[] = {
 	TEST_NO_TAG("SIP frag", test_sipfrag),
 	TEST_NO_TAG("Malformed invite", testMalformedMessage),
 	TEST_NO_TAG("Malformed from", testMalformedFrom),
+	TEST_NO_TAG("Malformed from 2", testMalformedFrom2),
+	TEST_NO_TAG("From without tag", testFromWithoutTag),
 	TEST_NO_TAG("Malformed mandatory field", testMalformedMandatoryField),
 	TEST_NO_TAG("Malformed invite with bad begin", testMalformedMessageWithWrongStart),
 	TEST_NO_TAG("Malformed register", testMalformedOptionnalHeaderInMessage),
+	TEST_NO_TAG("Malformed from 3", testMalformedFrom3),
 	TEST_NO_TAG("Channel parser error recovery", channel_parser_tester_recovery_from_error),
 	TEST_NO_TAG("Channel parser malformed start", channel_parser_malformed_start),
 	TEST_NO_TAG("Channel parser truncated start", channel_parser_truncated_start),
